// FILE: ./assets/modules/produto/templates/controllers/SafraController.js
var SafraController = function($scope, templateManager, $element) {

	var currentSelectedElement;

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (rgb) {
	    return "#" + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1);
	};

	this.initColors = function(field) {
		var currentRow = $scope.field.widget.currentRow;
		var safra = currentRow[field.colorsFlag.OPTION1];
		var initFin = currentRow[field.colorsFlag.OPTION2];
		var entre = currentRow[field.colorsFlag.OPTION3];
		
		var htmlParentContainer = $element.get(0);
		var flagElements = htmlParentContainer.querySelectorAll('.flag');

		safra = safra? this.TColorToHex(safra) : 'transparent';
		initFin = initFin? this.TColorToHex(initFin) : 'transparent';
		entre = entre? this.TColorToHex(entre) : 'transparent';

		flagElements[0].style.backgroundColor = safra;
		flagElements[1].style.backgroundColor = initFin;
		flagElements[2].style.backgroundColor = entre;

		var colorMonths = currentRow[field.propertieToChange];
		var months = htmlParentContainer.querySelectorAll('.month');
		this.initColorsMonth(months, [safra, initFin, entre], colorMonths);
	};

	this.initColorsMonth = function(months, colorFlags, colorMonths) {
		var color;
		if (!colorMonths) {
			months.forEach(function(element, key) {
				element.children[1].style.backgroundColor = 'transparent';
			});
		} else {
			colorMonths = colorMonths.split('');
			months.forEach(function(element) {
				var month = $(element).attr('month');
				var monthCorlorElement = element.children[1];
				
				var codeFlag = colorMonths[month -1];
				var colorMonth =  codeFlag != ' ' ? colorFlags[codeFlag -1] : 'transparent';
	
				monthCorlorElement.style.backgroundColor = colorMonth;
			});
		}	
	};

	this.getOptLabel = function(field, opt) {
		if (opt == '1') {	
			return field.option1 === "" || field.option1 ? field.option1 : "option1 not found";
		} else if (opt == '2') {
			return field.option2 === "" || field.option2 ? field.option2 : "option2 not found";
		} else {
			return field.option3 === "" || field.option3 ? field.option3 : "option3 not found";
		}
	};

	this.selectFlag = function($event) {
		if (this.isEdit()) {
			if (currentSelectedElement === $event.currentTarget) {
				currentSelectedElement.style.backgroundColor = "transparent";
				currentSelectedElement = null;
			} else {
				if (currentSelectedElement) {
					currentSelectedElement.style.backgroundColor = "transparent";
				}
				currentSelectedElement = $event.currentTarget;
				currentSelectedElement.style.backgroundColor = '#67a965';
			}

		}
	};

	this.setInMonth = function($event, field) {
		if (this.isEdit() && currentSelectedElement) {
			var flagElement = currentSelectedElement.children[0];
			var color = flagElement.style.backgroundColor;
			var monthElement = $event.currentTarget.children[1];
			monthElement.style.backgroundColor = color;
			var month = $($event.currentTarget).attr('month');
			this.changePropertieInCurrentRow(field, month);
		}
	};	

	this.changePropertieInCurrentRow = function(field, month) {
		if (!field.propertieToChange) {
			throw 'propertieToChange not found';
		} else {
			var option = $(currentSelectedElement).attr('option');
			var propertieToChange = field.propertieToChange;
			var value;
			if (!field.widget.currentRow[propertieToChange]) {
				value = '          ';
			} else {
				 value = field.widget.currentRow[propertieToChange];
			}
			value = value.split('');
			value[month -1] = option;
			value = value.join('');
			field.widget.currentRow[propertieToChange] = value;
		}
	};

	this.isEdit = function() {
		var widget = $scope.field.widget;
		if (widget.mode === 'edit') {
			return true;
		} else {
			return false;
		}
	};

	this.isCurrentMonth = function(month) {
		var currentMonth = moment().month() + 1;
		if (currentMonth == month) {
			return true;
		} else {
			return false;
		}
	};

	$scope.$watch(function() {
		return templateManager.container ? templateManager.container.getActiveWidget().currentRow : false;
	}, function(){
		var widget = templateManager.container.getActiveWidget();
		var fieldsSafra = widget.fields.filter(function(field) {
			return field.template === 'safra-component#field/safra.html' && field.isVisible;
		});
		fieldsSafra.forEach(function(field) {
			this.initColors(field);
		}, this);
	}.bind(this));
};

// FILE: ./assets/modules/produto/templates/controllers/register.js
var register = function(decodeTemplateFieldService, eventAggregator) {
    eventAggregator.subscribe('beforeProjectStartup', function register() {
        decodeTemplateFieldService.registerViewTemplate('safra-component#field/safra.html', 'safra-component#field/safra.html');
    });
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('register', register);
});


(function registerTemplates() {
    window.templateUrls = window.templateUrls || [];
    window.templateUrls.push(
        {
            'package': 'safra-component',
            'baseUrl': './assets/modules/produto/templates/'
        }
    );
})();

// FILE: ./assets/js/utilities/Endereco_Controller.js
function Endereco_Controller(General, ScreenService, templateManager, ZHPromise) {
    var self = this;

    //////////////////////////////////////////////////////////
    //                                                      //
    // ESSE CONTROLLER ESPERA AS SEGUINTES CARACTERÍSTICAS: //
    //                                                      //
    // na widget principal:                                 //
    //     addressWidget, que indica o nome da widget       //
    //     do grid de endereço;                             //
    //                                                      //
    //     addressWidgetToForm, que indica o nome da        //
    //     widget do form de endereço.                      //
    //                                                      //
    // na widget de formulário de endereço:                 //
    //     addressTypeColumn, que indica o nome do field    //
    //     do tipo de endereço.                             //
    //                                                      //
    //     event WidgetOnEnter com o método                 //
    //                     addressWidgetOnEnter em seu code //
    //                                                      //
    //////////////////////////////////////////////////////////

    /* **************************************************
     * Procedimento qeu verifica a flag de visitação da
     * widget de formulário de endereço.
     *
     * @param Widget widget
     */
    self.addressWidgetOnEnter = function(widget) {
        if(widget.visited == 404)
            setTimeout(function() {widget.isValid();}, 5000); //blame Wagão for setTimeout
                                                              //R.I.P. Wagão ??? - 2016

        widget.visited = true;
    };

    /* **************************************************
     * Procedimento que cancela o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.cancelInsertion = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);

        self.clearRow(mainWidget);

        self.exit(mainWidgetName, validationFunction, validationParams);
    };

    /* **************************************************
     * Procedimento que remove a currentRow da widget
     * principal para o caso de cancelamento da inserção.
     *
     * @param Widget mainWidget
     */
    self.clearRow = function(mainWidget) {
        var indexOfCurrentRow = mainWidget.dataSource.data.indexOf(mainWidget.currentRow);

        if(indexOfCurrentRow !== -1) mainWidget.dataSource.data.pop();

        mainWidget.setCurrentRow({});
    };

    /* **************************************************
     * Procedimento que encerra o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.exit = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), true);

        self.prepareView(mainWidget);
    };

    /* **************************************************
     * Procedimento que abre o swipe de inserção da
     * widget principal junto com a sua widget de
     * formulário de endereço.
     *
     * Recebe o nome da widget principal que será aberta,
     * e permite o uso de uma função de validação
     * opcional a ser executada após a abertura do swipe,
     * que pode ou não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.newRow = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);
        
        try{
            let fieldTpInscricao = mainWidget.getField('IDTPIJURFILI');
    		fieldTpInscricao.readOnly = false;
        }catch(error){ 
            //do nothing
        }
        
        formAddressWidget.visited = false;

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), false);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        self.prepareForm(mainWidget, formAddressWidget);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);
    };

    /* **************************************************
     * Procedimento que configura as widgets da inserção
     * de um registro com endereço para dito processo.
     *
     * @param Widget mainWidget
     * @param Widget formAddressWidget
     */
    self.prepareForm = function(mainWidget, formAddressWidget) {
        mainWidget.newRow();

        formAddressWidget.setCurrentRow({});
        formAddressWidget.edit();

        formAddressWidget.currentRow.__is_new = true;
        formAddressWidget.currentRow[formAddressWidget.addressTypeColumn] = 'P';

        formAddressWidget.getField(formAddressWidget.addressTypeColumn).readOnly = true;
    };

    /* **************************************************
     * Procedimento que configura a widget principal para
     * sair do modo de inserção.
     *
     * @param Widget mainWidget
     */
    self.prepareView = function(mainWidget) {
        mainWidget.list();

        ScreenService.closeSwipe();

        mainWidget.activate();
    };

    /* **************************************************
     * Procedimento que checa a validez das duas widgets
     * de inserção e efetua o salvamento quando possível.
     *
     * Também avisa o usuário sobre qual(is) widget(s)
     * estão inválidas, e direciona-o caso necessário.
     *
     * Recebe o nome da widget principal que será salva,
     * além da função que será executada para efetuar o
     * salvamento. Permite o uso de funções de sucesso e
     * erro opcionais a serem executadas após o
     * salvamento.
     *
     * IMPORTANTE: A função "saveParams" DEVE retornar
     * promise!
     *
     * @param string mainWidgetName
     * @param Function saveFunction
     * @param Array saveParams
     * @param Function successFunction
     * @param Function errorFunction
     */
    self.save = function(mainWidgetName, saveFunction, saveParams, successFunction, errorFunction) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        var mainIsValid = mainWidget.isValid();
        
        this.checkAndRegisterNeighborhood(formAddressWidget.getField('NMBAIRRO'), formAddressWidget.visited === true).then(resultado => {
            if(formAddressWidget.visited === true){
                if(resultado.dataset.bairro.length > 0)
                    formAddressWidget.getField('NMBAIRRO').dataSource.data = resultado.dataset.bairro;
                if(resultado.dataset.bairro.length == 1){
                    let bairro = resultado.dataset.bairro[0];
                    formAddressWidget.currentRow.CDBAIRRO = bairro.CDBAIRRO;
                    formAddressWidget.currentRow.NMBAIRRO = bairro.NMBAIRRO;
                }
            } 
            //checks address widget validity
                /* note: Widget's "isValid" function returns true if the widget wasn't visited,
                 * even if it's not actually valid. So, its necessary to check the real address
                 * validity, in this case, using a flag. */
            var addressIsValid = (formAddressWidget.isValid() && (formAddressWidget.visited === true));
    
            switch(true) {
                case mainIsValid && addressIsValid:
                    if(saveFunction instanceof Function)
                        saveFunction.apply(this, saveParams).then(function(callback) {
                            self.exit(mainWidgetName);
    
                            if(successFunction instanceof Function)
                                successFunction.apply(this, callback);
                        }, errorFunction);
                    break;
    
                case !mainIsValid && !addressIsValid:
                    if(formAddressWidget.visited === true)
                        ScreenService.showMessage("Preencha todos os campos obrigatórios das duas telas!");
                    else
                        formAddressWidget.visited = 404;
                    break;
    
                case !mainIsValid && !mainWidget.isActive():
                    ScreenService.showMessage("Preencha os campos obrigatórios!");
    
                    mainWidget.currentWidget = mainWidget;
    
                    mainWidget.activate();
                    break;
    
                case !addressIsValid && !formAddressWidget.isActive():
                    formAddressWidget.visited = 404;
    
                    ScreenService.showMessage("Preencha os campos obrigatórios!");
    
                    formAddressWidget.activate();
                    break;
            }
        });
        
    };

    /* **************************************************
     * Procedimento que troca entre a widget de grid de
     * endereço e widget de formulário de endereço.
     *
     * Para que o procedimento funcione corretamente, a
     * widget de grid deve ter sua visibilidade marcada
     * inicialmente como true.
     *
     * @param Widget mainAddressWidget
     * @param Widget formAddressWidget
     */
    self.switchAddressVisibility = function(mainAddressWidget, formAddressWidget) {
        var previousState = mainAddressWidget.isVisible;

        mainAddressWidget.enabled = mainAddressWidget.showAsTab = mainAddressWidget.isVisible = !previousState;

        formAddressWidget.enabled = formAddressWidget.isVisible = formAddressWidget.showAsTab = previousState;
    };
    
    this.checkAndRegisterNeighborhood = function(neighborhoodField){
        if(!neighborhoodField.value() && neighborhoodField.dataSource.data.length == 0){
            let neighborhoodValue = neighborhoodField.dataSourceFilter.filter(data => data.operator == 'LIKE_ALL' && data.value != '%%');
            if(neighborhoodValue.length > 0){
                neighborhoodValue = neighborhoodValue[0].value.replace(/%/g, '');
                let neighborhoodSequentialFilter = neighborhoodField.dataSourceFilter;
                neighborhoodSequentialFilter.forEach(filter =>{
                    if(filter.name == 'NMMUNICIPIO')
                        filter.value = neighborhoodValue;
                });
                
                return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', neighborhoodSequentialFilter);
            }else{
                return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', []);
            }
        }else
            return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', []);
    };
    
    Number.prototype.pad = function(size) {
      var s = String(this);
      while (s.length < (size || 2)) {s = "0" + s;}
      return s;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Endereco_Controller', Endereco_Controller);
});

// FILE: ./assets/js/utilities/EstUtil.js
var EstUtil = function(templateManager, General, ZHPromise) {

    this.getFieldValue = function(filedName, widgetName, fromFilter = false) {
        try {
            var container = templateManager.container;
            if (container) {
                let valuesWidget = container.getWidget(widgetName);
                if (fromFilter) {
                    valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
                }
    
                if (valuesWidget) {
                    var valueField = valuesWidget.currentRow[filedName];
                    return valueField || null;
                }
                else return null;
            }
            else return null;
        }catch(error){ 
            return null; 
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('EstUtil', EstUtil);
});


// FILE: ./assets/js/utilities/ProductField.js
function ProductField(templateManager) {
    const isUserFiltering = ({selectWidget: widget}) => !!widget.filterApplied.length;

    const updateInnerWidgetExhibitionMode = ({selectWidget: widget}, mode) => {
        const treeModifier = mode === 'tree'? 'tree/' : '';

        widget.template = `widget/${treeModifier}grid.html`;

        return templateManager.updateTemplate();
    };

    //Tree behavior blocked, waiting Zeedhi Framework to become faster
    this.updateExhibitionMode = field => {
        const mode = 'grid'; //isUserFiltering(field)? 'grid' : 'tree';

        field.showSelectAs = mode;

        return updateInnerWidgetExhibitionMode(field, mode).then(() => mode);
    };

    this.updateProductFilters = widget => {
        const fields = widget.fields.filter(field => field.label.startsWith('Produto'));
        const {CDFILIAL} = widget.currentRow;


        //switch to updateSelectFieldFilialFilter when Zeedhi Framework manage to become faster
        fields.forEach(removeSelectFieldFilialFilter);
    };

    const isFilterable = field => field.template.includes("select") &&
                                 !field.template.includes("native") &&
                                !!field.dataSourceFilter;

    const removeSelectFieldFilialFilter = field => {
        if(isFilterable(field)) {
            field.dataSourceFilter = field.dataSourceFilter.filter(filter => filter.name != 'CDFILIAL');
            field.reload();
        }
    };

    const updateSelectFieldFilialFilter = field => {
        if(isFilterable(field)) {
            field.dataSourceFilter.find(filter => filter.name == 'CDFILIAL').value = CDFILIAL;
            field.reload();
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ProductField', ProductField);
});


// FILE: ./assets/modules/produto/assets/js/utilities/produtoValidations.js
var produtoValidations = function (ScreenService, templateManager, ValidationEngine, MessageService) {

    var validSemiLevel = function(value, use) {
        
        /* MSG NOVA */
		var variaveis = {
            LEVELMASKPROD: getLevelMaksPenul()
        };
		
		/* FIM MSG NOVA */  
        
        return {
            "valid": validLevel(value, use),
            "message": MessageService.showMessageByCode("GETLEVELMASKPROD", variaveis, {"type":"error"})
        };
        // "message": ScreenService.i18n('Nível do produto deve ser igual a '+getLevelMaksPenul())
    };

    var validNivante = function(value, params) {
        return {
            "valid": validRealNivante(value, params.widgetName, params.nivanteValue),
            "message": ScreenService.i18n(params.message || 'Código inválido para produto pai selecionado.')
        };
    };

    ValidationEngine.registerValidation('containsNivante', validNivante);
    ValidationEngine.registerValidation('penultimateLevel', validSemiLevel);

    function validLevel(value, use) {
        var rightValue = getLevelMaksPenul();
        if (value != rightValue && use) {
            return false;
        } else {
            return true;
        }
    }

    function validRealNivante(value, widgetName, cdnivanteName) {
        var widget = templateManager.container.getWidget(widgetName);
        var cdnivante = widget.currentRow[cdnivanteName] || null;
        var valueDesmasked = value.split('.').join('');

        if (cdnivante && cdnivante !== valueDesmasked.substr(0, cdnivante.length)) {
            return false;
        } else {
            return true;
        }
    }

    function getLevelMaksPenul() {
        var mask = getMaskPenulLevel();
        mask = mask.split('.');
        return mask.length;
    }
    

    function getMaskPenulLevel() {
        var mask = getLocalVar('GER_CDPICTPROD');
		mask = mask.split('.');
		mask.pop();
		return mask.join('.');
    }



};

Configuration(function (ContextRegister, eventEngine) {
    ContextRegister.register('produtoValidations', produtoValidations);
});

// FILE: ./assets/js/repositories/Ger01800_UnidadeMedida_Repository.js
function Ger01800_UnidadeMedida_Repository(restEngine) {

    this.excluirUnidMedida = function(row) {
        return restEngine.post('cadEst#/excluirUnidMedida', {
            "requestType": "Row",
            "row": row
        });
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger01800_UnidadeMedida_Repository', Ger01800_UnidadeMedida_Repository);
});

// FILE: ./assets/js/repositories/Ger10000_SegmentoDeMercado_Repository.js
function Ger10000_SegmentoDeMercado_Repository(restEngine) {

    this.saveProdutosSegmento = function(row) {
         return restEngine.post('cadEst#/saveProdutosSegmento', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.deleteProdutosSegmento = function(row) {
         return restEngine.post('cadEst#/deleteProdutosSegmento', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.updateProdAtivo = function(rows) {
         return restEngine.post('cadEst#/crud/produtoseg_produto/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger10000_SegmentoDeMercado_Repository', Ger10000_SegmentoDeMercado_Repository);
});


// FILE: ./assets/js/repositories/Ger14600_ComposicaoDoProduto_Repository.js
function Ger14600_ComposicaoDoProduto_Repository(restEngine) {

    this.deletaFilhos = function(row) {
         return restEngine.post('cadEst#/grupromoc/remove', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.verificaExistenciaDados = function(row) {
        var filtro = [
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": row.CDPRODUTO
            }    
        ];
        return restEngine.post(row.ROTA, {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
    this.verificaExistenciaDadosIL = function(row) {
       
        var filtro = [
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": row.CDPRODUTO
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            }
        ];
        return restEngine.post(row.ROTA, {
            "requestType": "FilterData",
            "filter": filtro
        });

    };
    
    this.saveBlob = function(row) {
         return restEngine.post('cadEst#/saveBlob', {
            "requestType": "Row",
            "row": row
        });
    };
   

}

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger14600_ComposicaoDoProduto_Repository', Ger14600_ComposicaoDoProduto_Repository);
});


// FILE: ./assets/js/repositories/ParamfilialRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {

	var ParamfilialRepository = RepositoryFactory.factory('cadEst#/crud/paramfilial', 'LOCAL', 1, 30000);

	ParamfilialRepository.getParamfilial = function(nrorg, cdfilial) {
		return this.download(Query.build()
			.where('nrorg').equals(nrorg)
			.where('cdfilial').equals(cdfilial)
		);
	};

	ContextRegister.register('ParamfilialRepository', ParamfilialRepository);

});

// FILE: ./assets/js/repositories/ParamgeralRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {

	var ParamgeralRepository = RepositoryFactory.factory('cadEst#/crud/paramgeral', 'LOCAL', 1, 30000);

	ParamgeralRepository.getParamgeral = function(nrorg) {
		return this.download(Query.build()
			.where('nrorg').equals(nrorg)
		);
	};

	ContextRegister.register('ParamgeralRepository', ParamgeralRepository);

});

// FILE: ./assets/js/repositories/Unidade_Repository.js
var Unidade_Repository = function(localStorageEngine, restEngine) {
    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("cadEst#/crud/filial", filter);
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("cadEst#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("cadEst#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Unidade_Repository', Unidade_Repository);
});

// FILE: ./assets/js/repositories/ger01700_unidade_repository.js
function ger01700_unidade_repository(restEngine) {
    var self = this;

    /* **************************************************
     * Função que requisita a atualização dos produtos de
     * uma filial.
     *
     * @param string CDFILIAL
     * @param number CDNVPRODUTO
     *
     * @return Promise
     */
    self.requestAtualizaProduto = function(CDFILIAL, CDNVPRODUTO) {
        return restEngine.post("cadEst#/atualizaProduto", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "P_CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                },
                {
                    name: "P_CDNVPRODUTO",
                    operator: "=",
                    value: CDNVPRODUTO
                }
            ]
        }, undefined, undefined, undefined, undefined, 8001000); //the timeout limit É MAIS DE OITO MIL!!!11!1
    };

    /* **************************************************
     * Função que requisita a validação de endereço.
     *
     * @param string CDFILIAL
     *
     * @return Promise
     */
    self.requestEndefiliValidation = function(CDFILIAL) {
        return restEngine.post("cadEst#/endefili_validation", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "P_CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };

    /* **************************************************
     * Função que requisita as validações de Paramgeral.
     *
     * @return Promise
     */
    self.requestParametrizacaoValidation = function() {
        return restEngine.post("cadEst#/paramgeral_paramfilipadr_validation", {
            "requestType": "FilterData",
            "filter": []
        });
    };

    /* **************************************************
     * Função que requisita as validações da PK de uma
     * Unidade.
     *
     *  @param string CDFILIAL
     *
     * @return Promise
     */
    self.requestUnidadeCodeValidation = function(CDFILIAL) {
        return restEngine.post("cadEst#/unidade_code_check", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "P_CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger01700_unidade_repository', ger01700_unidade_repository);
});

// FILE: ./assets/js/services/Ger01800_UnidadeMedida_Service.js
function Ger01800_UnidadeMedida_Service(Ger01800_UnidadeMedida_Repository) {

	this.excluirUnidMedida = function (row) {
		return Ger01800_UnidadeMedida_Repository.excluirUnidMedida(row);
	};
	
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger01800_UnidadeMedida_Service', Ger01800_UnidadeMedida_Service);
});

// FILE: ./assets/js/services/Ger09900_AgrupamentoDeProdutos_Service.js
var Ger09900_AgrupamentoDeProdutos_Service = function(restEngine, localStorageEngine) {

    this.verificaProdutos = function(filter){
        return restEngine.post('cadEst#/crud/produto_nvsubgrprod', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    this.saveSubGrupo = function(data) {
        return restEngine.post('cadEst#/saveSubGrupo', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.saveProduto = function(data) {
        return restEngine.post('cadEst#/saveProduto', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.deleteProduto = function(data) {
        return restEngine.post('cadEst#/deleteProduto', {
            "requestType": "DataSet",
            "dataset": data
        });
    };
    this.deleteGrupo = function(data) {
        return restEngine.post('cadEst#/deleteGrupo', {
            "requestType": "DataSet",
            "dataset": data
        });
    };
    this.deleteSubGrupo = function(data) {
        return restEngine.post('cadEst#/deleteSubGrupo', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register("Ger09900_AgrupamentoDeProdutos_Service", Ger09900_AgrupamentoDeProdutos_Service);
});

// FILE: ./assets/js/services/Ger10000_SegmentoDeMercado_Service.js
function Ger10000_SegmentoDeMercado_Service(Ger10000_SegmentoDeMercado_Repository) {

	this.saveProdutosSegmento = function (row) {
		return Ger10000_SegmentoDeMercado_Repository.saveProdutosSegmento(row);
	};
	
	this.deleteProdutosSegmento = function (row) {
		return Ger10000_SegmentoDeMercado_Repository.deleteProdutosSegmento(row);
	};
	
	this.updateProdAtivo = function (rows) {
		return Ger10000_SegmentoDeMercado_Repository.updateProdAtivo(rows);
	};
	
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger10000_SegmentoDeMercado_Service', Ger10000_SegmentoDeMercado_Service);
});

// FILE: ./assets/js/services/Ger14600_ComposicaoDoProduto_Service.js
function Ger14600_ComposicaoDoProduto_Service(Ger14600_ComposicaoDoProduto_Repository) {

	this.deletaFilhos = function (row) {
		return Ger14600_ComposicaoDoProduto_Repository.deletaFilhos(row);
	};

	this.verificaExistenciaDados = function (row) {
		return Ger14600_ComposicaoDoProduto_Repository.verificaExistenciaDados(row);
	};
	
	this.saveBlob = function (row) {
		return Ger14600_ComposicaoDoProduto_Repository.saveBlob(row);
	};
	
	this.setDataSourceFilter = function (widget, widgetToFilter) {
		widgetToFilter.dataSourceFilter=[
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": widget.currentRow.CDPRODUTO
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORG')
            }
		];
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger14600_ComposicaoDoProduto_Service', Ger14600_ComposicaoDoProduto_Service);
});

// FILE: ./assets/js/services/ParamfilialService.js
var ParamfilialService = function(ParamfilialRepository) {

	this.getParamfilial = function(nrorg, cdfilial) {
		return ParamfilialRepository.getParamfilial(nrorg, cdfilial);
	};

	this.getDtprocessa = function(nrorg, cdfilial) {
		return this.getParamfilial(nrorg, cdfilial).then(function(paramfilial) {
			return paramfilial[0].DTPROCESSA;
		});
	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ParamfilialService', ParamfilialService);
});

// FILE: ./assets/js/services/ParamgeralService.js
var ParamgeralService = function(ParamgeralRepository) {

	this.getParamgeral = function(nrorg) {
		return ParamgeralRepository.getParamgeral(nrorg);
	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ParamgeralService', ParamgeralService);
});

// FILE: ./assets/js/services/ProcProdWorkflowService.js
var ProcProdWorkflowService = function(restEngine) {

	this.setProcWorkflowProfile = function(){

		var filter = [
			{
				"name": "NRPROCESSOAPROVA",
				"operator": "=",
				"value": "2"
			}
		];

		return restEngine.post("cadEst#/crud/processoaprova", {
			"requestType":"FilterData",
			"filter": filter
		}).then(function(callback){
			setLocalVar("INFLIGHTOR_WORKFLOWPROD", callback.dataset.processoaprova[0].IDATIVO);
		});

	};

	this.procProdWorkflow = function(row){

		return restEngine.post("cadEst#/ProcProdWorkflow/procProdWf", {
			"requestType":"Row",
			"row": row
		});

	};

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('ProcProdWorkflowService', ProcProdWorkflowService);
});

// FILE: ./assets/js/services/ProdutoService.js
var ProdutoService = function(restEngine) {

	this.saveProd = function(row){

		return restEngine.post("cadEst#/produtocrud/save", {
			"requestType":"Row",
			"row": row
		});

	};

	this.saveProdArvProd = function(row){

		return restEngine.post("cadEst#/arvoreprodtree/save", {
			"requestType":"Row",
			"row": row
		});

	};
	

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('ProdutoService', ProdutoService);
});

// FILE: ./assets/js/services/QueryService.js
function QueryService(restEngine){
    this.RunQuery = function(row){
        return restEngine.post({
                "service":"cadEst#/queryRun",
                "params": {
                    "requestType" : "Row",
                    "row" : row
                },
                "timeout": 600000,
                "typeRequest": "POST"
            });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('QueryService', QueryService);
});

// FILE: ./assets/js/services/ger01700_unidade_service.js
function ger01700_unidade_service(Endereco, General, ger01700_unidade_repository, restEngine) {
    var self = this;

    self.alterarEstrutura = function ({NMFILIAL, CDFILIAL, NRORG}) {
        const filial = {NMFILIAL, CDFILIAL, NRORG};

        return General.callRestEngine('cadEst#/alterarEstrutura', 'Row', filial);
    };

    /* **************************************************
     * Função que prepara a atualização dos produtos de
     * uma unidade e retorna o resultado da operação.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.atualizaProduto = function(cdfilial) {
        var cdnvproduto = General.getLevelByMask('GER_CDPICTPROD','1').length;

        return ger01700_unidade_repository.requestAtualizaProduto(cdfilial, cdnvproduto);
    };

    /* **************************************************
     * Função que retorna um request de validação de CEP
     * efetuado para a API dos Correios.
     *
     * @param String CEP
     *
     * @return Promise
     */
    self.validateCEP = function(CEP) {
        return Endereco.buscarCep(CEP);
    };

    /* **************************************************
     * Função que requisita a validação de endereço.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.validateEndefili = function(cdfilial) {
        return ger01700_unidade_repository.requestEndefiliValidation(cdfilial);
    };

    /* **************************************************
     * Função que retorna a validez de uma key para um
     * field específico, ou para números inteiros.
     *
     * @param int key
     * @param string fieldName
     *
     * @return boolean
     */
    self.validateKeys = function(key, fieldName) {
        var integerNumbers = [48, 49, 50, 51,  52,  53,  54,  55,  56,  57];
        var validKeys = {};

        validKeys.integer = integerNumbers;
        validKeys.MSKINSJURFILI = integerNumbers.concat([45, 46, 47]); //'-', '.', '/'

        if(fieldName && validKeys.hasOwnProperty(fieldName))
            return validKeys[fieldName].indexOf(key) < 0;
        else
            return validKeys.integer.indexOf(key) < 0;
    };

    /* **************************************************
     * Função que trata o objeto de inscrição, montando
     * as mensagens de erro caso necessário.
     *
     * @param int key
     * @param string fieldName
     *
     * @return object(inscricao)
     */
    self.validateNumeroDeInscricao = function(inputValue) {
        var inscricao = General.inscricaoFormat(inputValue);
        
         /* **************************************************
          * parseInt() e toString() apenas para retirar os zeros a esquera que a função  
          * "General.inscricaoFormat(inputValue)" coloca 
          */

        if(!inscricao.valid) {
            inscricao.error = {};
            switch(inscricao.tipo.cod) {
                case 'J':
                    inscricao.error.msg = "CNPJ \"" + inscricao.value + "\" inválido!";
                    break;
                case 'F':
                    inscricao.error.msg = "CPF \"" + inscricao.value + "\" inválido!";
                    break;
                case null:
                    break;
                case 'I':
                    /* Dont manipulate international codes */
                    break;
                default:
                    inscricao.error.msg = inscricao.tipo.desc + " \"" + inscricao.value + "\" inválida!";
            }
        }

        return inscricao;
    };

    /* **************************************************
     * Função que requisita as validações de Paramgeral.
     *
     * @return Promise
     */
    self.validateParametrizacao = function() {
        return ger01700_unidade_repository.requestParametrizacaoValidation();
    };

    /* **************************************************
     * Função que requisita as validações da PK de uma
     * unidade.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.validateUnidadePK = function(cdfilial) {
        return ger01700_unidade_repository.requestUnidadeCodeValidation(cdfilial);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger01700_unidade_service', ger01700_unidade_service);
});


// FILE: ./assets/js/controllers/Ger01800_UnidadeMedida_Controller.js
var Ger01800_UnidadeMedida_Controller = function(ScreenService, General,ZHPromise, Ger01800_UnidadeMedida_Service) {
	var firstLoad = true;
	var focus = false;
	var fieldToFocus = '#SGUNIDADE';
	
	this.filterConvByUnid = function(widget){
		if(widget.currentRow.__is_new){
			widget.container.getWidget('conversao').isVisible = false;
		}else{
			widget.container.getWidget('conversao').isVisible = true;		
			widget.container.getWidget('conversao').dataSourceFilter = [
				{
					"name": "SGUNIDADE",
					"operator": "=",
					"value": widget.currentRow.SGUNIDADE
				}
			];		
			widget.container.getWidget('conversao').reload();		
		}
	};

	this.onLoadDataSource = function (widget) {
		var rows = widget.dataSource.data;
		rows.forEach(function(row){
			row.MSG = row.VRFTCONVER + " " + row.DSUNIDPARA + " (s) equivale a 1 " + row.DSUNIDDE;
		});
	};
	
	this.triggerFocusField = function(widget){
		//ScreenService.showMessage("a");
		focus = true;
		widget.reload();	
	};
	
	this.focusField = function(widget) {
		if(focus) {
			if(firstLoad) {
				widget.reload();
				firstLoad = false;
			} else {
				document.querySelector(fieldToFocus).focus();	
				fieldToFocus = '#SGUNIDADE';
			}
		}
	};
	
	this.cancelAdd = function(args) {
		General.backOrCancel(args);
		focus = false;
	};
	
	this.saveAdd = function(widget) {
		General.saveCrud(widget).then(function() {
			ScreenService.goBack();
			widget.reload();	
		});
		focus = false;
	};
	
	this.editUnit = function(widget) {
		General.edit(widget);
		focus = true;
		fieldToFocus = '#DSUNIDADE';
		widget.reload();
	};
	
	this.onAddConvercao = function(widget){
		widget.newRow();
		widget.getField('RELACAOVALORES').dataSource = {};
		var currentRowUnidade = widget.container.getWidget('unidade').currentRow;
		widget.currentRow.SGUNIDDE = currentRowUnidade.SGUNIDADE;
		widget.currentRow.DSUNIDDE = currentRowUnidade.DSUNIDADE;
	};

	this.onChangeUnidConv = function(field) {
		if(field.value()){
			field.widget.getField('RELACAOVALORES').readOnly = false;
			field.widget.getField('VRFTCONVER').readOnly = false;
		}else{
			field.widget.getField('RELACAOVALORES').readOnly = true;
			field.widget.getField('VRFTCONVER').readOnly = true;	
		}
	};

	this.loadOpsRelacao = function(field){
		var currentRow = field.widget.currentRow;
		field.widget.getField('RELACAOVALORES').dataSource.data = [
			{
				"name": "dePara",
				"value": "1 "+ currentRow.DSUNIDDE +" equivale a "+ currentRow.DSUNIDPA + "(s)"
			},
			{
				"name": "paraDe",
				"value": "1 "+ currentRow.DSUNIDPA +" equivale a "+ currentRow.DSUNIDDE + "(s)"
			}
		];
	};

	this.prepVrFtConv = function(widget){
		if(widget.getField('RELACAOVALORES').value() === "dePara"){
			widget.currentRow.VRFTCONVER = General.valueToFloat(widget.currentRow.VRFTCONVER);
		}else{
			widget.currentRow.VRFTCONVER = this.roundFload(1 / (General.valueToFloat(widget.currentRow.VRFTCONVER)), 5);
		}
		widget.currentRow.IDTIPOPE = "*";
		General.saveCrud(widget).then(function(){
			widget.reload();
		});	
	};
	
	this.roundFload = function(value, decimals) {
		return Number(Math.round(value+'e'+decimals)+'e-'+decimals);
	};

	this.convView = function(widget){
		if(widget.currentRow.__is_new){
			// widget.getField('RELACAOVALORES').readOnly = true;
			widget.getField('VRFTCONVER').readOnly = true;
			widget.getField('DSUNIDPA').readOnly = false;
		}else{
			widget.currentRow.DSUNIDPA = widget.currentRow.DSUNIDPARA;
			widget.getField('RELACAOVALORES').dataSource.data = [
				{
					"name": "dePara",
					"value": "1 "+ widget.currentRow.DSUNIDDE +" equivale a "+ widget.currentRow.DSUNIDPA + "(s)"
				},
				{
					"name": "paraDe",
					"value": "1 "+ widget.currentRow.DSUNIDPA +" equivale a "+ widget.currentRow.DSUNIDDE + "(s)"
				}
			];
			// widget.getField('RELACAOVALORES').readOnly = false;
			widget.getField('VRFTCONVER').readOnly = false;
			widget.getField('DSUNIDPA').readOnly = true;
		}
		
	};
	
	this.excluirUnidade = function(widget){
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            Ger01800_UnidadeMedida_Service.excluirUnidMedida(widget.currentRow).then(function(){
				ScreenService.goBack();
				widget.reload();
				ScreenService.showMessage('Excluido com sucesso');
			});
    	});
	};
	
	this.validaRelacaoConv = function(field){
		var vrftconver = field.widget.currentRow.VRFTCONVER;
		if(vrftconver < 0.00001){
			ScreenService.showMessage('Relação (valor de x) deve ser maior que 0.00001');
		}
	};

};

Configuration(function(ContextRegister) {
	ContextRegister.register("Ger01800_UnidadeMedida_Controller", Ger01800_UnidadeMedida_Controller);
});

// FILE: ./assets/js/controllers/Ger03800_ProdutosUnidade_Controller.js
var Ger03800_ProdutosUnidade_Controller = function(ScreenService, General, templateManager, ZhFilterPreferences){
    var mensagens = {
        tempoMedio: 'O tempo médio de reposição deve estar entre 0 e 999.',
        unidadeNaoControlaEstoque: 'A unidade selcionada não controla estoque.',
        estoquePossuiSaldo : "Operação Bloqueada! Alteração requer que o produto seja zerado no estoque. É recomendado que antes de zerar o produto seja emitida a posição atual do mesmo na filial." 
    };
    
    this.getFilialLogada = function(){
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine( 'cadEst#/crud/filial', 'FilterData', filter);
    };
    
    this.openFilter = function(widget) {
        
        var defaultFilter = Util.clone(widget.currentRow);
        
        this.getFilialLogada().then(function(response) {
            var filial = response.dataset.filial[0];
            defaultFilter.CDFILIAL = filial.CDFILIAL;
            defaultFilter.NMFILIAL = filial.NMFILIAL;
            defaultFilter.IDCNTRESTOQ = "A";
            defaultFilter.IDPRODATIV = "T";
            ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(function(){
                var filtro = templateManager.container.getActiveWidget();
                this.validaFilial(filtro);
                widget.getField("NMPRODINI").reload();
            }.bind(this));
        }.bind(this)).catch(() => {
            ZhFilterPreferences.openFilterWidget(widget);
        });
        
    };
    
    this.validaFilial = function(widget) {
        if(widget.currentRow.NMFILIAL && widget.currentRow.CDFILIAL) {
            widget.getField('NMPRODINI').readOnly = false;
        }
        else {
            widget.getField('NMPRODINI').readOnly = true;
            widget.getField('NMPRODFIN').readOnly = true;
            widget.currentRow.NMPRODFIN   = '';
    		widget.currentRow.P_CDPRODFIN = '';
    		widget.currentRow.NMPRODINI   = '';
    		widget.currentRow.P_CDPRODINI = '';
        }
    };
    
    this.FilterProdIni = function(args){
		var widget = args.owner.field.widget;
        var row = widget.currentRow;
        var prodIniIsSet = this.fieldIsSet(row.P_CDPRODINI);
        if(!prodIniIsSet) {
    		widget.currentRow.NMPRODFIN = '';
    		widget.currentRow.P_CDPRODFIN = '';
        	delete widget.getField('NMPRODINI').validations.required;
        	delete widget.getField('NMPRODFIN').validations.required;
        }
        else {
    	 widget.getField('NMPRODFIN').validations.required = true;
        }
        widget.getField('NMPRODFIN').readOnly = !prodIniIsSet;
        widget.getField('NMPRODFIN').readOnlyOnEdit = !prodIniIsSet;
    };
    
    this.fieldIsSet = function(fieldValue) {
        if(fieldValue) return true;
        else return false;
    };
    
    this.reloadWidget= function(widget, field){
        widget.getField(field).reload();
    };
    

    this.FilterProdFin = function(args){
    	var widget = args.owner.field.widget;
        var row = widget.currentRow;
        var prodFinIsSet = this.fieldIsSet(row.P_CDPRODFIN);
		if(prodFinIsSet) {
			widget.getField('NMPRODINI').validations.required = true;
            if(row.P_CDPRODINI > row.P_PRODFIN){
                ScreenService.showMessage("O codigo do produto inicial não pode ser maior que do final");
                return 0;
            }
        } 
        else {
    		delete widget.getField('NMPRODINI').validations.required;
        }
        widget.reload();
    };

    this.openPopupCadastro = function(widget) {
        var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICO');
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
    };
    
    this.openPopupCadastroProdFili = function(widget) {
        var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICOPRODFILI');
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
    };
    
    this.ajustaFiltro = function(args, widget) {
        args.e.preventDefault();
    	if(!widget.appliedFilter.FILTER_ROW.P_CDPRODINI && !widget.appliedFilter.FILTER_ROW.P_CDPRODFIN) {
    		widget.dataSourceFilter[0].value = 'T';
            // widget.getParent().widgets[1].dataSourceFilter[0].value = 'T';
    	}
    	else {
    	    widget.dataSourceFilter[0].value = '';
    	    widget.dataSourceFilter[1].value = widget.appliedFilter.FILTER_ROW.P_CDPRODFIN;
    	    widget.dataSourceFilter[2].value = widget.appliedFilter.FILTER_ROW.P_CDPRODINI;
    	    
    	   // widget.getParent().widgets[1].dataSourceFilter[0].value = '';
    	   // widget.getParent().widgets[1].dataSourceFilter[1].value = widget.appliedFilter.FILTER_ROW.P_CDPRODFIN;
    	   // widget.getParent().widgets[1].dataSourceFilter[2].value = widget.appliedFilter.FILTER_ROW.P_CDPRODINI;
    
    	}
    	var newFilter = widget.dataSourceFilter.filter(f =>{
    	    if(!f.static && !(f.name == 'IDPRODATIV' && f.value == 'T'))
    	        return f;
    	});
    	widget.dataSourceFilter = newFilter;
    	widget.reload();
    };
    
    this.AjustaCtrEstoque = function(args){
        args.e.preventDefault();
        if(args.e.filters[1].value === "A"){
            let filter =  args.owner.dataSourceFilter.filter(param => { 
                if (param.name != "IDCNTRESTOQ"){
                    return true;
                   }else{
                       return false;
                   }
            });
            args.owner.dataSourceFilter = filter;
            args.owner.reload();
        }
    };
    
    this.saveCadastroAutomatico = function(widget) {

        var row = widget.currentRow;
        row.CDNVPRODUTO = General.getLevelByMask('CDPICTPROD',1).length;
        if(widget.isValid()) {
            General.callRestEngine('cadEst#/cadastroAutomaticoCtrlEstoq', 'Row', row).then(function(success){
                var widgetActivate = templateManager.container.getWidget('widgetProduto');
                var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICO');

                widgetPopup.isVisible = false;
                widgetPopup.currentRow = {};
                ScreenService.closePopup(widgetPopup);
                ScreenService.showMessage('Cadastrado com sucesso');
                widgetActivate.activate();
                widgetActivate.reload();

            },
            function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );


        }
    };
    
    this.saveCadastroAutomaticoProdFili = function(widget) {
        var row = widget.currentRow;
        
        for (var property in row){
            if(!row[property]){
                delete row[property];
            }
        }
        
        row.CDNVPRODUTO = General.getLevelByMask('CDPICTPROD',1).length;
        if(widget.isValid()) {
            General.callRestEngine('cadEst#/cadastroAutomaticoProdFili', 'Row', row).then(function(success){
                var widgetActivate = templateManager.container.getWidget('widgetProdFili');
                var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICOPRODFILI');
                    
                widgetPopup.isVisible = false;
                widgetPopup.currentRow = {};
                ScreenService.closePopup(widgetPopup);
                ScreenService.showMessage('Operação realizada com sucesso');
                widgetActivate.activate();
                widgetActivate.reload();

            },
            function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
    };

    this.cancelarCadastroAutomaticoProdFili = function() {
        var widget = templateManager.container.getWidget('CADASTROAUTOMATICOPRODFILI');
        widget.currentRow = {};
        var widgetActivate = templateManager.container.getWidget('widgetProdFili');
        widget.isVisible = false;
        ScreenService.closePopup(widget);
        widgetActivate.activate();
        widgetActivate.reload();
    };	
    
    this.saveProdFili = function(widget) {
        let percentFields = ["VRPTOLQTITAF", "VRPTOLVRITAF", "VRMINEMPENHOAF", "VRMINEMPENHOOL",  "VRPTOLAJUSINV"];
        percentFields.forEach(percentField => {
           let field = widget.getField(percentField);
           delete field.validations;
        });
        if(widget.isValid()) {
            validaValorPercentual(percentFields, widget);
        }
    };
    
    this.cancelarCadastroAutomaticoItem = function() {
        var widget = templateManager.container.getWidget('CADASTROAUTOMATICO');
        widget.currentRow = {};
        var widgetActivate = templateManager.container.getWidget('widgetProduto');
        widget.isVisible = false;
        ScreenService.closePopup(widget);
        widgetActivate.activate();
        widgetActivate.reload();
    };	
    

    function validaValorPercentual(fields, widget) {
        var existeErro = false;
        var fieldsErro = fields.filter((field) => {
            if(widget.currentRow[field] && (widget.currentRow[field] < 0 || widget.currentRow[field] > 100)) 
                return field;
        });
        var mensagem = "Valor digitado não é um valor válido para o(s) campo(s): ";
        fieldsErro.forEach((field) => {
            mensagem += widget.getField(field).label + ",";
            existeErro = true;
        });
        if(existeErro) {
            mensagem = mensagem.substr(0,(mensagem.length - 1));
            ScreenService.showMessage(mensagem + "!");
            return false;
        }
        else {
            widget.dataSource.name = 'cadEst#/crud/prodporfili_produto';
            General.saveCrud(widget).then(success => {
                widget.dataSource.name = 'cadEst#/prodporfili_produto';
            });   
        }
    }
    
    this.changeDataSourceToSave = function(widget){
        widget.dataSource.name = 'cadEst#/crud/prodfili_produto_ctrlestq'; 
        General.saveCrud(widget).then(success => {
                widget.dataSource.name = 'cadEst#/prodfili_produto_ctrlestq';
            });
    };
    
    this.cancelarAtualizaProdFili = function() {
        var widget = templateManager.container.getWidget('ATUALIZAPRODFILI');
        widget.currentRow = {};
        var widgetActivate = templateManager.container.getWidget('widgetProdFili');
        widget.isVisible = false;
        ScreenService.closePopup(widget);
        widgetActivate.activate();
        widgetActivate.reload();
    };
    
    this.openAtualizaProdFili = function(widget) {
        var widgetPopup = templateManager.container.getWidget('ATUALIZAPRODFILI');
        widgetPopup.newRow();
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
    };

    this.onChangeDestino = function(widget, lastChange, clearField) {
        var cdLastChange = "CD" + lastChange; 
        var nmLastChange = "NM" + lastChange;
        var cdClearField = "CD" + clearField;
        var nmClearField = "NM" + clearField;
        if(widget.currentRow[nmLastChange].length > 0) {
           if(widget.currentRow[nmClearField].length > 0) {
                widget.currentRow[cdClearField] = '';
                widget.currentRow[nmClearField] = '';
            } 
            widget.getField(nmClearField).validations = {};
            widget.getField(nmLastChange).validations = { required: true };
        }
        else if(widget.currentRow[nmClearField].length <= 0) {
            widget.getField(nmClearField).validations = { required: true };
            widget.getField(nmLastChange).validations = { required: true };
        } 
        else {
            widget.getField(nmClearField).validations = { required: true };
            widget.getField(nmClearField).validations = {};
        }
        
    };
    
    this.onChangeConfig = function(widget) {
        var nmListaParam = widget.getField("NMLISTAPARAM");
        if(widget.currentRow.IDTPCONFPAR == 1 || widget.currentRow.IDTPCONFPAR == 2) {
            nmListaParam.readOnly    = true;
            nmListaParam.validations = {};
        }  
        else {
            nmListaParam.readOnly    = false;
            nmListaParam.validations = { required: true};
        }
    };
    
    this.ctrlOnChange = function(widget){
        if(templateManager.container.getActiveWidget().name !== "zhFilterPreferencesSelectConditions" ){
            if(widget.row.IDCNTRESTOQ === "N"){
                this.verificaQntEstoque(widget).then(function(success){
                    if(success.dataset.qntEstoque_Produto.length > 0){
                    if(success.dataset.qntEstoque_Produto[0].QTESTOQDIA != "0"){
                        widget.row.IDCNTRESTOQ = "S";
                        ScreenService.showMessage(mensagens.estoquePossuiSaldo);
            }
                }
        }.bind(this));
             }
        }
    };
    
    this.verificaQntEstoque = function(widget){
       var filter = [
            {
                "name":"P_FILIAL",
                "operator":"=",
                "value": widget.row.CDFILIAL
            },
              {
                "name":"P_PRODFIN",
                "operator":"=",
                "value": widget.row.CDPRODUTO
            },
              {
                "name":"P_PRODINI",
                "operator":"=",
                "value": widget.row.CDPRODUTO
            }
        ];
        
       return General.callRestEngine('cadEst#/crud/qntEstoque_Produto', 'FilterData', filter);
    };
    
    
    this.atualizarProdFili = function(widget) {
      if(widget.isValid()) {
          if(widget.currentRow.P_CDPRODINI >  widget.currentRow.P_CDPRODFIN) {
              ScreenService.showMessage("Produto inicial deve ser menor ou igual que produto final.");
          }
          else if(widget.currentRow.CDFILIORIG == widget.currentRow.CDFILIDEST) {
              ScreenService.showMessage("Filial de origem deve ser diferente da filial de destino.");
          }
          else {
              General.callRestEngine('cadEst#/atualizaProdFili', 'Row', widget.currentRow).then(function(success){
                var widgetActivate = templateManager.container.getWidget('widgetProdFili');
                var widgetPopup = templateManager.container.getWidget('ATUALIZAPRODFILI');
                    
                widgetPopup.isVisible = false;
                widgetPopup.currentRow = {};
                ScreenService.closePopup(widgetPopup);
                ScreenService.showMessage('Operação realizada com sucesso');
                widgetActivate.activate();
                widgetActivate.reload();

            },
            function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
          }
      }  
    };
    
    this.getFieldValueFilter = function(filedName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];
                    
                return valueField || null;
            }
            else return null;
        }
        else return null;
    };
    
    this.fixMetadata = function(args) {
        let columns = args.e.metadata.columns;
        $.each(columns, (column) => { 
            if(columns[column].format != undefined)
                if(columns[column].format.type == 'viewFloat')
                    columns[column].format.type = 'float';
        });
    };
    
    this.cleanFieldsOnChange = function(field){
        let widget = field.widget;
        let fieldProdIni = widget.getField('NMPRODINI');
        let fieldProdFim = widget.getField('NMPRODFIN');
        
        switch(field.name){
            case 'NMPRODFIN':
                widget.currentRow.P_CDPRODFIN = widget.currentRow.P_CDPRODFIN + "zzzzzzzzzzzzzzzzzzzzz";
                widget.currentRow.P_CDPRODFIN = widget.currentRow.P_CDPRODFIN.substring(0, 20);
                break;
            case 'NMPRODINI':
                fieldProdIni.validations = {"required": true};
                fieldProdFim.validations = {"required": true};
                break;
        }
        
    };
    
    this.validaMinMaxCampo = function(field){
        if(field.value() < 0 || field.value() > 100){
            ScreenService.showMessage('Valor deve ser entre 0 e 100%');
            var fieldName = field.name;
            field.value("");
            delete field.widget.currentRow[fieldName];
            field.reload();
        }
    };
    
    this.setOldControlsOnRow = function(widget){
        let arrFieldsToStore = ['IDCNTRESTOQ', 'IDUTCTRLLOTE'];
        arrFieldsToStore.forEach(field => widget.currentRow['old_'+field] = widget.currentRow[field]);
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Ger03800_ProdutosUnidade_Controller', Ger03800_ProdutosUnidade_Controller);
});

// FILE: ./assets/js/controllers/Ger09900_AgrupamentoDeProdutos_Controller.js
function Ger09900_AgrupamentoDeProdutos_Controller(restEngine, ScreenService, General, Ger09900_AgrupamentoDeProdutos_Service, templateManager, ZhFilterPreferences) {



    this.moveRowSubGrupo = function(widget) {
        if (widget.mode == 'list') {
            General.buildDataSourceFilter(widget.dataSourceFilter, {
                CDGRUPPROD: widget.getParent().currentRow.CDGRUPPROD,
                CDFILISUBGR: "P"
            });
            if (widget.name == 'subGrupoFilial') {
                var filter = [{
                    "name": "CDGRUPPROD",
                    "operator": "=",
                    "value": widget.getParent().currentRow.CDGRUPPROD
                }];
                widget.getField('NMSUBGRPROD').dataSourceFilter = filter;
            }
            else {
                widget.getField('NMSUBGRPROD').dataSourceFilter = [];
            } 
            widget.getField('NMSUBGRPROD').reload();
            widget.reload();
        }
        
        let widgetProdutoSubGrupo = templateManager.container.getWidget("produtoSubGrupo");
        
        if(widget.currentRow.__is_new){
            widgetProdutoSubGrupo.isVisible=false;
        }else{
            widgetProdutoSubGrupo.isVisible=true;
        }
        
    };

    this.addProduto = function(widget) {
        if (widget.dataSource.data.length < widget.getParent().currentRow.NRMAXITSUBGR) {
            widget.newRow();
        }
        else {
            ScreenService.showMessage("Operação bloqueada. O número de produtos cadastrados excede número máximo permitido.");
        }
    };

    this.moveRowProduto = function(widget) {
        if (widget.mode == 'list') {

            var CDGRUPPROD = widget.getParent().currentRow.CDGRUPPROD;
            var CDSUBGRPROD = widget.getParent().currentRow.CDSUBGRPROD;
            var CDFILISUBGR = widget.getParent().currentRow.CDFILISUBGR;
            
            General.buildDataSourceFilter(widget.dataSourceFilter, {
                CDGRUPPROD: CDGRUPPROD,
                CDSUBGRPROD: CDSUBGRPROD,
                CDFILISUBGR: (widget.name == 'produtoSubGrupoFilial') ? CDFILISUBGR : "P"
            });
            if (widget.getParent().getParent().currentRow.IDCADNVSUBGR == 'N') {
                var filter = [{
                    "name": "CDNVPRODUTO",
                    "operator": "=",
                    "value": General.getLevelByMask('GER_CDPICTPROD', 1).length
                }];

                widget.getField('NMPRODUTO').dataSourceFilter = filter;
            }
            else {
                widget.getField('NMPRODUTO').dataSourceFilter = [];
            }
            widget.getField('NMPRODUTO').reload();
            widget.reload();
        }
    };

    this.saveGrupo = function(widget) {

        if (widget.currentRow.IDCADNVSUBGR == 'N') {
            var filter = [{
                    "name": "CDGRUPPROD",
                    "value": widget.currentRow.CDGRUPPROD,
                    "operator": "="
                },
                {
                    "name": "CDNVPRODUTO",
                    "value": General.getLevelByMask('GER_CDPICTPROD', 1).length,
                    "operator": "<"
                }

            ];
            Ger09900_AgrupamentoDeProdutos_Service.verificaProdutos(filter).then(function(callback) {
                var dataset = callback.dataset.produto_nvsubgrprod;
                if (!dataset.length) {
                    General.saveCrud(widget);
                }
                else {
                    ScreenService.showMessage("Operação bloqueada. Existe produto de nível incompleto cadastrado.");
                    widget.currentRow.IDCADNVSUBGR = 'S';
                }
            });
        }
        else {
            General.saveCrud(widget);
        }
    };

    this.saveSubGrupo = function(widget) {
        if (widget.isValid()) {
            var data = [];
            var row = widget.currentRow;
            row.CDGRUPPROD = widget.getParent().currentRow.CDGRUPPROD;
            if (widget.name != 'subGrupoFilial') {
                row.CDFILISUBGR = 'P';
            }

            data.push(row);
            Ger09900_AgrupamentoDeProdutos_Service.saveSubGrupo(data).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    this.saveProduto = function(widget) {
        if (widget.isValid()) {
            var data = [];
            var row = widget.currentRow;
            row.CDGRUPPROD = widget.getParent().getParent().currentRow.CDGRUPPROD;
            row.CDSUBGRPROD = widget.getParent().currentRow.CDSUBGRPROD;
            row.CDFILISUBGR = (widget.getParent().name == 'subGrupoFilial') ? widget.getParent().currentRow.CDFILISUBGR : 'P';
            row.CDNVPRODUTO = General.getLevelByMask('GER_CDPICTPROD', 1).length;
            data.push(row);
            Ger09900_AgrupamentoDeProdutos_Service.saveProduto(data).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    this.deleteGrupo = function(widget) {
        ScreenService.confirmMessage("O grupo de produtos selecionado será excluído. Confirma a exclusão do grupo, seus sub-grupos e todos os produtos já cadastrados?", 'question', function() {
            Ger09900_AgrupamentoDeProdutos_Service.deleteGrupo([widget.currentRow]).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.activate();
            }, function(error) {
                if(error.data.error.includes("ORA-02292")){
                    ScreenService.showMessage("Operação bloqueada! Agrupamento já associado a Implantação ou Ajuste de inventário.");
                }else{
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.deleteSubGrupo = function(widget) {
        ScreenService.confirmMessage("O sub-grupo de produtos selecionado será excluído. Confirma a exclusão do sub-grupo e todos os produtos já cadastrados?", 'question', function() {
            Ger09900_AgrupamentoDeProdutos_Service.deleteSubGrupo([widget.currentRow]).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    this.deleteProduto = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            widget.currentRow.NVMAXPRODUTO = General.getLevelByMask('GER_CDPICTPROD', 1).length;
            Ger09900_AgrupamentoDeProdutos_Service.deleteProduto([widget.currentRow]).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.activate();
            });
        });
    };
    
    this.hideWidgetsOnGrupo = function(widget){
        let widgetSubGrupo = templateManager.container.getWidget("subGrupo");
        let widgetCongUnidade = templateManager.container.getWidget("subGrupoFilial");
        
        if(widget.currentRow.__is_new){
            widgetSubGrupo.isVisible=false;
            widgetCongUnidade.isVisible=false;
        }else{
            widgetCongUnidade.isVisible=true;
            widgetSubGrupo.isVisible=true;
        }
        
    };
    
    this.openFilter = function(widget) {
        
        this.setFieldsGerais(widget);
        let rowsToSet  = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILISUBGR = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            ZhFilterPreferences.openFilterWidget(widget,rowsToSet).then(function(success) {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
            }.bind(this));
        }.bind(this));
        
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    this.getFilialLogada = function() {
    var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
    return General.callRestEngine('cadEst#/crud/filial', 'FilterData', filter);
    };
    
    this.integerFields = function(widget){
        let codigo = widget.getField('CDSUBGRPROD');
        let nrMax = widget.getField('NRMAXITSUBGR');
        
        General.justInteger('#CDSUBGRPROD');
        codigo.reload();
        General.justInteger('#NRMAXITSUBGR');
        nrMax.reload();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger09900_AgrupamentoDeProdutos_Controller', Ger09900_AgrupamentoDeProdutos_Controller);
});


// FILE: ./assets/js/controllers/Ger14600_ComposicaoDoProduto_Controller.js
function Ger14600_ComposicaoDoProduto_Controller(Ger14600_ComposicaoDoProduto_Service, ScreenService, General, templateManager, ZhFilterPreferences, ZHPromise, restEngine) {

	var mensagens = {
		Porcentagem: 'O Valor do desconto(%) deve ser um valor entre 0 e 100',
		Delete: 'Deletado com sucesso!',
		SaveInstrucao: 'Instrução salva com sucesso!',
		Quantidades: "Quantidade Mínima deve ser menor ou igual a Quantidade Máxima.",
		ProdutoObrigatorio: "O Produto Obrigatório só pode ser 'Sim' para produtos que estiver Pré-Selecionado como 'Sim'.",
		valorDesconto: "O Valor Desconto de ser maior ou igual a 0 (zero).",
		MaiorQueZero: "A Quantidade deve ser maior ou igual a 0 (zero).",
		ComposicaoRecursiva: "Não é possível adicionar o produto na sua própria composição!",
		ErroData: "A data final deve ser maior que a data inicial!",
		DuplaAlteracao: "Grupos iguais não podem ser alterados simultaneamente.",
		VigenciaInvalidaProd: "Data de vigência inválida. É possível cadastrar uma data apenas uma vez para o mesmo produto. Verifique os demais registros desse produto.",
		VigenciaInvalidaGrupo: "Data de vigência inválida. É possível cadastrar uma data apenas uma vez para o mesmo grupo. Verifique os demais registros desse grupo."
	};
	
	const PROMOCAO_COMBINADA_VALUE = '3';
	const NRCOMPPRODFLAG = 'NRCOMPPRODFLAG';

	this.openFilter = function (widget) {
		var defaultFilter = Util.clone(widget.currentRow);

		General.callRestEngine("cadEst#/crud/filial", "FilterData", [{ name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA" }]).then(function (response) {
			defaultFilter.CDFILIAL = response.dataset.filial[0].CDFILIAL;
			defaultFilter.NMFILIAL = response.dataset.filial[0].NMFILIAL;
			defaultFilter.NRCOMPPRODFLAG = 'T';
			defaultFilter.IDTIPOCOMPPR = '_0_1_2_3_4_5_6_';
			defaultFilter.NMTIPOCOMPPR = '_0_1_2_3_4_5_6_';
			ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
			ZhFilterPreferences.openFilterWidget(widget, defaultFilter);
		}).catch(() => {
			ZhFilterPreferences.openFilterWidget(widget);
		});

	};

	this.saveComposicao = function (widget) {
		var row = widget.currentRow;
		resetFilterValidations(widget);
		row.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
		row.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
		General.saveCrud(widget).then(() => {
			General.changeWidgetAcessibility(widget.widgets, true);
		});
	};

	this.deleteFilhos = function (widget) {
		// Deleta Registros filhos
		var row = widget.currentRow;
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			Ger14600_ComposicaoDoProduto_Service.deletaFilhos(row).then(function (response) {
					ScreenService.showMessage(mensagens.Delete);
					ScreenService.closeSwipe();
					widget.reload();

				},
				function (callback) {
					if (callback.data) {
						ScreenService.showMessage(callback.data.error);
					}
				});
		});

	};

	this.deleteCrudCheckedRows = function (widget) {
		var deferred = ZHPromise.defer();
		ScreenService.confirmMessage(ScreenService.i18n('Deletar registros selecionados?'), 'question', function (success) {
			widget.removeCheckedRows().then(function () {
				widget.reload();
				widget.unSelectAll();
				deferred.resolve();
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					ScreenService.showMessage('Exclusão não permitida! O(s) registro(s) possui(em) registros filhos.');
				}
				else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};
	// ----------------------PRODUTO-----------------------------//

	this.fieldOnChangeTipoFiltro = function (widget) {
		if (widget.currentRow.TIPOFILTRO === 'S') {

			widget.getField('NMPRODINI').isVisible = false;
			widget.getField('NMPRODFIN').isVisible = false;
			widget.getField('CDPRODUTO').isVisible = false;
			widget.currentRow.CDPRODUTO = 'S';
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.P_PRODFIN = '';
			widget.currentRow.NMPRODINI = '';
			widget.currentRow.P_PRODINI = '';
		}
		else if (widget.currentRow.TIPOFILTRO === 'L') {
			widget.getField('NMPRODINI').isVisible = false;
			widget.getField('NMPRODFIN').isVisible = false;
			widget.getField('CDPRODUTO').isVisible = true;
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.P_PRODFIN = '';
			widget.currentRow.NMPRODINI = '';
			widget.currentRow.P_PRODINI = '';
		}
		else if (widget.currentRow.TIPOFILTRO === 'I') {
			widget.getField('CDPRODUTO').isVisible = false;
			widget.getField('NMPRODINI').isVisible = true;
			widget.getField('NMPRODFIN').isVisible = true;
			widget.currentRow.P_PRODINI = '';
			widget.currentRow.P_PRODFIN = '';
			widget.currentRow.CDPRODUTO = 'I';
		}
		widget.reload();
	};

	this.fieldOnChangeProdutoLista = function (widget) {
		if (!widget.currentRow.__is_new) {
			// Concatena os dados do vetor de CDPRODUTO em uma string
			var vetorCDPRODUTO = widget.currentRow.CDPRODUTO;
			widget.currentRow.CDPRODUTOS = vetorCDPRODUTO.join('_');
		}
	};

	this.fieldOnChangeComposicao = function (widget) {
		// Concatena os dados do vetor de NMTIPOCOMPPR em uma string(IDTIPOCOMPPR)
		var count = 0;
		var vetorComposicao = widget.currentRow.NMTIPOCOMPPR;
		for (count = 0; count < vetorComposicao.length; count += 2) {
			if (count === 0) {
				widget.currentRow.IDTIPOCOMPPR = '_' + vetorComposicao[0] + '_';
			}
			else {
				widget.currentRow.IDTIPOCOMPPR = widget.currentRow.IDTIPOCOMPPR.concat(vetorComposicao[count] + '_');
			}
		}
	};

	this.fieldOnChangeSegmento = function (widget) {
		if (!widget.currentRow.CDSEGMER) {
			widget.currentRow.CDSEGMER = 'T';
		}
	};

	this.ajustaFiltroProduto = function (args, widget) {
		args.e.preventDefault();
		// Ajusta os valores do filtro
		var filterPopup = widget.appliedFilter.FILTER_ROW;
		var filters = [];
		var idtipocomp = false;
		widget.dataSourceFilter = widget.dataSourceFilter.map(function (row) {
			var filter = { name: '', operator: '=', value: '' };
			switch (row.name) {
			case 'NMTIPOCOMPPR':
				filter.name = 'NMTIPOCOMPPROD';
				var NMTIPOCOMPPROD = (typeof (filterPopup.NMTIPOCOMPPR) == 'object') ? filterPopup.NMTIPOCOMPPR.join('_') : '_0_1_2_3_4_5_6_';
				filter.value = NMTIPOCOMPPROD;
				filters.push(filter);
				idtipocomp = true;
				break;

			case 'CDFILIAL':
				filter.name = 'CDFILIAL';
				filter.value = filterPopup.CDFILIAL;
				filters.push(filter);
				break;

			case 'TIPOFILTRO':
				var TODOS = filterPopup.TIPOFILTRO || 'T';
				filter.value = 'T';
				filter.name = 'TODOS';
				filters.push(filter);
				break;

			case 'NRCOMPPRODFLAG':
				filter.name = 'NRCOMPPRODFLAG';
				filter.value = filterPopup.NRCOMPPRODFLAG;
				filters.push(filter);
				break;

			case 'P_PRODINI':
				filter.name = row.name;
				var P_PRODINI = filterPopup.P_PRODINI || null;
				filter.value = P_PRODINI;
				filters.push(filter);
				break;

			case 'P_PRODFIN':
				// Completa o filtro com zzz quando o tipo de filtro for intervalo
				var P_PRODFIN = filterPopup.P_PRODFIN || 'zzzzzzzzzzzzzzzzzzzz';
				if (P_PRODFIN !== 'zzzzzzzzzzzzzzzzzzzz') {
					var vetorPRODFIN;
					var mask = getLocalVar('CDPICTPROD');
					mask = mask.replace(/\./g, '');
					vetorPRODFIN = P_PRODFIN.padEnd(mask.length, 'z');

					P_PRODFIN = vetorPRODFIN.toString();
					P_PRODFIN = P_PRODFIN.replace(/\,/g, '');
				}
				filter.name = row.name;
				filter.value = P_PRODFIN;
				filters.push(filter);
				break;

			case 'CDPRODUTO':
				var CDPRODUTO = filterPopup.CDPRODUTO || 'S';
				if (CDPRODUTO !== 'S' && CDPRODUTO !== 'I') {
					filter.value = filterPopup.CDPRODUTOS;

				}
				else(filter.value = CDPRODUTO);

				filter.name = row.name;
				filters.push(filter);
				break;

			case 'CDNVPRODUTO':
				var CDNVPRODUTO = General.getLevelByMask('CDPICTPROD', 1).length;
				filter.name = row.name;
				filter.value = CDNVPRODUTO;
				filters.push(filter);
				break;

			case 'CDSEGMER':
				filter.name = row.name;
				var CDSEGMER = filterPopup.CDSEGMER || 'T';
				filter.value = CDSEGMER;
				filters.push(filter);
				break;
			}
		});
		if (!idtipocomp) {
			var filter = { name: 'NMTIPOCOMPPROD', operator: '=', value: '_0_1_2_3_4_5_6_' };
			filters.push(filter);
		}
		widget.dataSourceFilter = filters;
		widget.reload();
	};

	this.prodIni = function (widget) {
		var row = widget.currentRow;
		widget.getField('NMPRODFIN').readOnly = !row.NMPRODINI;
		widget.getField('NMPRODFIN').readOnlyOnEdit = !row.NMPRODINI;
		row.NMPRODFIN = '';
		this.prodFin(widget);
	};

	this.prodFin = function (widget) {
		var row = widget.currentRow;
		widget = widget.container.getWidget('PRODUTO');
		widget.getField('NMPRODFIN').dataSourceFilter[0].value = row.P_PRODINI;
		widget.getField('NMPRODFIN').reload();
	};

	this.saveProdutoPromocional = function (widget) {

		if (widget.isValid()) {
			widget.currentRow.CDGRUPROMOC = widget.getParent().currentRow.CDGRUPROMOC;
			General.saveCrud(widget);
		}
	};

	this.removeFilterValidations = function (widget) {
		widget.getField(NRCOMPPRODFLAG).validations = false;
	};

	var resetFilterValidations = function (widget) {
		widget.getField(NRCOMPPRODFLAG).validations = true;
	};

	// ----------------------PRODUTO-----------------------------//

	// ---------------------COMPOSIÇÃO---------------------------//

	this.reloadField = function () {
		templateManager.container.getActiveWidget().reload();
	};
	
	this.checkVigenciaAndSaveComp = function (widget, local) {
		if (widget.currentRow.DTINIVGPRODROT) {
			var rows = widget.dataSource.data.filter(f => ((!f.__is_new) && f.CDSUBPRODUTO == widget.currentRow.CDSUBPRODUTO && f.NRSEQROTFAB != widget.currentRow.NRSEQROTFAB));
			var flag = true;
			var currentVigIni = new Date(moment(widget.currentRow.DTINIVGPRODROT, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(widget.currentRow.DTFINVGPRODROT, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {
				var vigIni = new Date(moment(row.DTINIVGPRODROT, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGPRODROT, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaProd);
			} else {
				this.save(widget);
			}
		} else {
			this.save(widget);
		}
	};

	this.checkVigenciaAndSaveGrupo = function (widget) {
		if (widget.currentRow.DTINIVGGRPROMO) {
			var rows = widget.dataSource.data.filter(f => ((!f.__is_new) && f.CDGRUPROMOC == widget.currentRow.CDGRUPROMOC && f.NRSEQGRPROMO != widget.currentRow.NRSEQGRPROMO));
			var flag = true;
			var currentVigIni = new Date(moment(widget.currentRow.DTINIVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(widget.currentRow.DTFINVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {

				var vigIni = new Date(moment(row.DTINIVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaGrupo);
			} else {
				this.save(widget);
			}
		} else {
			this.save(widget);
		}
	};

	this.save = function (widget) {
		if (widget.isValid()) {
			var row = widget.currentRow;
			row.NRORG = getLocalVar('NRORG');
			var parentRow = widget.getParent().getParent().currentRow;
			
			// CDPRODUTO -> Usado para salvar um grupo de composição
			
			row.CDPRODUTO = parentRow.CDPRODUTO;
			if ((row.CDARVPROD + "").replace(/\./g, "") === (parentRow.CDARVPROD + "").replace(/\./g, "")) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			} else {
				if (!Util.isUndefinedOrNull(row.QTSUBPRODROT)) {
					row.QTSUBPRODROT += '';
					row.QTSUBPRODROT = row.QTSUBPRODROT.replace(/\./g, '');
					row.QTSUBPRODROT = row.QTSUBPRODROT.replace(',', '.');
					row.QTSUBPRODROT = parseFloat(row.QTSUBPRODROT);
				}

				// CDPRODPROMOCAO -> Usado para salvar um produto de um grupo promocional
				
				if (!Util.isUndefinedOrNull(row.CDGRUPROMOC)) {
					row.CDPRODPROMOCAO = parentRow.CDPRODUTO;
					var menor = parseFloat(row.QTPRGRUPROMIN),
						maior = parseFloat(row.QTPRGRUPPROMOC);
					if (!validaQuantidades(menor, maior)) return;
					
					///VALIDA DATA////
					
					if (widget.getField('DTFINVGGRPROMO').value()) {
						if (widget.getField('DTINIVGGRPROMO').value()) {
							if (!self.compareDates(widget.currentRow.DTINIVGGRPROMO, widget.currentRow.DTFINVGGRPROMO) &&
								widget.currentRow.DTINIVGGRPROMO !== widget.currentRow.DTFINVGGRPROMO) {
								ScreenService.showMessage(mensagens.ErroData);
							} else {
								General.saveCrud(widget);
							}
						} else {
							General.saveCrud(widget);
						}
					} else {
						General.saveCrud(widget);
					}
				} else {
					if (widget.getField('DTFINVGPRODROT').value()) {
						if (widget.getField('DTINIVGPRODROT').value()) {
							if (!self.compareDates(widget.currentRow.DTINIVGPRODROT, widget.currentRow.DTFINVGPRODROT) &&
								widget.currentRow.DTINIVGPRODROT !== widget.currentRow.DTFINVGPRODROT) {
								ScreenService.showMessage(mensagens.ErroData);
							} else {
								this.checkRecursiveAndSave(widget, row);
							}
						} else {
							this.checkRecursiveAndSave(widget, row);
						}
					} else {
						this.checkRecursiveAndSave(widget, row);
					}
				}
			}
		}
	};
	
	this.checkRecursiveAndSave = function (widget, row) {
		var parentWidget = widget.getParent().getParent();

		var requestRow = {
			'CDPRODUTOPAI': parentWidget.currentRow.CDPRODUTO,
			'CDPRODUTOFILHO': row.CDARVPROD,
			'CDFILIAL': parentWidget.appliedFilter.FILTER_ROW.CDFILIAL
		};

		General.callRestEngine('cadEst#/checkForRecursiveComposition', 'Row', requestRow).then(function (response) {
			if (!!response.dataset.isRecursiveComposition[0]) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			} else {
				General.saveCrud(widget).then(function () {
					widget.currentRow.changed = false;
				});
			}
		});	
	};

	var validaQuantidades = function (menor, maior) {
		if (maior < menor) {
			ScreenService.showMessage(mensagens.Quantidades);
			return false;
		}
		if (maior < 0 || menor < 0) {
			ScreenService.showMessage(mensagens.MaiorQueZero);
			return false;
		}
		return true;
	};

	this.updateFilter = function (widget) {
		if (!widget.currentRow.NMFILIAL) {
			templateManager.container.getWidget('INSTRUCOESLOCAL').enabled = false;
			templateManager.container.getWidget('INSTRUCOESLOCAL').enabled = false;
		}
		else {
			templateManager.container.getWidget('INSTRUCOESLOCAL').enabled = true;
			templateManager.container.getWidget('INSTRUCOESLOCAL').enabled = true;
		}
		templateManager.container.getWidget('COMPOSICAOLOCAL').reload();
		templateManager.container.getWidget('GRUPOPROMOCIONALLOCAL').reload();
		templateManager.container.getWidget('INSTRUCOESLOCAL').reload();

	};

	this.saveInstrucao = function (widget) {
		// Verifica se a tabela possui dados
		// Seta o __is_new
		// Salva o as instruções
		var cdProduto = widget.getParent().getParent().currentRow.CDPRODUTO;
		var widgetName = widget.name;
		var rota;
		if (widgetName === 'INSTRUCOES') rota = 'cadEst#/crud/descroteiro';
		else rota = 'cadEst#/crud/descrotlocal';

		var row = {
			CDPRODUTO: cdProduto,
			ROTA: rota,
			CDFILIAL: widget.getParent().currentRow.CDFILIAL
		};

		if (rota === 'cadEst#/crud/descroteiro') {
			// Salva a Instrução
			widget.currentRow.CDPRODUTO = cdProduto;
			Ger14600_ComposicaoDoProduto_Service.saveBlob(widget.currentRow).then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.edit();

					widget.reload();
					widget.computeDescriptionOrder();
					widget.view();

				}
				delete widget.currentRow.CDPRODUTO;
				ScreenService.showMessage(mensagens.SaveInstrucao);
				widget.list();
				widget.template = widget.templateToView;
				templateManager.updateTemplate();
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);

			});
		}
		else if (rota === 'cadEst#/crud/descrotlocal') {
			// Salva a Instrução
			widget.currentRow.CDFILIAL = templateManager.container.getWidget('TABBEDCOMPOSICAOLOCAL').currentRow.CDFILIAL;
			widget.currentRow.CDPRODUTO = cdProduto;
			Ger14600_ComposicaoDoProduto_Service.saveBlob(widget.currentRow).then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.reload();
					widget.computeDescriptionOrder();
					widget.view();
					delete widget.currentRow.CDPRODUTO;
				}
				widget.list();
				widget.template = widget.templateToView;
				templateManager.updateTemplate();
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);

			});
		}
	};

	this.filtraInstrucoes = function (widget) {

		var widgetToFilter = templateManager.container.getWidget('INSTRUCOES');
		var widgetToFilter2 = templateManager.container.getWidget('INSTRUCOESLOCAL');

		Ger14600_ComposicaoDoProduto_Service.setDataSourceFilter(widget, widgetToFilter);
		Ger14600_ComposicaoDoProduto_Service.setDataSourceFilter(widget, widgetToFilter2);

	};

	this.setFormInstrucoes = function (widget) {
		// Coloca no form os dados do dataSource
		if (widget.name === 'INSTRUCOES') {
			if (widget.dataSource.data[0]) widget.currentRow.TXROTFAB = widget.dataSource.data[0].TXROTFAB;
			else widget.currentRow.TXROTFAB = '';
		}
		else {
			if (widget.dataSource.data[0]) widget.currentRow.TXROTFABLOC = widget.dataSource.data[0].TXROTFABLOC;
			else widget.currentRow.TXROTFABLOC = '';
		}
	};

	//---------------GRUPO PROMOCIONAL---------------------//
	this.setValidationValuePadrao = function (widget) {
		// Torna o campo de valor desconto obrigatorio quando IDPERVALORDES tem valor e deixa indisponivel o campo valor quando os campos desconto/acrecimo e valor/percentual nao estao selecionados alem de limpar o campo valor nesses casos

		if (widget.currentRow.IDPERVALORDES || widget.currentRow.IDDESCACRPROMO) {
			widget.getField('VRDESPRODPROMOC').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDDESCACRPROMO').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDOBRPRODSELEC').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDPERVALORDES').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('VRDESPRODPROMOC').readOnly = false;
		}
		else {
			widget.getField('VRDESPRODPROMOC').readOnly = true;
			delete widget.currentRow.$error;
			widget.currentRow.VRDESPRODPROMOC = null;
			widget.getField('VRDESPRODPROMOC').validations = {};
			widget.getField('IDDESCACRPROMO').validations = {};
			widget.getField('IDPERVALORDES').validations = {};
			widget.getField('IDOBRPRODSELEC').validations = {};
			templateManager.updateTemplate();






		}

	};

	this.setValidationValue = function (widget) {
		// Torna o campo de valor desconto obrigatorio quando IDPERVALORDES tem valor e deixa indisponivel o campo valor quando os campos desconto/acrecimo e valor/percentual nao estao selecionados alem de limpar o campo valor nesses casos
		if (widget.currentRow.IDPERVALDESFIL || widget.currentRow.IDDESCACRPROFIL) {
			widget.getField('VRDESPRPROMOFIL').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDDESCACRPROFIL').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDOBRPRODSELFIL').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};
			widget.getField('IDPERVALDESFIL').validations = {
				"required": {
					"message": "Campo Obrigatorio"

				}
			};

			widget.getField('VRDESPRPROMOFIL').readOnly = false;
		}
		else {
			widget.getField('VRDESPRPROMOFIL').readOnly = true;
			widget.currentRow.VRDESPRPROMOFIL = null;
			delete widget.currentRow.$error;
			widget.getField('VRDESPRPROMOFIL').validations = {};
			widget.getField('IDDESCACRPROFIL').validations = {};
			widget.getField('IDPERVALDESFIL').validations = {};
			widget.getField('IDOBRPRODSELFIL').validations = {};
			templateManager.updateTemplate();
		}
	};

	this.fieldOnChangeVrDesconto = function (widget) {
		// Verifica se o valor desconto esta entre 0 e 100 se IDPERVALORDES for porcentagem
		if (widget.currentRow.IDPERVALORDES === 'P' &&
			(parseInt(widget.currentRow.VRDESPRODPROMOC) >= 100 || parseInt(widget.currentRow.VRDESPRODPROMOC) <= 0)) {
			ScreenService.showMessage(mensagens.Porcentagem);
			widget.currentRow.VRDESPRODPROMOC = '';
		}
	};

	this.saveProdutoPromocionalComposicao = function (widget) {
		// Salva um produto promocional dentro da aba Composição
		var validProdPrinMax = self.validProdPrinMax(widget.currentRow);
		if (widget.isValid() && validProdPrinMax) {
			if (!validacoesProdutoPromocionalComposicao(widget.currentRow)) return;
			//	widget.currentRow.__is_new = Util.isUndefinedOrNull(widget.currentRow.CDPRODPROMOCAO);
			//	widget.currentRow.changed = !widget.currentRow.__is_new;
			widget.currentRow.CDGRUPROMOC = widget.getParent().currentRow.CDGRUPROMOC;
			widget.currentRow.NRSEQGRPROMO =
				Util.isUndefinedOrNull(widget.currentRow.NRSEQGRPROMO) ? widget.getParent().currentRow.NRSEQGRPROMO : widget.currentRow.NRSEQGRPROMO;
			widget.currentRow.IDPERVALORDES =
				Util.isUndefinedOrNull(widget.currentRow.IDPERVALORDES) ? 'P' : widget.currentRow.IDPERVALORDES;
			widget.currentRow.CDPRODPROMOCAO = widget.getParent().getParent().getParent().currentRow.CDPRODUTO;
			General.saveCrud(widget).then(function () {
				if (!widget.currentRow.__is_new) {
					ScreenService.closeSwipe();
				}
				widget.reload().then(function (response) {
					self.validProdPrinMin(response.dataset, true);
				});
			});
		}

	};

	this.validProdPrinMax = function (row) {
		var retorno = true;
		if (row.QTPRGRUPPROMOC == row.IDPRINCPRODQT && row.IDPRINCPROD == 'S' && row.IDPRINCPROD != row.IDPRINCPRODBKP) {
			ScreenService.showMessage('Quantidade de produtos principais superior a quantidade de produtos maxima.');
			retorno = false;
		}
		return retorno;
	};

	this.validProdPrinMin = function (dataset, padrao) {
		var datasourceName = padrao ? 'desgrupromocpr_prodgrupromoc_produto' : 'desgrupromprfil_prodgrupromoc_produto';

		if (dataset[datasourceName].length > 0) {
			if (dataset[datasourceName][0].QTPRGRUPROMIN > dataset[datasourceName][0].IDPRINCPRODQT)
				ScreenService.showMessage('Quantidade de produtos principais inferior a quantidade de produtos minima.');
		}
	};

	var validacoesProdutoPromocionalComposicao = function (row) {
		if (row.IDPRODPRESELEC == 'N' && row.IDOBRPRODSELEC == 'S') {
			ScreenService.showMessage(mensagens.ProdutoObrigatorio);
			return false;
		}
		if (parseFloat(row.VRDESPRODPROMOC) < 0) {
			ScreenService.showMessage(mensagens.valorDesconto);
			return false;
		}
		return true;
	};

	this.validateImprCupomField = function (widget, fieldName) {
		const parentWidget = widget.getParent().getParent();
		if (parentWidget.currentRow.IDTIPOCOMPPROD === PROMOCAO_COMBINADA_VALUE &&
			parentWidget.currentRow.IDIMPPRODUTO === '2') {
			widget.getField(fieldName).readOnly = true;
		}
		else {
			widget.getField(fieldName).readOnly = false;
		}
	};

	//---------------GRUPO PROMOCIONAL---------------------//

	//------------------INSTRUÇÕES-------------------------//

	//------------------INSTRUÇÕES-------------------------//

	//----------------COMPOSIÇÃO LOCAL---------------------//

	this.OnEnterCompLocal = function (widget) {
		widget.currentRow.CDFILIAL = templateManager.container.getWidget('PRODUTO').appliedFilter.FILTER_ROW.CDFILIAL;
		widget.currentRow.NMFILIAL = templateManager.container.getWidget('PRODUTO').appliedFilter.FILTER_ROW.NMFILIAL;
	};
	
	this.checkVigenciaAndSaveCompLocal = function (widget) {
		if (widget.currentRow.DTINIVGPRODLOC) {
			var rows = widget.dataSource.data.filter(f => ((!f.__is_new) && f.CDSUBPRODLOC == widget.currentRow.CDSUBPRODLOC && f.NRSEQROTFABLOC != widget.currentRow.NRSEQROTFABLOC));
			var flag = true;
			var currentVigIni = new Date(moment(widget.currentRow.DTINIVGPRODLOC, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(widget.currentRow.DTFINVGPRODLOC, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {

				var vigIni = new Date(moment(row.DTINIVGPRODLOC, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGPRODLOC, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaProd);
			} else {
				this.saveLocal(widget);
			}
		} else {
			this.saveLocal(widget);
		}
	};
	
	this.checkVigenciaAndSaveGrupoLocal = function (widget) {
		if (widget.currentRow.DTINIVGGRPROFIL) {
			var rows = widget.dataSource.data.filter(f => ((!f.__is_new) && f.CDGRUPROMOC == widget.currentRow.CDGRUPROMOC && f.NRSEQGRPROMOFIL != widget.currentRow.NRSEQGRPROMOFIL));
			var flag = true;
			var currentVigIni = new Date(moment(widget.currentRow.DTINIVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(widget.currentRow.DTFINVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {

				var vigIni = new Date(moment(row.DTINIVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaGrupo);
			} else {
				this.saveLocal(widget);
			}
		} else {
			this.saveLocal(widget);
		}
	};

	this.saveLocal = function (widget) {
		// Usado para salvar um grupo promocional de composição local
		if (widget.isValid()) {
			var row = widget.currentRow;
			row.NRORG = getLocalVar('NRORG');
			var parentRow = widget.parent.parent.parent.currentRow;

			row.CDFILIAL = templateManager.container.getWidget('TABBEDCOMPOSICAOLOCAL').currentRow.CDFILIAL;
			row.CDPRODUTO = parentRow.CDPRODUTO;

			if ((row.CDARVPROD + "").replace(/\./g, "") === (parentRow.CDARVPROD + "").replace(/\./g, "")) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			} else {
				if (!Util.isUndefinedOrNull(row.QTSUBPRODROT)) {
					row.QTSUBPRODROT += '';
					row.QTSUBPRODROT = row.QTSUBPRODROT.replace(/\./g, '');
					row.QTSUBPRODROT = row.QTSUBPRODROT.replace(',', '.');
					row.QTSUBPRODROT = parseFloat(row.QTSUBPRODROT);
				}
				
				// CDPRODPROMOCAO -> Usado para salvar um produto de um grupo promocional
				
				if (!Util.isUndefinedOrNull(row.CDGRUPROMOC)) {
					row.CDPRODPROMOCAO = parentRow.CDPRODUTO;
					var menor = parseFloat(row.QTPRGRUPROMINFIL),
						maior = parseFloat(row.QTPRGRUPPROMFIL);
					if (!validaQuantidades(menor, maior)) return;
					
					///VALIDA DATA///
					
					if (widget.getField('DTFINVGGRPROFIL').value()) {
						if (widget.getField('DTINIVGGRPROFIL').value()) {
							if (!self.compareDates(widget.currentRow.DTINIVGGRPROFIL, widget.currentRow.DTFINVGGRPROFIL) &&
								widget.currentRow.DTINIVGGRPROFIL !== widget.currentRow.DTFINVGGRPROFIL) {
								ScreenService.showMessage(mensagens.ErroData);
							} else {
								General.saveCrud(widget);
							}
						} else {
							General.saveCrud(widget);
						}
					} else {
						General.saveCrud(widget);
					}
				} else {
					if (widget.getField('DTFINVGPRODLOC').value()) {
						if (widget.getField('DTINIVGPRODLOC').value()) {
							if (!self.compareDates(widget.currentRow.DTINIVGPRODLOC, widget.currentRow.DTFINVGPRODLOC) &&
								widget.currentRow.DTINIVGPRODLOC !== widget.currentRow.DTFINVGPRODLOC) {
								ScreenService.showMessage(mensagens.ErroData);
							} else {
								this.checkRecursiveAndSaveLocal(widget, row);
							}
						} else {
							this.checkRecursiveAndSaveLocal(widget, row);
						}
					} else {
						this.checkRecursiveAndSaveLocal(widget, row);
					}
				}
			}
		}
	};
	
	this.checkRecursiveAndSaveLocal = function (widget, row) {
		var requestRow = {
			'CDPRODUTOPAI': row.CDPRODUTO,
			'CDPRODUTOFILHO': row.CDARVPROD,
			'CDFILIAL': row.CDFILIAL
		};

		General.callRestEngine('cadEst#/checkForRecursiveComposition', 'Row', requestRow).then(function (response) {
			if (!!response.dataset.isRecursiveComposition[0]) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			} else {
				General.saveCrud(widget);
			}
		});		
	};

	this.saveProdutoPromocionalCLocal = function (widget) {
		//Salva um Produto Promocional dentro da aba de Composição local
		var validProdPrinMax = self.validProdPrinMax(widget.currentRow);
		if (widget.isValid() && validProdPrinMax) {
			if (!validacoesProdutoPromocionalComposicaoLocal(widget.currentRow)) return;
			widget.currentRow.__is_new = Util.isUndefinedOrNull(widget.currentRow.CDPRODPROMOCAO);

			widget.currentRow.NRSEQGRPROMOFIL =
				Util.isUndefinedOrNull(widget.currentRow.NRSEQGRPROMOFIL) ? widget.getParent().currentRow.NRSEQGRPROMOFIL : widget.currentRow.NRSEQGRPROMOFIL;

			widget.currentRow.IDPERVALDESFIL =
				Util.isUndefinedOrNull(widget.currentRow.IDPERVALDESFIL) ? 'P' : widget.currentRow.IDPERVALDESFIL;
			widget.currentRow.IDOBRPRODSELFIL =
				Util.isUndefinedOrNull(widget.currentRow.IDOBRPRODSELFIL) ? 'N' : widget.currentRow.IDOBRPRODSELFIL;
			widget.currentRow.VRDESPRPROMOFIL =
				Util.isUndefinedOrNull(widget.currentRow.VRDESPRPROMOFIL) ? 0 : widget.currentRow.VRDESPRPROMOFIL;
			widget.currentRow.CDPRODPROMOCAO = widget.getParent().getParent().getParent().currentRow.CDPRODUTO;
			widget.currentRow.CDFILIAL = templateManager.container.getWidget('TABBEDCOMPOSICAOLOCAL').currentRow.CDFILIAL;
			General.saveCrud(widget).then(function () {
				if (!widget.currentRow.__is_new) {
					ScreenService.closeSwipe();
				}
				widget.reload().then(function (response) {
					self.validProdPrinMin(response.dataset, false);
				});
			});
		}
	};

	var validacoesProdutoPromocionalComposicaoLocal = function (row) {
		if (row.IDPRODPRESELFIL == 'N' && row.IDOBRPRODSELFIL == 'S') {
			ScreenService.showMessage(mensagens.ProdutoObrigatorio);
			return false;
		}
		if (parseFloat(row.VRDESPRPROMOFIL) < 0) {
			ScreenService.showMessage(mensagens.valorDesconto);
			return false;
		}
		return true;
	};

	//----------------COMPOSIÇÃO LOCAL---------------------//

	// ---------------------COMPOSIÇÃO---------------------------//

	// -------------------ITENS ADICIONAIS-----------------------//

	this.saveItem = function (widget) {
		//Salva um Item adicional
		if (widget.isValid) {
			widget.currentRow.CDPRODUTO = widget.getParent().currentRow.CDPRODUTO;
			General.saveCrud(widget);
		}
	};

	// -------------------ITENS ADICIONAIS-----------------------//


	this.openPopup = function (type) {
		var wdgtprod = templateManager.container.getWidget('PRODUTO');
		var filter = wdgtprod.appliedFilter;
		var popup = templateManager.container.getWidget('popupImportComp');
		popup.getField('CDPRODUTODEST').dataSourceFilter = wdgtprod.dataSourceFilter;
		popup.getField('NMPRODUTOORIG').dataSourceFilter = wdgtprod.dataSourceFilter;
		popup.getField('CDPRODUTODEST').value('');
		popup.getField('NMPRODUTOORIG').value('');
		popup.getField('NMFILIALORIG').value('');
		popup.getField('IMPORTLOCAL').value('N');
		popup.getField('IMPORTPADRAO').value('S');
		popup.getField('IDIMPORTAGRUPOLOCAL').value('N');
		popup.getField('IDIMPORTAGRUPOPADRAO').value('S');
		popup.getField('NMFILIALORIG').readOnly = true;
		popup.getField('NMFILIALORIG').validations = {};
		popup.getField('NMFILIAL').readOnly = true;
		popup.getField('NMFILIAL').validations = {};
		ScreenService.openPopup(popup);
	};

	this.changeIsNew = function (widget) {
		widget.currentRow.__is_new = false;
	};

	var self = this;

	this.importarComposicao = function (widget) {
		var NAMEWIDGET = (!widget.getField('NMFILIAL').readOnly ? 'COMPOSICAOLOCAL' : 'COMPOSICAOPADRAO');
		var wdgComp = templateManager.container.getWidget(NAMEWIDGET);
		var widgetPai = templateManager.container.getWidget('PRODUTO');
		var row = widget.currentRow;

		widgetPai.currentRow.CDFILIAL = widgetPai.dataSourceFilter[0].value;
		row.NRORG = getLocalVar('NRORG');
		if (widget.isValid()) {
			var possuiComposicaoRecursiva = false;

			// Forced array conversions
			var produtosDestino = [].concat(row.CDPRODUTODEST);

			produtosDestino.forEach(prod => {
				possuiComposicaoRecursiva = possuiComposicaoRecursiva || row.CDPRODUTOORIG === prod;
			});

			if (possuiComposicaoRecursiva) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			}
			else {
				ScreenService.confirmMessage(ScreenService.i18n('Ao importar, a composição e os grupos promocionais atuais do produto destino serão sobrescritos. Confirma a operação?'), 'question', function (success) {
					var objectParam = {
						"requestType": "Row",
						"row": {
							"from": widget.currentRow
						}
					};

					var requestRow = {
						'CDPRODUTOPAI': row.CDPRODUTOORIG,
						'CDPRODUTOFILHO': produtosDestino,
						'CDFILIAL': row.CDFILIAL
					};

					General.callRestEngine('cadEst#/checkForRecursiveComposition', 'Row', requestRow).then(function (response) {
						if (!!response.dataset.isRecursiveComposition[0]) {
							ScreenService.showMessage(mensagens.ComposicaoRecursiva);
						}
						else {
							return restEngine.post(
								"cadEst#/importarComposicao", objectParam,
								undefined, undefined, undefined, undefined, 360000
							).then(function (response) {
								if (response.messages) {
									if (response.messages[1].message == 'LOCAL' && widget.currentRow.IMPORTPADRAO == 'S') {
										ScreenService.confirmMessage(ScreenService.i18n(response.messages[0].message + ' Deseja continuar?'), 'question', function (success) {
											widget.currentRow.IMPORTLOCAL = 'N';
											self.onChangeIMPORTLOCAL(widget.getField('IMPORTLOCAL'));
											self.importarComposicao(widget);
										});
									}
									else if (response.messages[1].message == 'PADRAO' && widget.currentRow.IMPORTLOCAL == 'S') {
										ScreenService.confirmMessage(ScreenService.i18n(response.messages[0].message + ' Deseja continuar?'), 'question', function (success) {
											widget.currentRow.IMPORTPADRAO = 'N';
											self.importarComposicao(widget);
										});
									}
									else if (response.messages[1]) {
										ScreenService.showMessage(response.messages[0].message, 'warning');
									}
								}
								else {
									ScreenService.closePopup();
									ScreenService.successNotification('Importação realizada com sucesso!');
									wdgComp.reload();
									widgetPai.reload();
								}
							});
						}
					});
				});
			}
		}
	};

	this.updateProdutos = function (args) {
		var field = args.owner.field;
		var widget = field.widget;
		widget.currentRow.CDPRODUTO = null;
		widget.currentRow.NMPRODUTO = null;
		widget.getField('NMPRODUTO').dataSourceFilter[0].value = widget.currentRow.CDFILIAL;
		widget.getField('NMPRODUTO').reload();
	};

	this.onChangeIMPORTLOCAL = function (field) {
		var wdgtprod = templateManager.container.getWidget('PRODUTO');
		var filter = wdgtprod.appliedFilter;
		var popup = templateManager.container.getWidget('popupImportComp');
		popup.currentRow.CDFILIAL = filter.FILTER_ROW.CDFILIAL;
		popup.currentRow.NMFILIAL = filter.FILTER_ROW.NMFILIAL;

		if (field.widget.getField('IDIMPORTAGRUPOLOCAL').value() == 'S' || field.widget.getField('IMPORTLOCAL').value() == 'S') {
			field.widget.getField('NMFILIALORIG').readOnly = false;
			field.widget.getField('NMFILIALORIG').validations = { 'required': true };
			field.widget.getField('NMFILIAL').readOnly = false;
			field.widget.getField('NMFILIAL').validations = { 'required': true };
		}
		else {
			field.widget.getField('NMFILIALORIG').readOnly = true;
			field.widget.getField('NMFILIALORIG').validations = {};
			field.widget.getField('NMFILIAL').readOnly = true;
			field.widget.getField('NMFILIAL').validations = {};
		}
		templateManager.updateTemplate();
	};

	this.getFieldValue = function (fieldName, widgetName) {
		if (templateManager.container) {
			var widget = templateManager.container.getWidget(widgetName);
			if (widget) {
				return widget.currentRow[fieldName];
			}
			else return null;
		}
		else return null;
	};

	this.exportarComposicao = function (namewidget) {
		var wdgtprod = templateManager.container.getWidget('PRODUTO');
		var filter = wdgtprod.appliedFilter;
		if (filter !== undefined) {
			var popup = templateManager.container.getWidget(namewidget);
			popup.getField('NMPRODUTODEST').dataSourceFilter = wdgtprod.dataSourceFilter;
			popup.getField('NMPRODUTODEST').value('');
			popup.getField('NMFILIALDEST').value('');
			popup.getField('EXPORTAR').value(['S','SC']);
			ScreenService.openPopup(popup);
		}
		else {
			ScreenService.showMessage('Favor realizar o filtro antes de exportar.');
		}
	};

	this.exportLocToLoc = function (widget) {
		var widgetPai = templateManager.container.getWidget('PRODUTO');
		var row = widget.currentRow;
		row.NRORG = getLocalVar('NRORG');

		if (widget.isValid()) {
			ScreenService.confirmMessage(ScreenService.i18n('Ao exportar, todos os itens da composição local e seus grupos promocionais locais serão sobrescritos. Confirma a operação?'), 'question', function (success) {
				var objectParam = {
					"requestType": "Row",
					"row": {
						"from": widget.currentRow
					}
				};

				/*var requestRow = {
					'CDFILIALORIG': row.CDFILIALORIG,
					'CDFILIAL': row.CDFILIALDEST,
					'CDPRODUTOS': row.CDPRODUTODEST,
					'IDEXPORTGRUPROMO': row.IDEXPORTGRUPROMO,
					'NRORG': row.NRORG
				};*/

				return restEngine.post("cadEst#/exportCompLocLoc", objectParam, undefined, undefined, undefined, undefined, 360000)
					.then(function () {
						ScreenService.closePopup();
						ScreenService.showMessage('Exportação realizada com sucesso!');
						widgetPai.reload();
					});
			});
		}
	};

	this.exportPadrToLoc = function (widget) {
		var widgetPai = templateManager.container.getWidget('PRODUTO');
		var row = widget.currentRow;
		row.NRORG = getLocalVar('NRORG');

		if (widget.isValid()) {
			ScreenService.confirmMessage(ScreenService.i18n('Ao exportar, todos os itens da composição local e seus grupos promocionais locais serão sobrescritos. Confirma a operação?'), 'question', function (success) {
				var objectParam = {
					"requestType": "Row",
					"row": {
						"from": widget.currentRow
					}
				};

				var requestRow = {
					'CDFILIAL': row.CDFILIALDEST,
					'CDPRODUTOS': row.CDPRODUTODEST,
					'NRORG': row.NRORG
				};

				return restEngine.post("cadEst#/exportCompPadrLoc", objectParam, undefined, undefined, undefined, undefined, 360000)
					.then(function (response) {
						if(response.messages) {
							ScreenService.showMessage(response.messages[0].message, 'warning');
						} else {
							ScreenService.closePopup();
							ScreenService.showMessage('Exportação realizada com sucesso!');
							widgetPai.reload();	
						}
					});
			});
		}
	};

	this.exportLocToPadr = function (widget) {
		var widgetPai = templateManager.container.getWidget('PRODUTO');
		var row = widget.currentRow;
		row.NRORG = getLocalVar('NRORG');

		if (widget.isValid()) {
			ScreenService.confirmMessage(ScreenService.i18n('Ao exportar, todos os itens da composição Padrão e seus grupos promocionais padrões serão sobrescritos. Confirma a operação?'), 'question', function (success) {
				var objectParam = {
					"requestType": "Row",
					"row": {
						"from": widget.currentRow
					}
				};

				var requestRow = {
					'CDFILIAL': row.CDFILIAL,
					'CDPRODUTOS': row.CDPRODUTODEST,
					'IDEXPORTGRUPROMO': row.IDEXPORTGRUPROMO,
					'NRORG': row.NRORG
				};

				return restEngine.post("cadEst#/exportCompLocPadr", objectParam, undefined, undefined, undefined, undefined, 360000)
					.then(function () {
						ScreenService.closePopup();
						ScreenService.showMessage('Exportação realizada com sucesso!');
						widgetPai.reload();
					});
			});
		}
	};

	this.onEnterProdutoGrupoPromo = function (args) {
		var widget = args.owner;
		widget.dataSourceFilter = [{
				"name": "CDPRODPROMOCAO",
				"operator": "=",
				"value": General.getFieldValue('CDPRODUTO', 'PRODUTO')
			},
			{
				"name": "CDGRUPROMOC",
				"operator": "=",
				"value": General.getFieldValue('CDGRUPROMOC', 'GRUPOPROMOCIONAL')
			},
			{
				"name": "NRSEQGRPROMO",
				"operator": "=",
				"value": General.getFieldValue('NRSEQGRPROMO', 'GRUPOPROMOCIONAL')
			}
		];
		widget.reload();
	};

	this.editGrupoPromocionalPadrao = function (args) {
		var widget = args.owner.widget;
		args.owner.widget.currentRow.IDPRINCPRODBKP = args.owner.widget.currentRow.IDPRINCPROD;
		General.edit(args.owner.widget);

		this.setValidationValuePadrao(widget);
		if (widget.currentRow.IDPERVALORDES == null || widget.currentRow.IDDESCACRPROMO == null) {
			widget.currentRow.__is_new = true;
		}
		else {
			widget.currentRow.__is_new = false;
		}

	};

	this.editGrupoPromocionalLocal = function (args) {
		var widget = args.owner.widget;
		args.owner.widget.currentRow.IDPRINCPRODBKP = args.owner.widget.currentRow.IDPRINCPROD;
		General.edit(args.owner.widget);

		this.setValidationValue(widget);
		if (widget.currentRow.IDDESCACRPROFIL == null ||
			widget.currentRow.IDPERVALDESFIL == null) {
			widget.currentRow.__is_new = false;
		}
		else {
			widget.currentRow.__is_new = true;
		}

	};

	var requestModulesMetadata = function () {
		var modulesPackage = window.modulesPackage || "modules";
		return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
			return response.data;
		});
	};
	this.voltarDashboard = function () {
		this.openWindow('product#dashboard');
	};

	this.openWindow = function (windowName, newWindow) {
		requestModulesMetadata().then(function (modules) {
			var subModule = windowName.split('#')[0];
			var module = subModule.split('_')[0];
			var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
			if (ScreenService.openLink) {
				ScreenService.openLink(url, newWindow);
			}
			else {
				ScreenService.openWindow(windowName);
			}
		});
	};


	this.tornarVisivelCobrarItem = function (widget) {
		widget.getField("IDCOBRAADIC").isVisible = (widget.getField('IDMODOVENDA').value() === 'TGO');
		templateManager.updateTemplate();
		if (widget.getField('IDMODOVENDA').value() !== 'TGO') {
			widget.getField('IDCOBRAADIC').value('S');
		}
	};

	this.setDatafinLocalObrigatoria = function (widget) {
		if (widget.currentRow.DTINIVGGRPROFIL) {
			widget.getField('DTFINVGGRPROFIL').validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
		}
		else {
			widget.getField('DTFINVGGRPROFIL').validations = {};
		}
		templateManager.updateTemplate();
	};
	
	this.setDatafinObrigatoria = function (widget) {
		if (widget.currentRow.DTINIVGGRPROMO) {
			widget.getField('DTFINVGGRPROMO').validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
		}
		else {
			widget.getField('DTFINVGGRPROMO').validations = {};
		}
		templateManager.updateTemplate();
	};
	
	this.setDatafinObrigatoriaComp = function (widget) {
		if (widget.currentRow.DTINIVGPRODROT) {
			widget.getField('DTFINVGPRODROT').validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
		}
		else {
			widget.getField('DTFINVGPRODROT').validations = {};
		}
		templateManager.updateTemplate();
	};
	
	this.setDatafinObrigatoriaCompLocal = function (widget) {
		if (widget.currentRow.DTINIVGPRODLOC) {
			widget.getField('DTFINVGPRODLOC').validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
		}
		else {
			widget.getField('DTFINVGPRODLOC').validations = {};
		}
		templateManager.updateTemplate();
	};

	this.validateDateLocal = function (field) {
		var widget = field.widget;

		if (!self.compareDates(widget.currentRow.DTINIVGGRPROFIL, widget.currentRow.DTFINVGGRPROFIL) || widget.currentRow.DTINIVGGRPROFIL == widget.currentRow.DTFINVGGRPROFIL) {
			ScreenService.showMessage(mensagens.ErroData);
			field.clearValue(field);
		}
	};

	this.compareDates = function (minorDate, majorDate) {
		var small = new Date(moment(minorDate, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
		var big = new Date(moment(majorDate, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
		return (small <= big);
	};


	this.addProdInGroupPromo = function (args) {
		var widget = args.owner.widget;
		widget.getField('NMPRODUTO').readOnly = false;
		widget.currentRow.CDGRUPROMOC = widget.getParent().currentRow.CDGRUPROMOC;
		widget.newRow();
	};
	///CADASTRO AUTOMATICO LOCAL INICIO/////
	this.openPopupAlteracaoAutomaticaLocal = function () {
		var widgetAlteracao = templateManager.container.getWidget('alteracaoAutomaticaLocalPrecosPopup');
		widgetAlteracao.currentRow = {};
		var row = widgetAlteracao.currentRow;

		widgetAlteracao.list();

		row.CDFILIAL = General.getFieldValue('CDFILIAL', 'TABBEDCOMPOSICAOLOCAL');
		row.IDAPLICADESCFIL = 'N';
		row.IDIMPGRPROMOFIL = 'N';
		self.setDatafinLocalObrigatoria(widgetAlteracao);
		ScreenService.openPopup(widgetAlteracao);
	};
	///CADASTRO AUTOMATICO LOCAL FIM/////

	///CADASTRO AUTOMATICO INICIO/////
	this.openPopupAlteracaoAutomatica = function () {
		var widgetAlteracao = templateManager.container.getWidget('alteracaoAutomaticaPrecosPopup');
		widgetAlteracao.currentRow = {};
		var row = widgetAlteracao.currentRow;

		widgetAlteracao.list();

		row.CDFILIAL = General.getFieldValue('CDFILIAL', 'TABBEDCOMPOSICAOLOCAL');
		row.IDAPLICADESC = 'N';
		row.IDIMPGRPROMO = 'N';

		self.setDatafinObrigatoria(widgetAlteracao);
		ScreenService.openPopup(widgetAlteracao);
	};

	this.setProdFinalPopup = function (args, fieldProdFin) {
		var field = args.owner.field;
		var row = field.widget.currentRow;
		var fieldFinal = field.widget.getField(fieldProdFin);

		fieldFinal.validations = { required: true };
		fieldFinal.readOnly = false;

		row[fieldProdFin] = row.NMPRODUTOINI;
		row.CDPRODUTOFIN = field.widget.currentRow.CDPRODUTOINI;
		row.CDARVPRODFIN = field.widget.currentRow.CDARVPRODINI;

		if (field.value()) {
			fieldFinal.readOnly = false;

			self.verificaFiltro(fieldFinal);

			fieldFinal.dataSourceFilter[6] = {
				"name": "CDARVPROD",
				"operator": ">=",
				"value": row.CDARVPRODINI
			};

			field.validations = { required: true };
			fieldFinal.validations = { required: true };
			fieldFinal.reload();
		}
		else {
			delete fieldFinal.validations;
			fieldFinal.readOnly = true;
		}
	};

	var saveAltAuto = function (widget) {
		const widgetGrupoPromo = widget.name === 'alteracaoAutomaticaLocalPrecosPopup' ?
			templateManager.container.getWidget('GRUPOPROMOCIONALLOCAL') :
			templateManager.container.getWidget('GRUPOPROMOCIONAL');

		General.saveCrud(widget).then(
			function () {
				widget.currentRow.changed = false;
				widget.currentRow.__is_new = false;
				widget.currentRow = {};
				ScreenService.closePopup();
				ScreenService.closePopup();
				widgetGrupoPromo.activate();
				widgetGrupoPromo.reload().then(function () {
					self.gpPromoOnEnter(widgetGrupoPromo);
				});
			});
	};
	
	this.checkVigenciaAndSaveAuto = function (CDGRUPROMOC, NRSEQGRPROMO, DTINIVGGRPROMO, DTFINVGGRPROMO) {
		const grupos = templateManager.container.getWidget('GRUPOPROMOCIONAL');
		
		if (DTINIVGGRPROMO !== null) {
			var rows = grupos.dataSource.data.filter(f => f.NRSEQGRPROMO !== NRSEQGRPROMO && f.CDGRUPROMOC == CDGRUPROMOC);
			var flag = true;
			var currentVigIni = new Date(moment(DTINIVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(DTFINVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {

				var vigIni = new Date(moment(row.DTINIVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGGRPROMO, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				return false;
			} else {
				return true;
			}
		} else {
			return true;
		}
	};

	this.AltAuto = function (widget) {
		if (widget.isValid()) {
			var row = widget.currentRow;
			var checkVigencia = true;
			
			const filteredArray = widget.currentRow.CDGRUPROMOC.filter(function(ele , pos){
			    return widget.currentRow.CDGRUPROMOC.indexOf(ele) == pos;
			});
			
			if(filteredArray.length < widget.currentRow.CDGRUPROMOC.length) {
				ScreenService.showMessage(mensagens.DuplaAlteracao);
				return false;
			}
			
			widget.currentRow.CDGRUPROMOC.forEach(function (element, key) {
				var NRSEQGRPROMO   = widget.currentRow.NRSEQGRPROMO[key];
				var DTINIVGGRPROMO = widget.currentRow.DTINIVGGRPROMO;
				var DTFINVGGRPROMO = widget.currentRow.DTFINVGGRPROMO;
				
				var flag = self.checkVigenciaAndSaveAuto(element, NRSEQGRPROMO, DTINIVGGRPROMO, DTFINVGGRPROMO);
				
				if(flag == false) {
					checkVigencia = false;
				}
			});
			
			if(!checkVigencia) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaGrupo);
				return false;
			}
			
			row.NRORG = getLocalVar('NRORG');
			var parentRow = widget.getParent().getParent().currentRow;
			// CDPRODUTO -> Usado para salvar um grupo de composição
			row.CDPRODUTO = parentRow.CDPRODUTO;

			if ((row.CDARVPROD + "").replace(/\./g, "") === (parentRow.CDARVPROD + "").replace(/\./g, "")) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			}
			else {
				// CDPRODPROMOCAO -> Usado para salvar um produto de um grupo promocional
				row.CDPRODPROMOCAO = parentRow.CDPRODUTO;

				var menor = parseFloat(row.QTPRGRUPROMINFIL),
					maior = parseFloat(row.QTPRGRUPPROMFIL);
				if (!validaQuantidades(menor, maior)) return;
				///VALIDA DATA////
				if (widget.getField('DTFINVGGRPROMO').value()) {
					if (widget.getField('DTINIVGGRPROMO').value()) {
						if (!self.compareDates(widget.currentRow.DTINIVGGRPROMO, widget.currentRow.DTFINVGGRPROMO) &&
							widget.currentRow.DTINIVGGRPROMO !== widget.currentRow.DTFINVGGRPROMO) {
							ScreenService.showMessage(mensagens.ErroData);
						}
						else {
							saveAltAuto(widget);
						}
					}
					else {
						saveAltAuto(widget);
					}
				}
				else {
					saveAltAuto(widget);
				}
			}
		}
	};
	
	this.checkVigenciaAndSaveAutoFil = function (CDGRUPROMOC, NRSEQGRPROMOFIL, DTINIVGGRPROFIL, DTFINVGGRPROFIL) {
		const grupos = templateManager.container.getWidget('GRUPOPROMOCIONAL');
		
		if (DTINIVGGRPROFIL !== null) {
			var rows = grupos.dataSource.data.filter(f => f.NRSEQGRPROMOFIL !== NRSEQGRPROMOFIL && f.CDGRUPROMOC == CDGRUPROMOC);
			var flag = true;
			var currentVigIni = new Date(moment(DTINIVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			var currentVigFin = new Date(moment(DTFINVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
			rows.forEach(row => {

				var vigIni = new Date(moment(row.DTINIVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();
				var vigFin = new Date(moment(row.DTFINVGGRPROFIL, 'DD/MM/YYYY').format('YYYY,MM,DD')).getTime();

				if (currentVigIni <= vigFin && currentVigFin >= vigIni) {
					flag = false;
				} else if (currentVigIni >= vigFin && currentVigFin <= vigIni) {
					flag = false;
				}
			});
			if (!flag) {
				return false;
			} else {
				return true;
			}
		} else {
			return true;
		}
	};

	this.AltAutoFil = function (widget) {
		if (widget.isValid()) {
			var row = widget.currentRow;
			var checkVigencia = true;
			
			const filteredArray = widget.currentRow.CDGRUPROMOC.filter(function(ele , pos){
			    return widget.currentRow.CDGRUPROMOC.indexOf(ele) == pos;
			});
			
			if(filteredArray.length < widget.currentRow.CDGRUPROMOC.length) {
				ScreenService.showMessage(mensagens.DuplaAlteracao);
				return false;
			}
			
			widget.currentRow.CDGRUPROMOC.forEach(function (element, key) {
				var NRSEQGRPROMOFIL = widget.currentRow.NRSEQGRPROMOFIL[key];
				var DTINIVGGRPROFIL = widget.currentRow.DTINIVGGRPROFIL;
				var DTFINVGGRPROFIL = widget.currentRow.DTFINVGGRPROFIL;

				var flag = self.checkVigenciaAndSaveAutoFil(element, NRSEQGRPROMOFIL, DTINIVGGRPROFIL, DTFINVGGRPROFIL);
				
				if(flag == false) {
					checkVigencia = false;
				}
			});
			
			if(!checkVigencia) {
				ScreenService.showMessage(mensagens.VigenciaInvalidaGrupo);
				return false;
			}
			
			row.NRORG = getLocalVar('NRORG');
			var parentRow = widget.getParent().getParent().currentRow;
			// CDPRODUTO -> Usado para salvar um grupo de composição
			row.CDPRODUTO = parentRow.CDPRODUTO;

			if ((row.CDARVPROD + "").replace(/\./g, "") === (parentRow.CDARVPROD + "").replace(/\./g, "")) {
				ScreenService.showMessage(mensagens.ComposicaoRecursiva);
			}
			else {
				// CDPRODPROMOCAO -> Usado para salvar um produto de um grupo promocional
				row.CDPRODPROMOCAO = parentRow.CDPRODUTO;

				var menor = parseFloat(row.QTPRGRUPROMINFIL),
					maior = parseFloat(row.QTPRGRUPPROMFIL);
				if (!validaQuantidades(menor, maior)) return;
				///VALIDA DATA////
				if (widget.getField('DTFINVGGRPROFIL').value()) {
					if (widget.getField('DTINIVGGRPROFIL').value()) {
						if (!self.compareDates(widget.currentRow.DTINIVGGRPROFIL, widget.currentRow.DTFINVGGRPROFIL) &&
							widget.currentRow.DTINIVGGRPROFIL !== widget.currentRow.DTFINVGGRPROFIL) {
							ScreenService.showMessage(mensagens.ErroData);
						}
						else {
							saveAltAuto(widget);
						}
					}
					else {
						saveAltAuto(widget);
					}
				}
				else {
					saveAltAuto(widget);
				}
			}
		}
	};
	this.checkVigIsRequired = function (field, local) {
		var DTINIVG, DTFINVG, CDGRUPRO;
		if (local) {
			DTINIVG = 'DTINIVGGRPROFIL';
			DTFINVG = 'DTFINVGGRPROFIL';
			CDGRUPRO = 'CDGRUPROMOC';
		}
		else {
			DTINIVG = 'DTINIVGGRPROMO';
			DTFINVG = 'DTFINVGGRPROMO';
			CDGRUPRO = 'CDGRUPROMOC';
		}
		if (field.currentRow.ISVIGREQUIRED) {
			field.getField(DTFINVG).validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
			field.getField(DTINIVG).validations = {
				"required": {
					"message": "Campo Obrigatorio"
				}
			};
		}
		else {
			field.getField(DTFINVG).validations = {};
			field.getField(DTINIVG).validations = {};
		}
		templateManager.updateTemplate();
	};

	////CADASTRO AUTOMATICO FIM/////
	this.gpPromoOnEnter = function (widget) {
		if (widget.dataSource.data.length === 0) {
			widget.getAction('altAutPreco').isVisible = false;
		}
		else {
			widget.getAction('altAutPreco').isVisible = true;
		}
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger14600_ComposicaoDoProduto_Controller', Ger14600_ComposicaoDoProduto_Controller);
});


// FILE: ./assets/js/controllers/QueryController.js
function QueryController(templateManager, QueryService, metaDataFactory, ScreenService, $compile){

    var self = this;
    var QUERY_SELECT = "SELECT";
    var QUERY_PROCEDURE = "PROCEDURE";
    var QUERY_TEMPLATE = "WITH A AS (%q) SELECT * FROM (SELECT ROWNUM AS ZH_ROWNUM, A.* FROM  A)  WHERE ZH_ROWNUM >= %s AND ZH_ROWNUM <= %e";
    var ENDL = "\n\n";
    var pageSize = 300;
    var pageIndex = 0;
    var hasNextPage = false;
    var loadingMore = false;
    var backupRow;


    function insertRowNumberInQuery(query, pageStart){
        query = QUERY_TEMPLATE.replace("%q", query);
        query = query.replace("%s", pageStart);
        query = query.replace("%e", pageStart + pageSize - 1);
        return query;
    }

    /**
     * Clear the breaklines
     *
     * @param {query} query to clear
     */
    function cleanQuery(query){
        query = query.replace(/(\r\n|\n|\r)/gm," ");
        return query;
    }

    function addRowNumber(query){
        var queryWidget = templateManager.container.getWidget('query');

        /*if(query.includes(';')){
            query = cleanQuery(query);
            var querys = query.split(';');
            querys.pop();
            querys.forEach(function(query, index, querys){
                if(query.toUpperCase().includes(QUERY_SELECT)){
                    query = insertRowNumberInQuery(query, (pageIndex * pageSize) + 1);
                }
            });
            query = querys;
        } else */if(query.toUpperCase().includes(QUERY_SELECT) && !query.toUpperCase().includes(QUERY_PROCEDURE)){
            query = insertRowNumberInQuery(query, (pageIndex * pageSize) + 1);
        }
        return query;
    }

    this.showResult = function(result, loadingMore){
        if(result instanceof Array){
            var resultWidget = templateManager.container.getWidget("results");
            if(!loadingMore){
                self.clearResults(resultWidget);
                for (var object in result[0]){
                    self.createField(resultWidget, object);
                }
                resultWidget.dataSource.data = result;
                hasNextPage = result.length == pageSize ? true : false;
            } else {
                hasNextPage = result.length == pageSize ? true : false;
                resultWidget.dataSource.data = resultWidget.dataSource.data.concat(result);
            }
        }

        if(!loadingMore){
            ScreenService.openSwipe(templateManager.container.getWidget("results"));
        }
    };

    this.loadMoreResults = function(){
        if(hasNextPage){
            pageIndex++;
            this.queryExecute(true);
        }
    };

    this.openParameterModal = function(parameters){
        var modal = templateManager.container.getWidget("parameterModal");
        var replace = true;

        if(parameters.length === modal.dataSource.data.length){
            replace = false;
            modal.dataSource.data.forEach(function(data,index){
                if(parameters[index] !== data.parameter){
                    replace = true;
                }
            });
        }

        if(replace){
            modal.dataSource.data = [];
            parameters.forEach(function(parameter){
                modal.dataSource.data.push(
                    {
                        parameter: parameter,
                        value: "",
                        ISNULL: false
                    }
                );
            });
        }

        ScreenService.openPopup(modal);
    };

    this.openConsoleSwipe = function(){
        ScreenService.closePopup();
        ScreenService.openSwipe(templateManager.container.getWidget("console"));
        templateManager.container.getWidget("console").activate();
    };

    this.closeResultsSwipe = function(){
        pageIndex = 0;
        ScreenService.closeSwipe().then(function(){
            templateManager.container.getWidget("query").activate();
        });
    };

    this.minifyQuery = function(){
        var queryWidget = templateManager.container.getWidget('query');
        queryWidget.getField("query").value(cleanQuery(queryWidget.getField("query").value()));
        ScreenService.closePopup();
    };

    this.hasParameters = function(query,returnParameters){
        var parameters = [];
        var isParameter = false;
        var parameter = '';
        var isString = false;

        if(query instanceof Array){
            query.forEach(function(q){
                if(self.hasParameters(q,true)){
                    parameters = parameters.concat(self.hasParameters(q,true));
                }
            });
        } else {
            for(var j = 0;j<query.length;j++){
                if((query.charAt(j) == '\'' || query.charAt(j) == '\"')){
                    isString = !isString;
                } else if(!isString){
                    if(query.charAt(j) == ' ' || query.charAt(j) == ',' || query.charAt(j) == '(' || query.charAt(j) == ')'){
                        if(isParameter){
                            parameters.push(parameter);
                        }
                        parameter = '';
                        isParameter = false;
                    } else if(isParameter){
                        parameter += query.charAt(j);
                    } else if(query.charAt(j) == ':'){
                        isParameter = true;
                    }
                }
            }
            if(isParameter){
                parameters.push(parameter);
            }
        }

        return (parameters.length && returnParameters) ? parameters : parameters.length;
    };

    function buildValueSQL(field){
        return field.ISNULL ? 'NULL' : ('\'' + field.value + '\'');
    }

    this.setParameters = function(args){
        if(backupRow.query instanceof Array){
                args.dataSource.data.forEach(function(field){
                    for(var i =0;i<backupRow.query.length;i++){
                        backupRow.query[i] = backupRow.query[i].replace(":" + field.parameter, buildValueSQL(field));
                    }
                });
        } else {
            args.dataSource.data.forEach(function(field){
                backupRow.query = backupRow.query.replace(":" + field.parameter, buildValueSQL(field));
            });
        }

        self.processQuery(backupRow);
        backupRow = undefined;
        ScreenService.closePopup();
    };

    this.logMessage = function(message){
        var log = templateManager.container.getWidget('console').getField('log');
        if(!log.value()){
            log.value(message + ENDL);
        } else log.value(log.value() + message + ENDL);
    };

    this.cleanConsole = function(){
        var log = templateManager.container.getWidget('console').getField('log');
        log.value('');
    };

    this.processQuery = function(row ,loadingMore){
        var resultWidget = templateManager.container.getWidget("results");

        try {
            QueryService.RunQuery(row).then(function(response){
                var successMessage;
                if(!response.dataset.result[0]){
                    self.logMessage(successMessage);
                    self.showResult(0);
                } else if (!response.dataset.result[0].type){
                    successMessage = getSuccessMessage(response);
                    handleNonTypedResult(response.dataset.result, loadingMore, successMessage );
                } else {
                    successMessage = getSuccessMessage(response);
                    getResultMessage(response, successMessage);
                }

            }, function(errorResponse){
                handleError(errorResponse);
            });

        } catch (err){
            ScreenService.alertNotification('Unable to execute query');
        }

        resultWidget.isVisible = true;
        resultWidget.formTemplate = "widget/grid.html";
    };

    function getSuccessMessage(response){
        var successMessage;
        var length = response.dataset.result[0].rows ? response.dataset.result[0].rows.length : response.dataset.result.length;
        successMessage = length > pageSize ? "Showing " + length + " paginated results" : "Query executed successfully, " + length + " results obtained";

        return successMessage;
    }

    function handleNonTypedResult(result, loadingMore, successMessage = "Success!"){
        if(result[0].Error){
            self.logMessage(result[0].Error);
            self.showResult(result, loadingMore);
        } else if(result[0].Mensagem){
            self.logMessage(result[0].Mensagem);
            self.showResult(result, loadingMore);
        } else {
            self.showResult(result, loadingMore);
            self.logMessage(successMessage);
        }
    }

    function getResultMessage(response, successMessage){
        for(var object in response.dataset.result){
            switch (response.dataset.result[object].type){
                case 'error':
                    self.logMessage(response.dataset.result[object].error[0].Error);
                    self.openConsoleSwipe();
                    break;
                case 'dataset':
                    self.logMessage(successMessage);
                    self.showResult(response.dataset.result[object].rows);
                    break;
                case 'message':
                    self.logMessage(response.dataset.result[object].message[0].Mensagem);
                    self.openConsoleSwipe();
                    break;
            }
        }
    }

    function handleError(errorResponse){
        var resultWidget = templateManager.container.getWidget("results");
        var response = {
            "Error": errorResponse.data.error
        };
        self.logMessage(errorResponse.data.error);
        self.openConsoleSwipe();
        self.createField(resultWidget, "Error");
        resultWidget.dataSource.data.push(response);
    }

    this.queryExecute = function(loadingMore = false){
        var queryWidget = templateManager.container.getWidget('query');
        var resultWidget = templateManager.container.getWidget("results");
        if(!loadingMore)this.clearResults(resultWidget);
        if(verifyNeededFields()){

            var query = queryWidget.currentRow.query;

            query = addRowNumber(query);

            var row = Util.clone(queryWidget.currentRow);

            /*if(self.hasParameters(query,false) && !loadingMore){
                row.query = query;
                backupRow = row;
                self.openParameterModal(self.hasParameters(query,true));
            } else {*/
                row.query = query;
                self.processQuery(row, loadingMore);
            //}
        }
    };

    function verifyNeededFields(){
        var queryWidget = templateManager.container.getWidget('query');
        var resultWidget = templateManager.container.getWidget("results");
        if (!queryWidget.currentRow.query){
            ScreenService.alertNotification("Please inform the query");
            resultWidget.isVisible = true;
            return false;
        } else {
            return true;
        }
    }

    /**
     * Creates fields to show the query results
     *
     * @param {widget} Widget where the fields will be created
     * @param {name}   Name to the field to be created
     */
    this.createField = function(widget, name){
        var field = {
            "name": ScreenService.i18n(name),
            "label": name,
            "showOnForm": true,
            "showOnView": true,
            "showOnList": true,
            "readOnly": true,
            "template": "field/text-edit.html"
        };

        metaDataFactory.factoryField(field, widget, true);
    };

    /**
     * Clear the fields created to the last query executed
     *
     * @param {widget} widget to clear
     */
    this.clearResults = function(widget){
        widget.fields = [];
        widget.dataSource.data = [];
        templateManager.container.getWidget("query").activate();
    };

    this.containsLimiter= function(query){
        return ((query.includes("FETCH FIRST")) && (query.includes("ROWS ONLY")));
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('QueryController', QueryController);
});

// FILE: ./assets/js/controllers/ger10000_segmento_de_mercadoController.js
function ger10000_segmento_de_mercadoController(ScreenService, Ger10000_SegmentoDeMercado_Service, General, templateManager) {
	this.saveSegmento = function (widget) {
		if (widget.isValid()){
			widget.saveRow().then(function (callback) {
				var currentRow = widget.currentRow;
				currentRow.__is_new = false;
				currentRow.changed = false;
				widget.reload().then(function () {
					ScreenService.closeSwipe();
					widget.list();
				});
			}, function (callback) {
				ScreenService.showMessage(callback.data.error);
			});
		}

	};

	this.delete = function (widget) {

		widget.removeCheckedRows().then(function (callback) {
			var currentRow = widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			widget.reload().then(function () {
				ScreenService.closeSwipe();
				widget.list();
			});
		}, function (callback) {
			ScreenService.showMessage("Não é possivel deletar!Existem um ou mais registros filhos.");
		});

	};
	
	this.saveProdutosSegmento = function(widget){
		var row = widget.currentRow;
		
		Ger10000_SegmentoDeMercado_Service.saveProdutosSegmento(row).then(function (callback){
			widget.currentRow.__is_new = false;
			widget.currentRow.changed = false;
			widget.reload().then(function () {
				ScreenService.closeSwipe();
				widget.list();
			});
		}, function (callback) {
			ScreenService.showMessage(callback.data.error);
		});
	};
	
	this.deleteProdutosSegmento = function(widget){
		const row = widget.currentRow;
		const afterDelete = (widget)=>{
        	widget.currentRow.__is_new = false;
			widget.currentRow.changed = false;
			ScreenService.closeSwipe();
			widget.list();
			widget.reload();
        };
        
		row.CDFILIAL = getLocalVar('CDFILIAL');
		
		ScreenService.confirmMessage(ScreenService.i18n('Deletar registro?'), 'question', function(success) {
			row.PRICECHECK = false;
            Ger10000_SegmentoDeMercado_Service.deleteProdutosSegmento(row).then(function (callback){
				afterDelete(widget);
			}, function (callback) {
				if(callback.data.error == 'Exception: Produto(s) com preço cadastrado.'){
					ScreenService.confirmMessage(ScreenService.i18n('Produto(s) com preço cadastrado. Deseja continuar?'), 'question', success=> {
						row.PRICECHECK = true;
						Ger10000_SegmentoDeMercado_Service.deleteProdutosSegmento(row).then(function (callback){
							afterDelete(widget);
						});
					});
				}else{
					ScreenService.showMessage(callback.data.error);
				}
			});
        });
	};
	
	this.updateProdAtivo = function(widget) {
        var rows = widget.dataSource.data;
        rows = rows.filter(function(row){
            return row.changed === true && !row.__is_new;
        });
        rows.forEach(function(row) {
            var rowItem = row;
            rowItem.IDPROSEGATIV = rowItem.IDPROSEGATIV_inline || rowItem.IDPROSEGATIV;
            delete rowItem.IDPROSEGATIV_inline;
        });
        Ger10000_SegmentoDeMercado_Service.updateProdAtivo(rows).then(function(response){
        	widget.list();
        	widget.reload();
        });
        
        widget.list();
    };

	this.changeWidgetAcessibility = function(widget, flag = false){
		General.changeWidgetAcessibility(widget.widgets, flag);
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('ger10000_segmento_de_mercadoController', ger10000_segmento_de_mercadoController);
});

// FILE: ./assets/js/controllers/ger01700_unidade_controller.js
function ger01700_unidade_controller(engine, Endereco_Controller, eventAggregator, General, ger01700_unidade_service, restEngine, ScreenService, MessageService, templateManager, ZHPromise, LibLoginController, localStorageEngine) {
	
	var self = this;

	/* **************************************************
	 * Procedimento que bloqueia teclas específicas do
	 * teclado de acordo com o fieldName recebido.
	 * Versão personalizada do método "justInteger" do
	 * General.
	 *
	 * @param string fieldName
	 */
	self.blockInvalidKeys = function (fieldName) {
		
		/* inactive. changed to validateDigits on keyupEvent
		var element = $("#" + fieldName);
		element.keypress(function (e) {
			if (ger01700_unidade_service.validateKeys(e.which, fieldName))
				e.preventDefault();
		});
		*/
	};
	
	self.validateDigits = function(widget) {
        let inputInscricao = widget.currentRow.MSKINSJURFILI;
        let inputTipoInscricao = widget.currentRow.IDTPIJURFILI;
        if(widget.currentRow.MSKINSJURFILI != '' && inputTipoInscricao !== "I" ){
            widget.currentRow.MSKINSJURFILI = widget.currentRow.MSKINSJURFILI.replace(/[^0-9]/g, '');
        }
        
    };
    
    this.trocaLabel = function(widget){
        let inputTipo = widget.currentRow.IDTPIJURFILI;
        let inputInscricao = widget.getField('MSKINSJURFILI');
      
        //TROCA O NOME DO LABEL DE ACORDO COM TIPO DE INSCRICAO
        if(inputTipo == "I" ){
          inputInscricao.label = 'CNPJ/CPF/RFC';
          //CASO TIPO DE INSCRIÇÃO SEJA Inscrição Internacional, OCULTA O BOTÃO CONSULTAR, POIS API N VAI FUNCIONAR EM OUTRO PAÍS.
          //inputInscricao.class = '6';
          
        }else{
            inputInscricao.label = 'CNPJ/CPF';
            //inputInscricao.class = '4';
        }
        //Definindo tamanho até 5 caracter quando tipo de inscrição seja Código genérico
        if(inputTipo == "O"){
            
            widget.getField('MSKINSJURFILI').maxlength = 5;
        }else{
            widget.getField('MSKINSJURFILI').maxlength = 20;
        }
    	widget.getField('MSKINSJURFILI').clearValue();
    };



	/* **************************************************
	 * Procedimento que ordena e espera a atualização do
	 * produto, informando o usuário do resultado da
	 * execução.
	 *
	 * @param Widget widget
	 */
	self.buildAtualizaProduto = function (widget) {
		ger01700_unidade_service.atualizaProduto(widget.currentRow.CDFILIAL).then(function (response) {
			if (response.dataset.success)
				ScreenService.showMessage(response.dataset.success.MESSAGE);
			ScreenService.closePopup();
		}, self.promiseErrorProcedure);
	};

	/* **************************************************
	 * Função que requisita e armazena na widget a
	 * validação de endereço caso seja necessário ou caso
	 * forçada por parâmetro.
	 *
	 * @param Widget widget
	 * @param boolean force
	 *
	 * @return Promise
	 */
	self.buildEndefiliValidation = function (widget, force) {
		var deferred = ZHPromise.defer();
		var unidadeWidget = widget.getParent();

		if (!widget.validation || (widget.validation.CDFILIAL != unidadeWidget.currentRow.CDFILIAL) || force) {
			widget.validation = {};

			var CDFILIAL = (widget.getParent().currentRow.__is_new) ? 'IGNORE' : widget.currentRow.CDFILIAL;

			ger01700_unidade_service.validateEndefili(CDFILIAL).then(function (response) {
				if (!widget.getParent().currentRow.__is_new) {
					widget.validation.allTypes = response.dataset.validation.ALLTYPES;
					widget.validation.usedTypes = response.dataset.validation.USEDTYPES;
					widget.validation.freeTypes = response.dataset.validation.FREETYPES;
					widget.validation.mayInsert = response.dataset.validation.MAYINSERT;
				}

				widget.validation.requireEndFields = response.dataset.validation.REQENDFIELDS;

				deferred.resolve(widget.validation);
			}, self.promiseErrorProcedure);
		} else
			deferred.resolve(widget.validation);

		return deferred;
	};

	/* **************************************************
	 * Função que monta um filtro à partir do filterName
	 * para o field recebido. Retorna true caso o filtro
	 * tenha sido criado ou alterado.
	 *
	 * @param Field field
	 * @param string filterName
	 *
	 * @return boolean
	 */
	self.buildFilter = function (field, filterName) {
		var changed = 404;
		var widget = field.widget;

		if (!field.dataSourceFilter)
			field.dataSourceFilter = [];

		field.dataSourceFilter.forEach(function (filter) {
			if (filter.name == filterName) {
				if (filter.value != widget.currentRow[filterName]) {
					filter.value = widget.currentRow[filterName];
					changed = true;
				} else
					changed = false;
			}
		});

		if (changed === 404)
			field.dataSourceFilter.push({
				name: filterName,
				operator: "=",
				value: widget.currentRow[filterName]
			});

		return changed !== false;
	};

	/* **************************************************
	 * Procedimento que requisita as validações de
	 * Paramgeral.
	 *
	 * @param Widget widget
	 *
	 * @return Promise
	 */
	self.buildParametrizacaoValidation = function (widget) {
		var deferred = ZHPromise.defer();

		if (!widget.validation || (Object.keys(widget.validation).length === 0 && widget.validation.constructor === Object)) {
			ger01700_unidade_service.validateParametrizacao().then(function (response) {
				if (!widget.validation)
					widget.validation = {};

				widget.validation.showParamTab = response.dataset.validation.SHOWPARAMTAB;

				deferred.resolve(widget.validation);
			}, self.promiseErrorProcedure);
		} else
			deferred.resolve(widget.validation);

		return deferred;
	};

	/* **************************************************
	 * Procedimento que coloca na widget atual os códigos
	 * necessários de sua widget parent. Pode receber uma
	 * string com o nome da propriedade desejada ou um
	 * array de propriedades. Permite o envio do nome da
	 * parentWidget para eventuais casos extra ordinários
	 *
	 * @param Widget currentWidget
	 * @param String|Array primaryKeys
	 * @param String parentName
	 */
	self.buildParentPrimaryKeys = function (currentWidget, primaryKeys, parentName) {
		var parentWidget = (parentName) ? currentWidget.container.getWidget(parentName) : currentWidget.getParent();

		if (Util.isString(primaryKeys))
			primaryKeys = [primaryKeys];

		primaryKeys.forEach(function (PK) {
			currentWidget.currentRow[PK] = parentWidget.currentRow[PK];
		});
	};

	/* **************************************************
	 * Procedimento que inicializa as widgets filhas da
	 * widget recebida, colocando o valueColumn em suas
	 * currentRows e em seus filtros.
	 *
	 * @param Widget widget
	 * @param string valueColumn
	 */
	self.buildSubWidgetFilters = function (widget, valueColumn) {
		widget.widgets.forEach(function (subWidget) {
			var found = 404;

			subWidget.dataSourceFilter.forEach(function (filter) {
				if (filter.name == valueColumn) {
					filter.value = widget.currentRow[valueColumn];
					found = true;
				}
			});

			if (found !== true)
				subWidget.dataSourceFilter.push({
					name: valueColumn,
					operator: "=",
					value: widget.currentRow[valueColumn]
				});

			subWidget.currentRow[valueColumn] = widget.currentRow[valueColumn];
		});
	};

	/* **************************************************
	 * Procedimento que gerencia o cancelamento do
	 * formulário da widget Unidade.
	 *
	 * @param Widget widget
	 */
	self.cancelFormUnidade = function (widget) {
		let fieldTpInscricao = widget.getField('IDTPIJURFILI');
		fieldTpInscricao.readOnly = true;
		if (widget.currentRow.__is_new)
			Endereco_Controller.cancelInsertion(widget.name);
		else
			General.widgetCancelForm(widget);
	};

	/* **************************************************
	 * Procedimento que checa os campos opcionais do
	 * endereço da filial, validando-os de acordo com os
	 * parâmetros do banco de dados.
	 *
	 * @param Widget widget
	 */
	self.checkAddressFields = function (widget) {
		var validation = widget.validation;
		var addressFieldNames = ["NMBAIRRO",
                                 "DSENDEFILISEP",
                                 "NRENDEFILISEP",
                                 "DSLOGRADOURO",
                                 "NRCEPFILI"];

		addressFieldNames.forEach(function (fieldName) {
			var field = widget.getField(fieldName);

			if (validation.requireEndFields ^ (field.validations && field.validations.required)) {
				if (field.dependencies && field.dependencies.parentFields)
					Object.keys(field.dependencies.parentFields).forEach(function (dependency) {
						if (!(validation.requireEndFields ^ field.dependencies.parentFields[dependency].require))
							field.dependencies.parentFields[dependency].require = !validation.requireEndFields;
					});

				if (!field.validations)
					field.validations = {};

				self.switchFieldRequirement(field);
			}
		});
	};

	/* **************************************************
	 * Função que retorna true se o número de um endereço
	 * for válido, ou se não existir número.
	 *
	 * @param Widget widget
	 *
	 * @return boolean
	 */
	self.checkAddressNumber = function (widget) {
		return (!widget.currentRow.NRENDEFILISEP) ? true : /^[0-9]+[a-zA-Z]?$/.test(widget.currentRow.NRENDEFILISEP);
	};

	/* **************************************************
	 * Função que retorna true se um tipo de endereço
	 * puder ser inserido.
	 *
	 * @param Widget widget
	 *
	 * @return boolean
	 */
	self.checkAddressType = function (widget) {
		var validTypes = widget.validation.freeTypes;
		var currentType = widget.currentRow.IDTPENDEFILI;

		return validTypes.indexOf(currentType) != -1;
	};

	/* **************************************************
	 * Procedimento que checa o input de CEP, validando o
	 * dado inserido e retornando os dados do endereço da
	 * API dos Correios, tratando erros e efetuando
	 * validações de acordo com o necessário.
	 *
	 * @param Field field
	 */
	self.checkCEP = function (field) {
		var widget = field.widget;
		var CEP = field.value();

		if (CEP && CEP.length == 10) {
			ger01700_unidade_service.validateCEP(CEP).then(function (response) {
				var address = response.dataset.ENDERECO;
				if(widget.currentRow.__is_new == true){
					widget.currentRow.CDPAIS = (address.CDPAIS) ? address.CDPAIS : '';
					widget.currentRow.NMPAIS = (address.NMPAIS) ? address.NMPAIS : '';
					widget.currentRow.SGESTADO = (address.SGESTADO) ? address.SGESTADO : '';
					widget.currentRow.NMESTADO = (address.NMESTADO) ? address.NMESTADO : '';
					widget.currentRow.CDMUNICIPIO = (address.CDMUNICIPIO) ? address.CDMUNICIPIO : '';
					widget.currentRow.NMMUNICIPIO = (address.NMMUNICIPIO) ? address.NMMUNICIPIO : '';
					widget.currentRow.CDBAIRRO = (address.CDBAIRRO) ? address.CDBAIRRO : '';
					widget.currentRow.NMBAIRRO = (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.NMBAIRFILI = (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.CDLOGRADOURO = (address.CDLOGRADOURO) ? address.CDLOGRADOURO : '';
					widget.currentRow.DSLOGRADOURO = (address.DSLOGRADOURO) ? address.DSLOGRADOURO : '';
					widget.currentRow.DSENDEFILISEP = (address.DSENDERECO) ? address.DSENDERECO : '';
				}else{
					widget.currentRow.CDPAIS = widget.currentRow.CDPAIS? widget.currentRow.CDPAIS : (address.CDPAIS) ? address.CDPAIS : '';
					widget.currentRow.NMPAIS = widget.currentRow.NMPAIS? widget.currentRow.NMPAIS : (address.NMPAIS) ? address.NMPAIS : '';
					widget.currentRow.SGESTADO = widget.currentRow.SGESTADO? widget.currentRow.SGESTADO : (address.SGESTADO) ? address.SGESTADO : '';
					widget.currentRow.NMESTADO = widget.currentRow.NMESTADO? widget.currentRow.NMESTADO : (address.NMESTADO) ? address.NMESTADO : '';
					widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIO? widget.currentRow.CDMUNICIPIO : (address.CDMUNICIPIO) ? address.CDMUNICIPIO : '';
					widget.currentRow.NMMUNICIPIO = widget.currentRow.NMMUNICIPIO? widget.currentRow.NMMUNICIPIO : (address.NMMUNICIPIO) ? address.NMMUNICIPIO : '';
					widget.currentRow.CDBAIRRO = widget.currentRow.CDBAIRRO? widget.currentRow.CDBAIRRO : (address.CDBAIRRO) ? address.CDBAIRRO : '';
					widget.currentRow.NMBAIRRO = widget.currentRow.NMBAIRRO? widget.currentRow.NMBAIRRO : (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.NMBAIRFILI = widget.currentRow.NMBAIRFILI? widget.currentRow.NMBAIRFILI : (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.CDLOGRADOURO = widget.currentRow.CDLOGRADOURO? widget.currentRow.CDLOGRADOURO : (address.CDLOGRADOURO) ? address.CDLOGRADOURO : '';
					widget.currentRow.DSLOGRADOURO = widget.currentRow.DSLOGRADOURO? widget.currentRow.DSLOGRADOURO : (address.DSLOGRADOURO) ? address.DSLOGRADOURO : '';
					widget.currentRow.DSENDEFILISEP = widget.currentRow.DSENDEFILISEP? widget.currentRow.DSENDEFILISEP : (address.DSENDERECO) ? address.DSENDERECO : '';
				}

				self.constructFieldFilters(widget.getField('NMESTADO'));
				self.constructFieldFilters(widget.getField('NMMUNICIPIO'));
				self.constructFieldFilters(widget.getField('NMBAIRRO'));

				self.checkFieldDependencies(widget.getField('NMPAIS'));
			}, function (error) {
				var errorMessage = 'Timeout';

				widget.currentRow.NRCEPFILI = '';

				if (error.data)
					errorMessage = error.data.error;
				
				// ScreenService.showMessage("Erro na requisição do CEP '" + CEP + "': " + errorMessage, "warning");
				
				/* MSG NOVA */
				var variaveis = {
                    CEP: CEP,
                    errorMessage: errorMessage
                };
				
				MessageService.showMessageByCode("CEPINVALIDO", variaveis, {"type":"warning"});
				/* FIM MSG NOVA */
			});
		}
	};  

	/* **************************************************
	 * Procedimento que checa se o contrato da unidade
	 * foi rescindido e obriga ou não o preenchimento do
	 * field da data de rescisão.
	 *
	 * @param Widget widget
	 */
	self.checkContratoRescindido = function (widget) {
		var dateField = widget.getField("DTRESCCONTRATO");

		if (widget.currentRow.IDCNTRRESC == 'S') {
			dateField.readOnly = false;
			dateField.validations = {
				required: true
			};
		} else {
			dateField.readOnly = true;
			dateField.validations = {};
			self.nullifyFieldValues(dateField);
		}
	};

	/* **************************************************
	 * Procedimento que configura datas mínimas/máximas
	 * entre fields de acordo com a configuração das
	 * propriedades "successor" e "precursor".
	 *
	 * A propriedade "successor" indica o nome do field
	 * cuja data deve suceder a do field atual.
	 *
	 * A propriedade "precursor" indica o nome do field
	 * cuja data deve preceder a do field atual.
	 *
	 * @param Field currentField
	 */
	self.checkDateInterval = function (currentField) {
		var widget = currentField.widget;
		if (currentField.successor) {
			var laterDateField = widget.getField(currentField.successor);

			laterDateField.minDate = widget.currentRow[currentField.name];
		} else if (currentField.precursor) {
			var earlierDateField = widget.getField(currentField.precursor);

			earlierDateField.maxDate = widget.currentRow[currentField.name];
		}
	};

	/* **************************************************
	 * Função que valida um email de acordo com os termos
	 * do RFC2822.
	 *
	 * Found at
	 *    en.wikibooks.org/wiki/JavaScript/Best_practices
	 *
	 * @param String email
	 *
	 * @return boolean
	 */
	self.checkEmail = function (email) {
		//These comments use the following terms from RFC2822:
			//local-part, domain, domain-literal and dot-atom.
			//Does the address contain a local-part followed an @ followed by a domain?
			//Note the use of lastIndexOf to find the last @ in the address
			//since a valid email address may have a quoted @ in the local-part.
			//Does the domain name have at least two parts, i.e. at least one dot,
			//after the @? If not, is it a domain-literal?

		//This will accept some invalid email addresses
		//BUT it doesn't reject valid ones.

		var atSym = email.lastIndexOf("@");

		//no local-part
		if (atSym < 1)
			return false;

		//no domain
		if (atSym == email.length - 1)
			return false;

		//there may only be 64 octets in the local-part
		if (atSym > 64)
			return false;

		//there may only be 255 octets in the domain
		if (email.length - atSym > 255)
			return false;

		//is the domain plausible?
		var lastDot = email.lastIndexOf(".");

		//check if it is a dot-atom such as example.com
		if (lastDot > atSym + 1 && lastDot < email.length - 1)
			return true;

		//check if could be a domain-literal.
		if (email.charAt(atSym + 1) == '[' && email.charAt(email.length - 1) == ']')
			return true;

		return false;
	};

	////////////////////////////////////////////////////////
	//                 FIELD DEPENDENCIES                 //
	// A cadeia de funções e procedimentos que começa a   //
	// partir do método checkFieldDependencies supõe que  //
	// os fields que o chamam terão o objeto dependencies //
	//                                                    //
	// Tal objeto pode possuir as propriedades:           //
	//     parentFields, um objeto cujo cada propriedade  //
	//         indica uma relação de dependência com um   //
	//         field daquele nome. Pode apresentar as     //
	//         propriedades:                              //
	//             block                                  //
	//                 indica que o field deve ter seu    //
	//                 acesso bloqueado caso o parent não //
	//                 esteja preenchido corretamente     //
	//             filter                                 //
	//                 indica que o field deve receber um //
	//                 filtro criado a partir daquele     //
	//                 parent. Só funciona em fields do   //
	//                 tipo select                        //
	//             require                                //
	//                 indica que o field tornar-se-á     //
	//                 obrigatório caso o parent esteja   //
	//                 preenchido, e opcional caso        //
	//                 contrário                          //
	//     childrenFields, um array que indica os names   //
	//         dos fields que dependem diretamente desse  //
	//         field                                      //
	//     isAvailable, que é definida pela cadeia de     //
	//         funções e procedimentos, indica se o field //
	//         deveria estar acessível ou não             //
	//     isRequired, que é definida pela cadeia de      //
	//         funções e procedimentos, indica se o field //
	//         deveria estar obrigatório ou não           //
	//                                                    //
	////////////////////////////////////////////////////////

	/* **************************************************
	 * Procedimento que realiza a checagem do objeto
	 * dependencies. A passagem do parâmetro flags não é
	 * necesária, pois sua utilidade é interna à cadeia
	 * de métodos.
	 *
	 * Retorna um Array com todos os fields verificados
	 * pela cadeia de métodos.
	 *
	 * @param Field field
	 * @param Array flags
	 *
	 * @return Array
	 */
	self.checkFieldDependencies = function (field, flags) {
		if (!flags) flags = [];

		if (!flags.find(function (flag) {
				return flag === field.name;
			}))
			flags.push(field.name);

		if (field.dependencies) {
			if (self.checkFieldParentage(field))
				field.reload();

			flags = self.checkFieldFiliation(field, flags);
		}

		field.widget.isValid();
		templateManager.updateTemplate();

		return flags;
	};

	/* **************************************************
	 * Procedimento que chama o método de checagem do
	 * objeto dependencies para cada um dos fields
	 * dependentes do field atual. A passagem do
	 * parâmetro flags não é necesária, pois sua
	 * utilidade é interna à cadeia de métodos.
	 *
	 * Retorna um Array com todos os fields verificados
	 * pela cadeia de métodos.
	 *
	 * @param Field field
	 * @param Array flags
	 *
	 * @return Array
	 */
	self.checkFieldFiliation = function (field, flags) {
		if (field.dependencies.childrenFields && field.dependencies.childrenFields.length > 0) {
			var widget = field.widget;

			field.dependencies.childrenFields.forEach(function (child) {
				if (!flags.find(function (flag) {
						return flag === child;
					})) {

					var childField = widget.getField(child);

					flags = self.checkFieldDependencies(childField, flags);
				}
			});
		}

		return flags;
	};

	/* **************************************************
	 * Função que checa a disponibilidade do field, e, se
	 * necessário, limpa os dados, cria os filtros,
	 * requer o field ou bloqueia-o.
	 * Retorna true caso o filtro do field tenha sido
	 * alterado.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkFieldParentage = function (field) {
		if (field.dependencies.parentFields) {
			if (!field.validations)
				field.validations = {};

			self.checkFieldValidity(field);

			field.readOnly = !field.dependencies.isAvailable;

			if (field.dependencies.isRequired ^ field.validations.required)
				self.switchFieldRequirement(field);

			if (!field.dependencies.isAvailable) {
				self.nullifyFieldValues(field);
				return false;
			} else if (self.constructFieldFilters(field)) {
				self.nullifyFieldValues(field);
				return true;
			}
		}

		return false;
	};

	/* **************************************************
	 * Procedimento que checa os parents do field e
	 * configura o flag de disponibilidade de acordo.
	 *
	 * @param Field field
	 */
	self.checkFieldValidity = function (field) {
		var widget = field.widget;
		var forceRequire = (field.validations.required === true);

		field.dependencies.isAvailable = true;
		field.dependencies.isRequired = false;

		Object.keys(field.dependencies.parentFields).forEach(function (parent) {
			var parentField = widget.getField(parent);

			var valueColumn = self.getFieldValueColumn(parentField);

			if (field.dependencies.parentFields[parent].block && !widget.currentRow[valueColumn])
				field.dependencies.isAvailable = false;

			if (field.dependencies.parentFields[parent].require) {
				forceRequire = false;

				if (widget.currentRow[valueColumn])
					field.dependencies.isRequired = true;
			}
		});

		if (forceRequire)
			field.dependencies.isRequired = true;
	};

	/* **************************************************
	 * Procedimento que checa a função operacional da
	 * unidade e obriga ou não o preenchimento do field
	 * de fornecedor.
	 *
	 * @param Widget widget
	 */
	self.checkFuncaoOperacional = function (widget) {
		var fornField = widget.getField("NMRAZSOCFORN");

		if (widget.currentRow.IDFUNCAOFILI == 'O') {
			fornField.validations = {
				required: true
			};
		} else {
			fornField.validations = {};
			self.nullifyFieldValues(fornField);
		}
	};

	/* **************************************************
	 * Procedimento que altera a visibilidade da action
	 * de inserção de acordo com a disponibilidade de
	 * tipos de endereço.
	 *
	 * @param Widget widget
	 */
	self.checkInsertAction = function (widget) {
		var insertAction = widget.actions.find(function (action) {
			return action.name == 'ADD';
		});

		insertAction.isVisible = widget.validation.mayInsert;
	};

	/* **************************************************
	 * Função que checa a validade de um field que pode
	 * conter múltiplos emails, separados apenas pelo
	 * caractere ";". Possíveis whitespaces que estiverem
	 * separando emails serão removidos.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkMultipleEmails = function (field) {
		var validation = {
			isValid: null,
			which: []
		};

		if (field.value()) {
			var emails = field.value();
			validation.isValid = true;

			var length = emails.length;
			while (emails[--(length)] === ';')
				emails = emails.slice(0, -1);

			emails = emails.split(';');

			for (var i = 0; i < emails.length; i++)
				emails[i] = emails[i].trim();

			emails.forEach(function (email) {
				if (!self.checkEmail(email)) {
					validation.isValid = false;
					validation.which.push(email);
				}
			});

			field.value(emails.join(";"));
		} else
			validation.isValid = !(field.validations && field.validations.required);

		return validation;
	};

	/* **************************************************
	 * Procedimento que checa o input de inscrição,
	 * validando-o e colocando o tipo de inscrição na
	 * currentRow de acordo.
	 *
	 * Nota: Esse método pressupõe que o input de
	 * inscrição possui as propriedades valueField e
	 * descriptionField, apesar de ser do template
	 * text-edit. Ele também supõe que o valueField
	 * e o descriptionField estarão na currentRow.
	 *
	 * @param Field field
	 */
	this.checkNumeroDeInscricao = function (field) {
		var validatedInscricao = ger01700_unidade_service.validateNumeroDeInscricao(field.widget.currentRow[field.descriptionField]);
		var widget = field.widget;
		
		let campoInscricao = widget.getField('MSKINSJURFILI');
        let campoTipoInscricao = widget.currentRow.IDTPIJURFILI;
        
        let regex = new RegExp('[^A-Za-z0-9]');
        if(regex.test(campoInscricao.value()) && campoTipoInscricao != "I"){
            MessageService.showMessageByCode("VALIDA_CARACTER_ESPECIAL");
            campoInscricao.clearValue();
        }

		if (validatedInscricao.valid) {
			if(widget.currentRow.IDTPIJURFILI != 'I'){
				widget.currentRow.IDTPIJURFILI = validatedInscricao.tipo.cod;
				switch (validatedInscricao.tipo.cod) {
				case 'O': //Código genérico
					validatedInscricao.rawValue = General.fill(validatedInscricao.parameter, 5);
					break;
				case 'I': //Inscrição genérica
					//validatedInscricao.rawValue = General.fill(validatedInscricao.parameter, 20);
					break;
				default: //CPF && CNPJ
					validatedInscricao.rawValue = validatedInscricao.parameter;
				}
				widget.currentRow[field.valueField] = validatedInscricao.rawValue;
				widget.currentRow[field.descriptionField] = validatedInscricao.value;
			}else{
				widget.currentRow[field.valueField] = widget.currentRow[field.descriptionField];
			}

		} else {
			if (validatedInscricao.error.msg)
				ScreenService.showMessage(validatedInscricao.error.msg, "warning");

			widget.currentRow.IDTPIJURFILI = null;
			widget.currentRow[field.valueField] = null;
			widget.currentRow[field.descriptionField] = null;
		}
		
		

	};

	/* **************************************************
	 * Função que checa se o valor percentual é válido na
	 * escala definida, ou de 0 à 100 caso nenhuma escala
	 * for recebida.
	 *
	 * @param integer percentage
	 * @param integer minValue
	 * @param integer maxValue
	 *
	 * @return boolean
	 */
	self.checkPercentage = function (percentage, minValue, maxValue) {
		minValue = minValue | 0;
		maxValue = maxValue | 100;

		return (percentage >= minValue && percentage <= maxValue);
	};

	/* **************************************************
	 * Procedimento que requisita a validação da tab de
	 * parametrização e troca a visibilidade da widget
	 * de acordo com a configuração da base.
	 *
	 * @param Widget widget
	 */
	self.checkTabParametrizacao = function (widget) {
		self.buildParametrizacaoValidation(widget).then(function (validation) {
			var paramWidget = widget.container.getWidget('parametrizacao');

			paramWidget.isVisible = paramWidget.showAsTab = validation.showParamTab;
		});
	};

	/* **************************************************
	 * Função que checa se o tipo de endereço selecionado
	 * em um field está dentre os endereços já
	 * previamente cadastrados para aquela filial.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkTipoDeEndereco = function (field) {
		var widget = field.widget;

		var validTypes = widget.validation.usedTypes;
		var currentType = field.value();
		
		if(currentType == 'I') return true;
		else return validTypes.indexOf(currentType) != -1;
		
	};

	/* **************************************************
	 * Procedimento que checa a validez de uma nova PK de
	 * Unidade.
	 *
	 * @param Widget widget
	 */
	self.checkUnidadeCode = function (widget) {
		ger01700_unidade_service.validateUnidadePK(widget.currentRow.CDFILIAL).then(function (response) {
			if (!response.dataset.validation.VALID) {
				
				// ScreenService.showMessage("Já existe uma filial com o código '" + response.dataset.validation.CODE + "'!");
				
				/* MSG NOVA */
				var variaveis = {
                    CODFILIAL: response.dataset.validation.CODE
                };
				
				MessageService.showMessageByCode("FILIALEXISTENTE", variaveis, {"type":"error"});
				/* FIM MSG NOVA */

				widget.currentRow.CDFILIAL = '';

				self.checkFieldDependencies(widget.getField('CDFILIAL'));
			}
		}.bind(self));
	};

	/* **************************************************
	 * Procedimento que realiza todas as validações
	 * necessárias para a widget Endefili.
	 *
	 * @param Widget widget
	 */
	self.checkWidgetEndefili = function (widget, onEnter) {
		self.buildParentPrimaryKeys(widget, "CDFILIAL");

		self.buildEndefiliValidation(widget).then(function () {
			self.checkAddressFields(widget);

			if (!widget.currentRow.__is_new)
				self.checkInsertAction(widget);
		});

		if (!onEnter) {
			self.checkCEP(widget.getField('NRCEPFILI'));

			self.checkFieldDependencies(widget.getField('NMPAIS'));

			self.buildParentPrimaryKeys(widget, 'CDFILIAL');
		}
	};

	/* **************************************************
	 * Procedimento que realiza todas as validações
	 * necessárias para a widget Unidade.
	 *
	 * @param Widget widget
	 */
	self.checkWidgetUnidade = function (widget) {
		self.checkContratoRescindido(widget);

		self.checkFieldDependencies(widget.getField("NMRAZSOCEMPR"));
		self.checkFieldDependencies(widget.getField("CDFILIAL"));

		self.checkFuncaoOperacional(widget);
		
		if (!widget.currentRow.__is_new) {
			self.checkDateInterval(widget.getField("DTINCONTRFIL"));
			self.checkDateInterval(widget.getField("DTFICONTRFIL"));

			widget.currentRow.MSKINSJURFILI = widget.currentRow.NRINSJURFILI;

			if (widget.currentRow.NRINSJURFILI !== undefined)
				this.checkNumeroDeInscricao(widget.getField("MSKINSJURFILI"));

			self.buildSubWidgetFilters(widget, "CDFILIAL");

			self.checkTabParametrizacao(widget);
		}
	};
	
	/* **************************************************
	 * Função que checa os parents do field e cria os
	 * filtros de acordo. Retorna true caso algum filtro
	 * tenha sido criado ou mudado, e falso caso
	 * contrário ou se o field não for do tipo select.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.constructFieldFilters = function (field) {
		if (field.template.indexOf("select") != -1) {
			var changed = false;
			var widget = field.widget;

			Object.keys(field.dependencies.parentFields).forEach(function (parent) {
				if (field.dependencies.parentFields[parent].filter) {
					var parentField = widget.getField(parent);
					var valueColumn = self.getFieldValueColumn(parentField);

					changed = self.buildFilter(field, valueColumn) || changed;
				}
			});

			return changed;
		}
		return false;
	};

	/* **************************************************
	 * Procedimento que valida e executa a exclusão de
	 * um registro do tipo Endereço.
	 *
	 * @param Widget widget
	 */
	self.deleteWidgetEndefili = function (widget) {
		const currentType = widget.currentRow.IDTPENDEFILI;
		const widgetDados = templateManager.container.getWidget("dados");
		
		if(self.validateAddressTypeBetweenWidgets(widgetDados,currentType)){
			General.deleteCrud(widget).then(function () {
				return self.buildEndefiliValidation(widget, true);
			}).then(function () {
				self.checkInsertAction(widget);
				self.checkAddressFields(widget);
			});
		}
	};
	
	self.validateAddressTypeBetweenWidgets = function(widgetDados,currentType){
		let boolCheck = true;
		
		widgetDados.reload().then(_=>{
			if(currentType == 'P'){
				ScreenService.showMessage("O endereço principal não pode ser excluído.");
				boolCheck =false;
			}else if(widgetDados.currentRow.IDENDEMISFIL === currentType){
				ScreenService.showMessage("Exclusão não permitida! Endereço vinculado aos Dados Fiscais (Endereço para Emissão)");
				boolCheck =false;
			} else if(widgetDados.currentRow.IDENDCFOPFIL === currentType){
				ScreenService.showMessage("Exclusão não permitida! Endereço vinculado aos Dados Fiscais (Endereço para C.F.O.P.)");
				boolCheck =false;
			}
		});
		return boolCheck;
	};

	/* **************************************************
	 * Procedimento que valida a widget Endefili antes de
	 * habilitar sua edição.
	 *
	 * @param Widget widget
	 */
	self.editWidgetEndefili = function (widget) {
		let fieldTpInscricao = widget.getField('IDTPIJURFILI');
		fieldTpInscricao.readOnly = true;
		self.checkWidgetEndefili(widget);

		General.widgetEdit(widget);
	};

	/* **************************************************
	 * Procedimento que valida a widget Unidade antes de
	 * habilitar sua edição.
	 *
	 * @param Widget widget
	 */
	self.editWidgetUnidade = function (widget) {
		self.checkWidgetUnidade(widget);

		General.widgetEdit(widget);
	};

	/* **************************************************
	 * Função que recebe um field e retorna o nome da
	 * coluna que contém o value referente ao mesmo.
	 *
	 * @param Field field
	 *
	 * @return string
	 */
	self.getFieldValueColumn = function (field) {
		var valueColumn = field.name;

		if (field.outData && field.valueField) {
			Object.keys(field.outData).forEach(function (key) {
				if (field.outData[key] == field.valueField)
					valueColumn = field.valueField;
				else if (key == field.valueField)
					valueColumn = field.outData[key];
			});
		}

		return valueColumn;
	};

	/* **************************************************
	 * Procedimento que limpa todos os valores do outData
	 *
	 * Checa se a widget está em modo view e apenas limpa
	 * os dados caso "clearView" estiver setado como true
	 *
	 * @param Field field
	 * @param boolean clearView
	 */
	self.nullifyFieldValues = function (field, clearView) {
		var widget = field.widget;

		if (widget.mode == 'edit' || clearView) {
			if (field.outData) {
				Object.keys(field.outData).forEach(function (key) {
					widget.currentRow[field.outData[key]] = null;
				});
			}

			widget.currentRow[field.name] = null;
		}
	};

	/* **************************************************
	 * Procedimento genérico para erros de promise.
	 *
	 * @param Error error
	 */
	self.promiseErrorProcedure = function (error) {
		var errorMessage = 'Timeout';

		if (error.data)
			errorMessage = error.data.error;

		ScreenService.showMessage(errorMessage);
	};

	/* **************************************************
	 * Função que salva um novo registro de Unidade com
	 * seu respectivo registro de Endereço Principal.
	 *
	 * @param Widget widget
	 *
	 * @return Promise
	 */
	self.saveNewUnidade = function (widget) {
		widget.currentRow.CDOPERADOR = getLocalVar('CDOPERADOR');
		widget.currentRow.NRORGFILIAL = getLocalVar('NRORG');
		
		var addressWidget = templateManager.container.getWidget(widget.addressWidgetToForm);
		self.normalizaFuncaoFili(widget);

		Object.keys(addressWidget.currentRow).forEach(function (key) {
			if(!widget.currentRow[key])
				widget.currentRow[key] = addressWidget.currentRow[key];
		});

		if (addressWidget.isValid() && !self.checkAddressNumber(addressWidget)) {
			var deferred = ZHPromise.defer();

			deferred.reject("Número inválido.");
			ScreenService.showMessage("Número inválido.");

			return deferred;
		} else
			return General.widgetSave(widget); //.then(relacionaOperadorLogado(widget));
	};
	
	self.newEndefili = function(widget){
		let adressType = widget.getField('IDTPENDEFILI');
		adressType.dataSource.data = adressType.dataSource.data.filter(row => {
			return !widget.dataSource.data.some(wRow => {
				return wRow.IDTPENDEFILI == row.IDTPENDEFILI;
			});
		});
		General.widgetNewRow(widget);
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Unidade, caso o formulário esteja válido e os
	 * tipos de endereço selecionados estiverem de acordo
	 * com os existentes para aquela filial. Por fim,
	 * atualiza os valores da widget principal de acordo.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetDadosFiscais = function (widget) {
		const emissionAddress= widget.getField("IDENDEMISFIL");
		const CFOPAddress= widget.getField("IDENDCFOPFIL");
		
		self.normalizaFuncaoFili(widget);
		
		if (!self.checkTipoDeEndereco(emissionAddress))
				ScreenService.showMessage("Endereço para Emissão Inválido!");
		else if (!self.checkTipoDeEndereco(CFOPAddress))
				ScreenService.showMessage("Endereço para C.F.O.P. Inválido!");
		else if (!self.checkPercentage((widget.getField("VRPECREPREST").value() | 0)))
			ScreenService.showMessage("Percentual de crédito deve ser entre 0 e 100.");
		else
			General.widgetSave(widget).then(function () {
				self.updateParentRow(widget);
			});
		
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, caso o formulário esteja válido e o tipo
	 * de endereço não estiver registrado para aquela
	 * filial. Por fim, requisita novamente as validações
	 * e verifica a action de inserção.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetEndefili = function (widget) {
		let fieldTpInscricao = widget.getField('IDTPIJURFILI');
		fieldTpInscricao.readOnly = true;
		if (widget.isValid()) {
			self.normalizaFuncaoFili(widget);
			if (!self.checkAddressNumber(widget))
				ScreenService.showMessage("Número inválido.");
			else {
				if (!widget.currentRow.__is_new || self.checkAddressType(widget)) {
					General.widgetSave(widget).then(function () {
						widget.reload([]).then(function(){
							widget.reload().then(function(){
								return self.buildEndefiliValidation(widget, true);
							});
						});
					}).then(function () {
						self.checkInsertAction(widget);
						self.checkAddressFields(widget);
					});
				} else
					ScreenService.showMessage("Tipo de endereço já cadastrado.");
			}
		}
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, caso o formulário esteja válido e os
	 * emails inseridos estiverem válidos. Por fim, força
	 * a atualização da widget de endereço.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetNFE = function (widget) {
		var email = self.checkMultipleEmails(widget.getField("DSEMAILEMXMLNFE"));
		self.normalizaFuncaoFili(widget);
		
		
		if (!email.isValid){
			// ScreenService.showMessage("Email(s) [\"" + email.which.join("\", \"") + "\"] Inválido(s)!");
			var EMAILS = email.which.join("\", \"");
			
			/* MSG NOVA */
			var variaveis = {
                EMAILS: EMAILS
            };
			
			MessageService.showMessageByCode("EMAILINVALIDO", variaveis, {"type":"error"});
			/* FIM MSG NOVA */
			
		}else{
			General.widgetSave(widget).then(function () {
				this.atualizaAbaNfe(widget);
			}.bind(this));
			
		}
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, validando o email de login de SMTP. Por
	 * fim, força a atualização da widget de endereço.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetSMTP = function (widget) {
		var emailValue = widget.getField("DSLOGINSMTP").value() ? widget.getField("DSLOGINSMTP").value() : '';
		self.normalizaFuncaoFili(widget);
		
		if (!self.checkEmail(emailValue))
			ScreenService.showMessage("Email de Login Inválido!");
		else
			General.widgetSave(widget).then(function () {
				widget.container.getWidget("endereco").reload();
			});
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo Unidade
	 * de maneiras diferentes, de acordo com o estado de
	 * inserção ou alteração.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetUnidade = function (widget) {
		self.normalizaFuncaoFili(widget);
		
		if(General.isValidFiliaisValuesCP(widget.currentRow.CDFILIAL)){
			if (widget.currentRow.__is_new) {
				
				Endereco_Controller.save(widget.name, self.saveNewUnidade, [widget]);
			}
			else 
				General.widgetSave(widget);
		}else{
			ScreenService.showMessage('Unidade não habilitada para utilização! Favor entrar em contato com o administrador do sistema para que seja cadastrada a unidade no Portal do Cliente (CP).');
		}
	};
	
	var relacionaOperadorLogado = function (widget){
		let olddataSourceName = widget.dataSourceName;
		let currentRow = widget.currentRow;
		let operador = getLocalVar('CDOPERADOR');
		currentRow.CDOPERADOR = operador;
		currentRow.__is_new = true;
		widget.dataSourceName = 'cadEst#/crud/filioper';
		General.saveCrud(widget).then(() =>{
			widget.dataSourceName = olddataSourceName;
		});
		
	};
	
	self.normalizaFuncaoFili = function(widget){
		if(widget.currentRow.IDFUNCAOFILI == "U")
		 	widget.currentRow.IDFUNCAOFILI = null;
	};
	
	/* **************************************************
	 * Procedimento que checa se o número de telefone
	 * tem entre 8 a 9 caractéres, e caso não tenha, 
	 * apaga o campo.
	 *
	 * @param Field field
	 */
	self.checkTelefone = function(field) {
		var value = String(field.value());
		if(value.length<8) {
			field.clearValue();
		}
	};

	/* **************************************************
	 * Procedimento que coloca na currentRow o primeiro
	 * dado do dataSource da widget. Pensado para ser
	 * utilizado em widgets do tipo form.
	 *
	 * @param Widget widget
	 */
	self.setForms = function (widget) {
		widget.currentRow = widget.dataSource.data[0];
	};

	/* **************************************************
	 * Função que troca a condição "required" de um field
	 * e retorna um boolean que indica se o field passou
	 * ou não a sê-lo.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.switchFieldRequirement = function (field) {
		if (field.validations) {
			if (field.validations.required) {
				delete field.validations.required;
				return false;
			} else {
				field.validations.required = true;
				return true;
			}
		}
		return false;
	};

	/* **************************************************
	 * Procedimento que copia os dados divergentes da
	 * widget atual para a widget parent.
	 *
	 * @param Widget widget
	 */
	self.updateParentRow = function (widget) {
		var parentWidget = widget.getParent();

		Object.keys(parentWidget.currentRow).forEach(function (value) {
			if (widget.currentRow.hasOwnProperty(value) && parentWidget.currentRow[value] != widget.currentRow[value])
				parentWidget.currentRow[value] = widget.currentRow[value];
		});
	};

	//--------------------------------[GER01742]--------------------------------

	var callbackError = function(err) {
		var message = '';
		if (err.data && err.data.error) {
			message = err.data.error;
		} else {
			message = JSON.stringify(err);
		}
		ScreenService.showMessage(message);
	};

	self.confirmUnitChange = function (unidade) {
		var mensagem = "Deseja confirmar a troca de unidade?";
		ScreenService.confirmMessage(mensagem, "question", changeUnit.bind(unidade));
	};

    const changeUnit = function() {
    	if(General.isValidFiliaisValuesCP(this.CDFILIAL)){
			ger01700_unidade_service.alterarEstrutura(this).then(updateLocalSession.bind(this), callbackError);
    	}else{
    		MessageService.showMessageByCode('lib#CP_BRANCH_NOT_ALLOWED');
    	}
	};

	self.paramfilial = function() {
		var nrorg = arguments[0] || localStorageEngine.getLocalVar("NRORG") || 'NRORG_LOGADO';
		var cdfilial = arguments[1] || localStorageEngine.getLocalVar("CDFILIAL") || 'FILIAL_LOGADA';
		return restEngine.post('cadEst#/crud/paramfilial', {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "NRORG",
					"operator": "=",
					"value": nrorg
				},
				{
					"name": "CDFILIAL",
					"operator": "=",
					"value": cdfilial
				}
			]
		});

	};

	self.retornaDataProcessamento = function(cdfilial, nrorg) {
		return self.paramfilial(nrorg, cdfilial).then(function(callback) {
			var result = callback.dataset.paramfilial;
			var dtprocessa = result.length > 0 ?
								result[0].DTPROCESSA : null;

			if(dtprocessa)
				return moment(dtprocessa,'DD/MM/YYYY HH:mm:ss').format('L');
			else
				return moment().format('L');
		});
	};

	var buildUserAndHeader = function (name, descriptions, headerInfo) {
		var openWindowOperador = function () {
			ScreenService.openWindow("ger00002_operador");
		};
		ScreenService.buildUserData(name, descriptions, openWindowOperador);
		ScreenService.setHeaderInfo(headerInfo);
	};

	var setLocalVars = function (unidade) {
		setLocalVar("CDFILIAL", unidade.CDFILIAL);
		setLocalVar("NMFILIAL", unidade.NMFILIAL);
		setLocalVar("DTPROCESSA", unidade.DTPROCESSA);
	};
	
	var updateBackendSession = function (unidade) {
		var user = localStorage.getItem('LOGIN_USER');
		var acl = localStorageEngine.getLocalVar('accessProfiles');
		var otherSessionValues = {};
		otherSessionValues.CDFILIAL = localStorageEngine.getLocalVar("CDFILIAL");
		otherSessionValues.NMFILIAL = localStorageEngine.getLocalVar('NMFILIAL');
		otherSessionValues.DTPROCESSA = localStorageEngine.getLocalVar('DTPROCESSA');
		LibLoginController.updateSession(user, acl, otherSessionValues).then(result => {
			engine.openWindow(templateManager.project.mainWindow);
			eventAggregator.publish("onUnitChange", this);
			return 'OK!';
		});
	};

	var updateLocalSession = function() {
		self.retornaDataProcessamento(self.CDFILIAL, self.NRORG).then(function (dtprocessa) {
			var nmoperador = getLocalVar("NMOPERADOR");
			var unitDescription = self.CDFILIAL + ' - ' + self.NMFILIAL;
			var headerInfo = dtprocessa + " | " + unitDescription;
			buildUserAndHeader(nmoperador, [dtprocessa, unitDescription], headerInfo);
			self.DTPROCESSA = dtprocessa;
			setLocalVars(this);
			updateBackendSession(this);
		}.bind(this), callbackError);
	};
	
	this.gerarCliente = function(widget) {
		General.callRestEngine("cadEst#/filialCliente", "Row", widget.currentRow).then(() => {
			ScreenService.successNotification("Cliente gerado com sucesso");	
			ScreenService.closePopup();
		},
		(error) => {
			ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
		}); 	
	};
	
	this.atualizaAbaNfe = function(widget) {
		General.callRestEngine("cadEst#/atualizaAbaNfe", "Row", widget.currentRow).then(() => {
			// ScreenService.successNotification("Salvo com sucesso");
			widget.container.getWidget("nfe").reload();
		},
		(error) => {
			ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
		}); 	
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('ger01700_unidade_controller', ger01700_unidade_controller);
});


// FILE: ./assets/js/controllers/ger01800_unidade_de_medida_controller.js
var ger01800_unidade_de_medida_controller = function(ScreenService, General,ZHPromise) {
	var firstLoad = true;
	var focus = false;
	var fieldToFocus = '#SGUNIDADE';
	
	this.filterConvByUnid = function(widget){
		if(widget.currentRow.__is_new){
			widget.container.getWidget('conversao').isVisible = false;
		}else{
			widget.container.getWidget('conversao').isVisible = true;		
			widget.container.getWidget('conversao').dataSourceFilter = [
				{
					"name": "SGUNIDADE",
					"operator": "=",
					"value": widget.currentRow.SGUNIDADE
				}
			];		
			widget.container.getWidget('conversao').reload();		
		}
	};

	this.onLoadDataSource = function (widget) {
		var rows = widget.dataSource.data;
		rows.forEach(function(row){
			row.MSG = row.VRFTCONVER + " " + row.DSUNIDPARA + " (s) equivale a 1 " + row.DSUNIDDE;
		});
	};
	
	this.triggerFocusField = function(widget){
		//ScreenService.showMessage("a");
		focus = true;
		widget.reload();	
	};
	
	this.focusField = function(widget) {
		if(focus) {
			if(firstLoad) {
				widget.reload();
				firstLoad = false;
			} else {
				document.querySelector(fieldToFocus).focus();	
				fieldToFocus = '#SGUNIDADE';
			}
		}
	};
	
	this.cancelAdd = function(args) {
		General.backOrCancel(args);
	};
	
	this.saveAdd = function(widget) {
		General.saveCrud(widget).then(() => {
			widget.reload();
		});
		
	};
	
	this.editUnit = function(widget) {
		General.edit(widget);
	};
	
	this.onAddConvercao = function(widget){
		widget.newRow();
		widget.getField('RELACAOVALORES').dataSource = {};
		var currentRowUnidade = widget.container.getWidget('unidade').currentRow;
		widget.currentRow.SGUNIDDE = currentRowUnidade.SGUNIDADE;
		widget.currentRow.DSUNIDDE = currentRowUnidade.DSUNIDADE;
	};

	this.onChangeUnidConv = function(field) {
		if(field.value()){
			field.widget.getField('RELACAOVALORES').readOnly = false;
			field.widget.getField('VRFTCONVER').readOnly = false;
		}else{
			field.widget.getField('RELACAOVALORES').readOnly = true;
			field.widget.getField('VRFTCONVER').readOnly = true;	
		}
	};

	this.loadOpsRelacao = function(field){
		var currentRow = field.widget.currentRow;
		field.widget.getField('RELACAOVALORES').dataSource.data = [
			{
				"name": "dePara",
				"value": "1 "+ currentRow.DSUNIDDE +" equivale a "+ currentRow.DSUNIDPA + "(s)"
			},
			{
				"name": "paraDe",
				"value": "1 "+ currentRow.DSUNIDPA +" equivale a "+ currentRow.DSUNIDDE + "(s)"
			}
		];
	};

	this.prepVrFtConv = function(widget){
		if(widget.getField('RELACAOVALORES').value() === "dePara"){
			widget.currentRow.VRFTCONVER = General.valueToFloat(widget.currentRow.VRFTCONVER);
		}else{
			widget.currentRow.VRFTCONVER = 1 / (General.valueToFloat(widget.currentRow.VRFTCONVER));
		}
		widget.currentRow.IDTIPOPE = "*";
		General.saveCrud(widget).then(function(){
			widget.reload();
		});	
	};

	this.convView = function(widget){
		if(widget.currentRow.__is_new){
			// widget.getField('RELACAOVALORES').readOnly = true;
			widget.getField('VRFTCONVER').readOnly = true;
			widget.getField('DSUNIDPA').readOnly = false;
		}else{
			widget.currentRow.DSUNIDPA = widget.currentRow.DSUNIDPARA;
			widget.getField('RELACAOVALORES').dataSource.data = [
				{
					"name": "dePara",
					"value": "1 "+ widget.currentRow.DSUNIDDE +" equivale a "+ widget.currentRow.DSUNIDPA + "(s)"
				},
				{
					"name": "paraDe",
					"value": "1 "+ widget.currentRow.DSUNIDPA +" equivale a "+ widget.currentRow.DSUNIDDE + "(s)"
				}
			];
			// widget.getField('RELACAOVALORES').readOnly = false;
			widget.getField('VRFTCONVER').readOnly = false;
			widget.getField('DSUNIDPA').readOnly = true;
		}
		
	};

};

Configuration(function(ContextRegister) {
	ContextRegister.register("ger01800_unidade_de_medida_controller", ger01800_unidade_de_medida_controller);
});

// FILE: ./assets/js/controllers/ger15300_agrupamento_unidade_controller.js
var ger15300_agrupamento_unidade_controller = function(General, ScreenService, templateManager) {

    this.filterWidgetsAgrupamento = function(widget) {
        widget.widgets.forEach(function(w){
            w.dataSourceFilter = [
                {
                    name: "CDGRUPFILI",
                    operator: "=",
                    value: widget.currentRow.CDGRUPFILI
                }
            ];
        });
    };

    this.filterWidgetsTab = function(args) {
        var outerWidget = args.owner;
        var groupWidget = outerWidget.getParent().currentRow.CDGRUPFILI;

        outerWidget.dataSourceFilter = [
            {
                "name": "CDGRUPFILI",
                "operator": "=",
                "value": groupWidget
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORGTRAB')
            }
        ];

        outerWidget.reload();
    };

    this.filterWidgetsSelect = function(args) {
        var outerWidget = args.owner.field;
        var groupWidget = outerWidget.widget.getParent().currentRow.CDGRUPFILI;

        outerWidget.dataSourceFilter = [
            {
                "name": "CDGRUPFILI",
                "operator": "<>",
                "value": groupWidget
            }
        ];

        outerWidget.reload();
    };

    this.ger15300_agrupamento_unidade_controllerActionEventCancel = function(args) {
		var widget = args.owner.widget;

		General.widgetCancelForm(widget);
	};

    this.ger15300_agrupamento_unidade_controllerActionEventEdit = function(args) {
	    var widget = args.owner.widget;

        General.widgetEdit(widget);

	};

    this.ger15300_agrupamento_unidade_controllerActionEventNew = function(args) {

        var widget = args.owner.widget;
        General.widgetNewRow(widget);
    };

    this.ger15300_agrupamento_unidade_controllerActionEventSaveTab = function(args) {

        var widget = args.owner.widget;
        widget.currentRow.CDGRUPFILI = widget.getParent().currentRow.CDGRUPFILI;

        General.saveCrud(widget).then(function(success){
            ScreenService.successNotification('Salva com sucesso!');
            General.changeWidgetAcessibility(widget.getParent().widgets, true);
        });
    };

    this.ger15300_agrupamento_unidade_controllerActionEventSave = function(args) {

        var widget = args.owner.widget;

        if (getValidCD(widget.currentRow.CDGRUPFILI, 10) !== null){
            widget.currentRow.CDGRUPFILI = getValidCD(widget.currentRow.CDGRUPFILI, 10);
            
            General.saveCrud(widget).then(function(callback){
                ScreenService.successNotification('Salva com sucesso!');
                General.changeWidgetAcessibility(widget.widgets, true);
            });
    
        }
    };

    this.ger15300_agrupamento_unidade_controllerActionEventDeleteTab = function(args) {

        var widget = args.owner.widget;
        widget.currentRow.CDGRUPFILI = widget.getParent().currentRow.CDGRUPFILI;

        General.deleteCrud(widget).then(function(success){
            ScreenService.successNotification('Deletado com sucesso!');

        });

        widget.reload();
    };

    this.ger15300_agrupamento_unidade_controllerActionEventDelete = function(args) {

        var widget = args.owner.widget;

        General.deleteCrud(widget).then(function(success){
            ScreenService.successNotification('Deletado com sucesso!');

        });

        widget.reload();
    };

    var getValidCD = function(check, sizeOfColumn) {
        check = check+'';
		if (check !== null) {

			var repeatZero = sizeOfColumn - check.length;
			var regex = /^[0-9]+$/g;
			regex.test(regex, sizeOfColumn.toString());
			return regex.test(check) ? ('0'.repeat(repeatZero) + check) : null;

		} else {

			return null;
		}
	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ger15300_agrupamento_unidade_controller', ger15300_agrupamento_unidade_controller);
});

// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex((item) => item.id === containerId) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex((item) => item.id === templateManager.container.id);
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = { item, index };

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(() => {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', (evt) => {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, (args) => {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', (favorites) => {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});

// FILE: ./assets/modules/produto/assets/js/controllers/ColorPickerController.js
function ColorPickerController() {

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (widget) {
		var r = widget.currentRow.CORCONFIGURACAO._r;
		var g = widget.currentRow.CORCONFIGURACAO._g;
		var b = widget.currentRow.CORCONFIGURACAO._b;
	    widget.currentRow.CORCONFIGURACAO =  "#" + ((1 << 24) + (Math.floor(r) << 16) + (Math.floor(g) << 8) + Math.floor(b)).toString(16).slice(1);
	};
	
	this.hexToRgb = function(hex) {
		var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
	    return result ? {
	        r: parseInt(result[1], 16),
	        g: parseInt(result[2], 16),
	        b: parseInt(result[3], 16)
	    } : null;
	};

	this.rgbToHexOnRow = function (row, field){
		var r = row[field]._r;
		var g = row[field]._g;
		var b = row[field]._b;
		row[field] = "#" + ((1 << 24) + (Math.floor(r) << 16) + (Math.floor(g) << 8) + Math.floor(b)).toString(16).slice(1);
		return row[field];	
	};
	
	this.rgbToHexOnRowWithoutHash = function (row, field){
		var r = row[field]._r;
		var g = row[field]._g;
		var b = row[field]._b;
		row[field] = ((1 << 24) + (Math.floor(r) << 16) + (Math.floor(g) << 8) + Math.floor(b)).toString(16).slice(1);
		return row[field];	
	};

	this.RGBToTColor = function(r, g, b){
		var Tcolor = r | (g << 8) | (b << 16);
		return Tcolor;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ColorPickerController', ColorPickerController);
});

// FILE: ./assets/modules/produto/assets/js/controllers/LayoutController.js
function LayoutController(General, ScreenService, ColorPickerController, restEngine, templateManager) {

	var self = this;
	var TAMANHO_FONTE_PADRAO = '08';

	this.setCicloColor = function(element, row, fieldName) {
		if (("" + row[fieldName]).charAt(0) != '#')
			self.convertTColorToHexa(row);
		if (row) {
			if (element.hasClass('zh-span-field ng-scope')) {
				element.find('p').css('background-clip', 'content-box');
				element.find('p').css('background-color', '#' + row[fieldName]);
				element.find('p').css('color', '#' + row[fieldName]);
			}
			else if (element.hasClass('td zh-standard-column  grid-align-left')) {
				element.find('p').css('background-clip', 'content-box');
				element.find('span').css('background-color', '#' + row[fieldName]);
				element.find('span').css('color', '#' + row[fieldName]);
				element.find('span').css('display', 'block');
			}
		}
	};
 
	this.convertTColorToHexa = function(row) {
		if (!isNaN(row.NRCOLORBACK)) {
			row.NRCOLORBACK = this.colorChange(row.NRCOLORBACK);
		}
		if (!isNaN(row.NRCOLORTEXT)) {
			row.NRCOLORTEXT = this.colorChange(row.NRCOLORTEXT);
		}
	};

	this.colorChange = function(Cor) {
		var r = Cor & 0XFF;
		var g = (Cor >> 8) & 0xFF;
		var b = (Cor >> 16) & 0XFF;
		var Hexa = "#" + ((1 << 24) + (Math.floor(r) << 16) + (Math.floor(g) << 8) + Math.floor(b)).toString(16).slice(1);
		return Hexa;
	};

	this.newRowGruposDeProdutos = function(args) {
		var widget = args.owner.widget;
		if(widget.bkpRow)
			 widget.currentRow = widget.bkpRow;
		var row = widget.currentRow; 
		 widget.bkpRow = Util.clone(widget.currentRow); 
		General.widgetNewRow(widget);
		
		widget.getField("NRBUTTON").readOnly = 
		widget.getField("DSBUTTON").readOnly = 
		row.__is_new = true;
		row.ANTG_NRBUTTON = '';
		row.CDNATPRODUTO = null;
		row.IDTPBUTTON = 1;
		row.NMPRODUTO = General.getFieldValue('NMPRODUTO', 'Detalhamento');
		row.CDPRODUTO = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		row.CDARVPROD = General.getFieldValue('CDARVPROD', 'Detalhamento');
		
		var filterData = !widget.appliedFilter ? widget.appliedFilter.FILTER_ROW : false; 
		
		if( !!filterData && !!filterData.CDFILIAL) {
			widget.getField('NMCONFTELA').readOnly = false;
			widget.getField('NMFILIAL').readOnly = true;
			row.NMFILIAL = filterData.NMFILIAL;
			row.CDFILIAL = filterData.CDFILIAL;
		} else {
			widget.getField('NMFILIAL').readOnly = false;
			widget.getField('NMCONFTELA').readOnly = true;
		}
		
		row.DSBUTTON2 = row.NMPRODUTO;
		row.NRFONTETEXT = TAMANHO_FONTE_PADRAO;
		row.NRCOLORBACK = '#fff';
		row.NRCOLORTEXT = '#ff5500';
		widget.currentRow = row;
		widget.reload();
	};
	
	this.editRowGruposDeProdutos = function(args) {
		var widget = args.owner.widget;
		General.widgetEdit(widget);
		widget.getField("NRBUTTON").readOnly = true;
		widget.currentRow.__is_new = false;
		widget.currentRow.ANTG_NRBUTTON = widget.currentRow.NRBUTTON;
		widget.reload();
	};
	
	this.changeFieldNRPGCONFTELA = function(args){
		var widget = args.owner.field.widget;
		if(args.owner.field.value() !== null){
			widget.getField("NRBUTTON").readOnly = false;
			widget.currentRow.NRBUTTON = "";
		}
	};
    
    this.grupoOnChange = function(args){
        var field = args.owner.field;
        var widget = field.widget;
        widget.currentRow.NRPGCONFTELA = '';
        field.widget.getField('NRPGCONFTELA').readOnly = (field.value() == null || field.value() == '');
        if((field.value() == null || field.value() == '')){	
        	field.widget.getField('NRPGCONFTELA').value(field.value());
        	field.widget.getField('NRBUTTON').value(field.value());
        	field.widget.getField('NRBUTTON').readOnly = (field.value() == null || field.value() == '');
        }
        widget.currentRow.NRCOLORBACK = this.colorChange(widget.currentRow.NRCOLORBACKGRUPO);
        widget.currentRow.NRCOLORTEXT = this.colorChange(widget.currentRow.NRCOLORTEXTGRUPO);
        templateManager.updateTemplate();
        widget.reload();
    };
    
    this.filialOnChange = function(args){
        var field = args.owner.field;
        field.widget.getField('NMCONFTELA').readOnly = (field.value() == null || field.value() == '');
        if((field.value() == null || field.value() == '')){	
        	field.widget.getField('NMCONFTELA').value(field.value());
        	field.widget.getField('DSBUTTON').value(field.value());
        	field.widget.getField('DSBUTTON').readOnly = (field.value() == null || field.value() == '');
        	field.widget.getField('NRPGCONFTELA').value(field.value());
        	field.widget.getField('NRPGCONFTELA').readOnly = (field.value() == null || field.value() == '');
        	field.widget.getField('NRBUTTON').value(field.value());
        	field.widget.getField('NRBUTTON').readOnly = (field.value() == null || field.value() == '');
        }
    };
    
    this.confgtelaOnChange = function(args){
        var field = args.owner.field;
        field.widget.getField('DSBUTTON').readOnly = (field.value() == null || field.value() == '');
        if((field.value() == null || field.value() == '')){	
        	field.widget.getField('DSBUTTON').value(field.value());
        	field.widget.getField('NRPGCONFTELA').value(field.value());
        	field.widget.getField('NRPGCONFTELA').readOnly = (field.value() == null || field.value() == '');
        	field.widget.getField('NRBUTTON').value(field.value());
        	field.widget.getField('NRBUTTON').readOnly = (field.value() == null || field.value() == '');
        }
    };

	this.setFilterNRBUTTON = function(field) {
		var row = field.widget.currentRow;
		var dataFilter = field.dataSourceFilter;
		dataFilter[2].value = row.NRCONFTELA;
		dataFilter[3].value = row.IDTPBUTONAUX;
		dataFilter[4].value = row.NRBUTTONAUX;
		dataFilter[5].value = row.NRPGCONFTAUX;
		dataFilter[6].value = row.IDTPBUTONAUX2;
		dataFilter[7].value = row.NRBUTTONAUX2;
		dataFilter[8].value = row.NRPGCONFTELA;
		dataFilter[9].value = row.CDGRUPO;
		dataFilter[10].value = row.PROPRIAPOSICAO;
		field.reload();
		
	};

	this.productsInsideGroupWidgetSaveAction = function(widget) {
		if (widget.isValid()) {
			widget.currentRow.NRORG = getLocalVar('NRORG');
			widget.currentRow.IDTPBUTONAUX = 2;
			//widget.currentRow.NRBUTTONAUX   = widget.currentRow.NRBUTTON;
			//widget.currentRow.NRPGCONFTAUX  = widget.currentRow.NRPGCONFTELAUX;
			widget.currentRow.CDIDENTBUTON = widget.currentRow.CDPRODUTO;
			widget.currentRow.CDNATPRODUTO = widget.currentRow.CDNATPROD ?  widget.currentRow.CDNATPROD : null;
			widget.currentRow.DSMESSAGEBUT = widget.currentRow.DSMESSAGEBUT ?  widget.currentRow.DSMESSAGEBUT : null;
			widget.currentRow.DSBUTTON = widget.currentRow.DSBUTTON2;
			widget.currentRow.CDOPERULTATU = getLocalVar('CDOPERADOR');
			widget.currentRow.DSENDEIMG = widget.currentRow.IMAGE;
			if (widget.currentRow.__is_new) { 
					widget.currentRow.IDTPBUTONAUX2 = '0';
					widget.currentRow.NRBUTTONAUX2 = '0';
					widget.currentRow.NRPGCONFTAUX2 = '0';
					self.setLocalVarsOnWidget(widget);
					widget.currentRow.CDGRUPO = widget.currentRow.NRPGCONFTELA + "" + widget.currentRow.NRBUTTON;
			}
			var hexBack = ColorPickerController.hexToRgb(widget.currentRow.NRCOLORBACK),
				hexText = ColorPickerController.hexToRgb(widget.currentRow.NRCOLORTEXT);
			widget.currentRow.NRCOLORBACK = ColorPickerController.RGBToTColor(hexBack.r, hexBack.g, hexBack.b);
			widget.currentRow.NRCOLORTEXT = ColorPickerController.RGBToTColor(hexText.r, hexText.g, hexText.b);


			General.widgetSave(widget).then(function() {

				widget.currentRow.changed = false;
				widget.currentRow.__is_new = false;
				ScreenService.closeSwipe();
				widget.reload();
			});

		}

	};

	this.setLocalVarsOnWidget = function(widget) {
		var row = widget.currentRow;

		row.DTULTATU = moment().format('DD/MM/YYYY HH:mm:ss');
		row.NRORG = getLocalVar('NRORG');
		row.CDOPERULTATU = getLocalVar('CDOPERADOR');
		row.NRORGULTATU = getLocalVar('NRORG');

		if (row.__is_new) {
			row.CDOPERINCLUSAO = getLocalVar('CDOPERADOR');
			row.NRORGINCLUSAO = getLocalVar('NRORG');
			row.DTINCLUSAO = moment().format('DD/MM/YYYY HH:mm:ss');
		}
	};
	
	this.buildImageInViewMode = function(wdgLogotipo){
        setTimeout(function(){
            if(!Util.isEmptyOrBlank(wdgLogotipo.currentRow.IMAGE)){
                var row = wdgLogotipo.currentRow.IMAGE;
                var fieldLogo = $('#span-field-IMAGE');
                var b64Image = typeof row === "object" ? row[0].b64File : row;
                var urlLogo = b64Image;
                var image = '<img src='+urlLogo+' class="img-responsive center-block img-cadEst">';
                if($('.img-responsive.center-block').length <= 0){
                    fieldLogo.find('img').remove();
                    fieldLogo.append(image);
                    fieldLogo.find('p').css({display:'none'});
                }
            }else{
                this.setDefaultImage();
            }
        }.bind(this), 50);
        wdgLogotipo.redraw();
    };
    
    this.setDefaultImage = function(){
        var fieldLogo = $('#span-field-IMAGE');
        fieldLogo.find('p').css({display:'none'});
        fieldLogo.find('img').remove();
        fieldLogo.append('<img></img>');
        fieldLogo.find('img').addClass('defaultImage');
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('LayoutController', LayoutController);
});


// FILE: ./assets/modules/produto/assets/js/controllers/TabelaPrecoController.js
function TabelaPrecoController(ScreenService, General) {
    
    const MENSAGEM_VALIDA_PRECO = "Os valores de Preço e Preço Subsidiado não podem ter valor menor que zero.";
    
    var checkValorMenorQueZero = function(value){
        return value < 0;
    };
    
    this.onChangeFilial = function(args, fieldToReadOnly) {
        var field = args.owner.field;
        var widget = field.widget;
        
        if (field.value() !== undefined && field.value() != ''){
            widget.getField(fieldToReadOnly).readOnly = false;
        }else{
            widget.getField(fieldToReadOnly).readOnly = true;
            widget.getField(fieldToReadOnly).value(null);
        }
        widget.getField(fieldToReadOnly).value(null);
    };

	this.newPriceTag = function(args) {
		var widget = args.owner.widget;
		General.widgetNewRow(widget);
		widget.currentRow.NMPRODUTO = General.getFieldValue('NMPRODUTO', 'Detalhamento');
		widget.currentRow.CDPRODUTO = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		widget.currentRow.CDARVPROD = General.getFieldValue('CDARVPROD', 'Detalhamento');
		if(Array.isArray(widget.appliedFilter) && (widget.appliedFilter.FILTER_ROW.CDFILIAL !== undefined && widget.appliedFilter.FILTER_ROW.CDFILIAL !== '' && widget.appliedFilter.FILTER_ROW.CDFILIAL !== null)){
			widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
		}
		widget.getField('CDFILIAL').readOnly = false;
		widget.getField('CDTABEPREC').readOnly = false;
		widget.reload();
	};
	
	
	
	this.parseStrToFloat = function (value) {
		if(!!value) {
	    	value = String(value);
	    	var numberOfDots = (value.match(/\./g) || []).length,
	        	hasVirgula   = !!(value.match(/\,/g) || []).length;
	    	
	    	if(hasVirgula) {
	    	    if(!!numberOfDots) {
	        		value = value.replace(/\./g, "");
	        	}
	    	    
	    	    value = value.replace(",", ".");
	    	}
	    } else {
	    	value = 0.00;
	    }
	    
	    return parseFloat(value);
	};

	this.savePriceTag = function(widget) {
		widget.currentRow.NRORG = getLocalVar('NRORG');
		widget.currentRow.CDPRODUTO = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		widget.currentRow.CDPRPAITABPR = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		 widget.currentRow.VRPRECITEM = this.parseStrToFloat(widget.currentRow.VRPRECITEM);
		widget.currentRow.VRPRESUGITEM = this.parseStrToFloat(widget.currentRow.VRPRESUGITEM);
		widget.currentRow.VRPRECITEMCL = this.parseStrToFloat(widget.currentRow.VRPRECITEMCL);
		widget.currentRow.VRCUSTITEM = this.parseStrToFloat(widget.currentRow.VRCUSTITEM);
		General.widgetSave(widget).then(function(){
            ScreenService.closeSwipe();
		    widget.reload();
		});
	};
	
	this.editPriceTag = function(args) {
		var widget = args.owner.widget;
		General.widgetEdit(widget);
		var CurrentRow = widget.currentRow;
		CurrentRow.CDFILIAL = Array.isArray(CurrentRow.CDFILIAL) ? CurrentRow.CDFILIAL : [CurrentRow.CDFILIAL];
		CurrentRow.CDTABEPREC = Array.isArray(CurrentRow.CDTABEPREC) ? CurrentRow.CDTABEPREC : [CurrentRow.CDTABEPREC];
		CurrentRow.NMTABEPREC = Array.isArray(CurrentRow.NMTABEPREC) ? CurrentRow.NMTABEPREC : [CurrentRow.NMTABEPREC];
		CurrentRow.CDFILIALTAB = Array.isArray(CurrentRow.CDFILIAL) ? CurrentRow.CDFILIAL : [CurrentRow.CDFILIAL];
		CurrentRow.DTFINVGPREC = Array.isArray(CurrentRow.DTFINVGPREC) ? CurrentRow.DTFINVGPREC : [CurrentRow.DTFINVGPREC];
		CurrentRow.DTINIVGPREC = Array.isArray(CurrentRow.DTINIVGPREC) ? CurrentRow.DTINIVGPREC : [CurrentRow.DTINIVGPREC];
		widget.getField('CDFILIAL').reload();
		widget.getField('CDTABEPREC').reload();
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('TabelaPrecoController', TabelaPrecoController);
});

// FILE: ./assets/modules/produto/assets/js/controllers/ger01900_cadastroproduto_controller.js
var ger01900_cadastroproduto_controller = function (ZhUtil, ScreenService, templateManager, General, ProcProdWorkflowService, ProdutoService, ZhFilterPreferences, restEngine, MessageService) {
		
	var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey", "$error"];
	
	this.setDynamicRequiredFields = function(mainWidget) {
		General.callRestEngine('cadEst#/crud/paramgeral', 'FilterData', []).then(success => {
			if(success.dataset.paramgeral.length){
				let paramgeral = success.dataset.paramgeral[0];
				let obrigaNCM = paramgeral.IDOBRNCMCADPR == 'S'? true : false;
				let widgetFiscal = templateManager.container.getWidget('Fiscal');
				let fieldNCM = widgetFiscal.getField('CDCLASFISC');
				if(obrigaNCM)
					fieldNCM.validations = {"required": {"message": "Campo Obrigatorio"}};
            	else
                	delete fieldNCM.validations.required;
			}
		});
	};
	
	this.openFilterWidget = function(widget) {  
		
		var defaultFilter = Util.clone(widget.currentRow);
		
		defaultFilter.IDPRODATIVPR = 'S';
		
        ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter);

	};
	
	this.allRulesWidgets = function() {
		this.rulesToCardapio(templateManager.container.getWidget('Cardapio'));
		this.rulesToCatering(templateManager.container.getWidget('Catering'));
		this.rulesToEstoque(templateManager.container.getWidget('Estoque'));
		this.rulesToProducao(templateManager.container.getWidget('Produção'));
	};	

	this.allWidgetsToView = function() {
		var widget = templateManager.container.getWidget('produtos2');
		widget.view();
		widget.getParent().list();
		widget.widgets.forEach(function(widget) {
			widget.view();
			widget.template = widget.templateToView;
		});
	};

    this.afterChange = function(widget) {
        var mainWidget = widget.getParent();
        mainWidget.widgets.forEach(function(widget) {
            widget.enabled = true;
        });
    	templateManager.container.getWidget('arvsprod').enabled = true;
    	templateManager.container.getWidget('vendas').enabled = true;

		widget.template = widget.templateToView;
		widget.view();
		mainWidget.view();
    };

	this.callSave = function(widget, route) {
		ScreenService.showLoader();
		var row = Util.clone(widget.currentRow, ignoredKeys);
		row.DEBUGMODE = false;
		General.callRestEngine(route, 'Row', row).then(function(result) {
			var currentProduct = result.dataset.produto[0];
			widget.currentRow = currentProduct;
			var widgetPrincipal = widget.container.getWidget('produtos');
			widgetPrincipal.currentRow = currentProduct;
			this.updateWidgets(widget.currentRow);
			this.allWidgetsToView();
			this.updateProductInMainWidget();
			this.afterChange(widget);
			if(result.messages) {
				ScreenService.alertNotification(result.messages[0].message);
			}
			ScreenService.hideLoader();
		}.bind(this), this.errorRequest);
	};

    this.cancelEdit = function(widget) {
    	var __is_new = angular.copy(widget.currentRow.__is_new);

    	this.afterChange(widget);

    	widget.currentRow = widget.backupRow;

		if (__is_new) {
			ScreenService.closeSwipe();
			widget.getParent().getParent().dataSource.data.pop();
		}
	};

	this.deleteGroupTree = function(widget) {
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			General.callRestEngine('cadEst_produto#/crud/prodarvoreprod/remove', 'Row', widget.currentRow).then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					widget.reload();
				});
			}, this.errorRequest);
		}.bind(this));
	};

    this.editMode = function (widgetEditing, skipeEnabled) {
    	var nameSkip = widgetEditing.name;
		var mainWidget = widgetEditing.getParent();
		
		if (!skipeEnabled) {
    		mainWidget.widgets.forEach(function(widget) {
    			if (widget.name !== nameSkip) {
    				widget.enabled = false;
    			}
			});
		}
		
    	templateManager.container.getWidget('arvsprod').enabled = false;
    	templateManager.container.getWidget('vendas').enabled = false;
    	let rowBackup = Util.clone(widgetEditing.currentRow, ignoredKeys);
    	widgetEditing.backupRow = rowBackup;
    	widgetEditing.template = widgetEditing.templateToForm;
		mainWidget.edit();
		widgetEditing.edit();
		ScreenService.showLoader();
    	setTimeout(time =>{
    		widgetEditing.currentRow = rowBackup;
    		ScreenService.hideLoader();
    	}, 1000);
    };
    
    this.errorRequest = function(error) {
    	if (error.data.error) {
			ScreenService.showMessage(error.data.error);
		} else {
			ScreenService.showMessage(error);
		}
		ScreenService.hideLoader();
    };

    this.gerFilterForPrductComplete = function() {
    	var levels = General.getLevelByMask('GER_CDPICTPROD', 1);

    	return levels.length;
	};
	
	this.getMaskPenulLevel = function() {
		var mask = getLocalVar('GER_CDPICTPROD');
		mask = mask.split('.');
		mask.pop();
		return mask.join('.');
	};

	this.grupFieldReadOnly = function() {
		var widget = templateManager.container.getWidget('arvsprod');
		var field = widget.getField('NMARVOREPROD');
		field.readOnly = true;
	};

	this.newRelacTreeeProd = function (widget) {
		widget.newRow();
		widget.currentRow.CDPRODUTO = widget.getParent().currentRow.CDPRODUTO;
	};

	this.isProductStock = function() {
		var widget = templateManager.container.getActiveWidget();
		var indent = widget.currentRow.IDESTOCOMP;
		if (indent === 'E') {
			return true;
		} else {
			return false;
		}
	};

    this.rulesItemGenero = function(widget) {
    	if (widget.currentRow.CDTPITEMPROD || widget.currentRow.CDGENPROD) {
    		widget.getField('DSGENPROD').validations.required = true;
    		widget.getField('NMTPITEMPROD').validations.required = true;
    	} else {
    		delete widget.getField('DSGENPROD').validations.required;
    		delete widget.getField('NMTPITEMPROD').validations.required;
    	}
	};
	
	this.removeHintFields = function(widgets) {
		if (!Array.isArray(widgets)){
			widgets = [widgets];
		}

		widgets.forEach(function(widget) {
			delete widget.currentRow.$error;
		});
	};
    
    this.rulesSelectClasfiscal = function(widget) {
    	var clasfiscal = widget.currentRow.CDCLASFISC;
    	var genero = clasfiscal.substr(0, 2);
    	var filter = [{
    		"name": "CDGENPROD",
    		"operator": "=",
    		"value": genero
    	}];
    	General.callRestEngine('cadEst_produto#/crud/generoprod_livromagnetico', 
    							'FilterData', filter).then(function(data) {
    		data = data.dataset.generoprod_livromagnetico;
    		if (data.length > 0) {
    			widget.currentRow.CDGENPROD = data[0].CDGENPROD;
    			widget.currentRow.DSGENPROD = data[0].DSGENPROD;
    		} else {
    			widget.currentRow.CDGENPROD = null;
    			widget.currentRow.DSGENPROD = null;
    		}
    		this.rulesItemGenero(widget);
    	}.bind(this), this.errorRequest);
	};
	
	this.validaFatoConv = function(widget) {
		let fieldFatoConv = widget.container.getWidget('Estoque').getField('VRFATOCONV');
		if (this.isProductStock()) {
			fieldFatoConv.setValue(1);
			widget.currentRow.VRFATOCONV = 1;
		}else{
			if(fieldFatoConv.value() <= 0){
				fieldFatoConv.setValue(null);
				ScreenService.showMessage('Favor de conversão deve ser maior que 0');
			}
		}
	};

	this.rulesToCardapio = function(widget) {
		if (this.isProductStock()) {
			widget.getField('VRARREDREQ').readOnly = false;
		} else {
			widget.getField('VRARREDREQ').readOnly = true;
		}
	};

	this.rulesToCatering = function(widget) {
		if (this.isProductStock()) {
			widget.getField('DSVRYIELD').readOnly = false;
			widget.getField('DSVRYIELD').validations.required = true;
		} else {
			widget.getField('DSVRYIELD').readOnly = true;
			delete widget.getField('DSVRYIELD').validations.required;
		}
	};
	
	this.rulesToEstoque = function(widget) {
		if (this.isProductStock()) {
			widget.getField('VRPESOUNID').readOnly = false;
			widget.getField('DSPEFATOCORR').readOnly = false;
			widget.getField('DSPEFATOCORR').validations.required = true;
			widget.getField('NMPRODESTO').readOnly = true;
			delete widget.getField('NMPRODESTO').validations.required;
			widget.getField('VRFATOCONV').readOnly = true;
			delete widget.getField('VRFATOCONV').validations.required;
			widget.getField('HIERARQUIAPRODESTO').readOnly = true;
		} else {
			widget.getField('VRPESOUNID').readOnly = true;
			widget.getField('DSPEFATOCORR').readOnly = true;
			delete widget.getField('DSPEFATOCORR').validations.required;
			widget.getField('NMPRODESTO').readOnly = false;
			widget.getField('NMPRODESTO').validations.required = true;
			widget.getField('VRFATOCONV').readOnly = false;
			widget.getField('VRFATOCONV').validations.required = true;
			widget.getField('HIERARQUIAPRODESTO').value('S');
			widget.getField('HIERARQUIAPRODESTO').readOnly = false;
		}
	};

    this.rulesToMoveRow = function(mainWidget) {
    	ScreenService.showLoader();
    	mainWidget.widgets[0].activate().then(function() {
			if (mainWidget.currentRow.__is_new) {	
				this.rulesToInsert(mainWidget);
			} else {
				this.getCorrecao(mainWidget);
				this.allWidgetsToView();
				this.updateWidgets(mainWidget.currentRow);
				ScreenService.hideLoader();
				templateManager.container.getWidget('Detalhamento').activate();
				templateManager.container.getWidget('Detalhamento').redraw();
				mainWidget.openTab('produtos2');
/*				mainWidget.widgets[0].activate().then(function() {
					mainWidget.widgets[0].widgets[0].activate().then(function() {
						mainWidget.widgets[0].widgets[0].activate();
						ScreenService.hideLoader();
					});
				});*/
			}
    	}.bind(this));
    	templateManager.container.getWidget('impostos').openFilter = true;
	};

	this.rulesToInsert = function(mainWidget) {
		var widget = mainWidget.widgets[0];
		General.callRestEngine('cadEst#/crud/paramplaneja', 'FilterData', []).then(function(data) {
			data = data.dataset.paramplaneja[0];
			widget.widgets.forEach(function(widgetChild) {
				this.editMode(widgetChild, true);
				widgetChild.currentRow.__is_new = true;
				widgetChild.currentRow.NRCORSAFRA = data? data.NRCORSAFRA : null;
				widgetChild.currentRow.NRCORINICSAF = data? data.NRCORINICSAF : null;
				widgetChild.currentRow.NRCORENTRSAF = data? data.NRCORENTRSAF : null;
				widgetChild.applyDefaultValue();
			}.bind(this));
			widget.activate();
			ScreenService.hideLoader();
		}.bind(this), this.errorRequest);
	};

	this.rulesToProducao = function(widget) {
		if (this.isProductStock()) {
			widget.getField('NMPRODPROD').readOnly = false;
			widget.getField('VRFATOCONVPR').readOnly = false;
			widget.getField('IDPRODPROD').readOnly = false;
		} else {
			widget.getField('NMPRODPROD').readOnly = true;
			widget.getField('VRFATOCONVPR').readOnly = true;
			widget.getField('IDPRODPROD').readOnly = true;
		}
		this.ruleToProdProducao(widget);
	};

	this.ruleToProdProducao = function(widget) {
		if (widget.currentRow.CDPRODPROD) {
			widget.getField('VRFATOCONVPR').validations.required = true;
		} else {
			delete widget.getField('VRFATOCONVPR').validations.required;
		}
	};
	
	this.rulesTreeGroup = function(widget) {
		var currentRow = widget.currentRow;
		var groupField = widget.getField('NMARVOREPROD');

		if (currentRow.NRTPARVPROD) {
			groupField.dataSourceFilter[0].value = currentRow.NRTPARVPROD;
			groupField.readOnly = false;
			groupField.reload();
		} else {
			groupField.readOnly = true;
		}

	};

    this.save = function(widget) {
    	this.validaFatoConv(widget);
    	this.setCorrecao(widget);
		if (widget.currentRow.__is_new) {
			this.saveNew(widget);
		} else if (widget.isValid()) {
			this.callSave(widget, 'cadEst_produto#/saveProduct');
		}
	};
	
	this.saveNew = function(widget) {
		var save = true;
		var widgets = widget.getParent().widgets;
		
		for (var index = 0; index < widgets.length; index++) {
			var widgetChild = widgets[index];
			var activateWidget = function(){
				this.activate();
			};
			
			if (!widgetChild.isValid()) {
				widgetChild.activate().then(activateWidget.bind(widgetChild));
				save = false;
				index = widgets.length;
			}
		}

		if (save) {
			this.callSave(widget, 'cadEst_produto#/saveProduct');
		}
		
	};

	this.saveFiscal = function(widget) {
		if (widget.currentRow.__is_new) {
			this.save(widget);
		} else if (widget.isValid()) {
			this.callSave(widget, 'cadEst_produto#/saveAbaFiscal');
		}
	};

	this.saveProdutoArvoreProduto = function(widget) {
		if (widget.isValid()) {
			var row = widget.currentRow;
			General.callRestEngine('cadEst_produto#/arvoreprodtree/save', 'Row', row).then(function (success) {
				widget.view();
				ScreenService.closeSwipe();
				widget.getField('NMARVOREPROD').readOnly = true;
			}, this.errorRequest);
		}
	};

	this.getCorrecao = function(widget) {
		var currentRow = widget.currentRow;
		if (currentRow.VRPEFATOCORR != null) {
			var vrpefatocorr = parseFloat(currentRow.VRPEFATOCORR);
			var dspefatocorr = 1 + (vrpefatocorr / 100);
			var dsvryield = 100 - vrpefatocorr;
			widget.currentRow.DSPEFATOCORR = dspefatocorr;
			widget.currentRow.DSVRYIELD = dsvryield;
		}
	};
	
	this.setCorrecao = function(widget) {
		var currentRow = widget.currentRow;

		if (currentRow.DSPEFATOCORR != null) { 
			currentRow.DSPEFATOCORR = String(currentRow.DSPEFATOCORR).replace(',','.'); 
			var dspefatocorr = parseFloat(currentRow.DSPEFATOCORR);
			if(dspefatocorr >= 1 && dspefatocorr < 2){
				dspefatocorr = (dspefatocorr-1) * 100;
				var dsvryield = 100 - dspefatocorr;
				currentRow.VRPEFATOCORR = dspefatocorr;
				widget.currentRow.DSVRYIELD = dsvryield;
			}else{
				if(currentRow.CDPRODESTO == currentRow.CDPRODUTO)
					ScreenService.showMessage('Fator de correção deve estar entre 1,00 e 1,99.');
				currentRow.DSPEFATOCORR = null;
			}
		}
	};

	this.setVrpefatocorr = function (currentRow) {
		var dsvryield = parseInt(currentRow.DSVRYIELD);
		if (dsvryield >= 1 && dsvryield <= 100) {
			currentRow.DSPEFATOCORR = 1 + ((100 - dsvryield) / 100);
			currentRow.VRPEFATOCORR = 100 - dsvryield;
		} else {
			currentRow.DSVRYIELD = null;
			currentRow.VRPEFATOCORR = null;
			currentRow.DSPEFATOCORR = null;
			ScreenService.showMessage('O YIELD deve ser de 1 até 100');
		}
	};
	
	this.setVrYield = function(currentRow) {
		var dspefatocorr = currentRow.DSPEFATOCORR; 
		if (dspefatocorr < 2 && dspefatocorr > 0.99) {
			currentRow.DSVRYIELD = (2 - dspefatocorr) * 100;
			dspefatocorr = parseInt((dspefatocorr - 1) * 100);
			currentRow.VRPEFATOCORR = dspefatocorr;
		} else { 
			currentRow.DSVRYIELD = null;
			currentRow.VRPEFATOCORR = null;
			currentRow.DSPEFATOCORR = null;
			ScreenService.showMessage('O Fator de Correção deve ser de 1,0 até 1,99');
		}
	};

	this.sugestedArvprod = function(widget) {
		var prodnivante = widget.currentRow.CDARVPRODNIVANTE;
		var nvprodnivante = widget.currentRow.CDNVPRODNIVANTE;
		var rigthLevel = (General.getLevelByMask('GER_CDPICTPROD', 1).length - 1);
		if (nvprodnivante && nvprodnivante != rigthLevel) {
			ScreenService.showMessage('Produto selecionado inválido. Só é permitido selecionar produtos de nível ' + rigthLevel + '.');
			
			/* MSG NOVA */
			var variaveis = {
                NIVELCOMPLETO: rigthLevel
            };
			
			MessageService.showMessageByCode("NIVELMAXPROD", variaveis, {"type":"error"});
			/* FIM MSG NOVA */
			
			widget.currentRow.CDPRODNIVANTE = null;
			widget.currentRow.CDARVPRODNIVANTE = null;
			widget.currentRow.CDNVPRODNIVANTE = null;
		} else if (nvprodnivante) {
			var row = {
				'PARENTCDPRODUTO': prodnivante,
				'PARENTNVPRODUTO': nvprodnivante
			};
			if (!widget.currentRow.CDARVPROD) {
				General.callRestEngine('cadEst_produto#/sugestedArvprod', 'Row', row).then(function(request) {
					var sugested = request.dataset.sugestedArvprod[0];
					widget.currentRow.CDARVPROD = sugested.CDDARVPROD;
				}.bind(this), this.errorRequest);
			}
		}
	};
	
	this.updateProductInMainWidget = function() {
		templateManager.container.getWidget('produtos').reload();
	};

	this.updateWidgets = function(valueToUpdate) {
		var widget = templateManager.container.getWidget('produtos2');
		valueToUpdate = Util.clone(valueToUpdate, ignoredKeys);
		widget.widgets.forEach(function(widget) {
		    widget.currentRow = valueToUpdate;
		});
	};
	
	this.updateWidgetsIfAdding = function(widget) {
		if (widget.currentRow.__is_new) {
			this.updateWidgets(widget.currentRow);
		}
	};

	this.validateTpTree = function(widget) {
		var row  = widget.currentRow;
		if (!row.NRARVOREPRODPAI) {
			delete row.NMTPARVORE;
			delete row.NRARVOREPROD;
			delete row.NRTPARVPROD;		
			delete row.NMARVOREPROD;
			ScreenService.showMessage('Não é possível escolher o primeiro nível do Agrupamento de Produto.');
		}
	};
	
	this.deleteProduct = function(widget){
    	var row = Util.clone(widget.currentRow, ignoredKeys);
		row.DEBUGMODE = false;
		General.callRestEngine('cadEst#/deleteProduct', 'Row', row).then(result =>{
			if(isset(() => result.messages)){
				ScreenService.showMessage(result.messages[0].message);
			}else{
				widget.getParent().getParent().reload();
				ScreenService.closeSwipe();
			}
		});
    };
	
	this.newProduto = function(widget){
		var detalhamento = widget.container.getWidget('Detalhamento');
		var codArvore = detalhamento.getField('CDARVPRODNIVANTE');
		widget.newRow();
		codArvore.dataSource.load();
	};
	
	this.LoadCodArvore = function(widget){
		var nivelProd = widget.getField('CDARVPRODNIVANTE');
		var fieldTpProduto = widget.getField('IDESTOCOMP');
		nivelProd.dataSource.load();
		var data = nivelProd.dataSource.data;
		
		if(data.length === 0){
			delete nivelProd.validations.required;
			fieldTpProduto.readOnly = false;
		}else{
			nivelProd.validations.required = true;
			if (widget.currentRow.CDARVPROD != '' && widget.currentRow.__is_new)
				fieldTpProduto.readOnly = false;
			else
				fieldTpProduto.readOnly = true;
		}
	};
	
	this.validaFator = function (widget) {
		var currentRow = widget.currentRow;
		if (currentRow.VRFATOCONV <= 0) {
			ScreenService.showMessage('Fator de Conversão deve ser maior que 0.');
			currentRow.VRFATOCONV = "";
		}
	};
	
	this.validateProductPermissions = function (widget){
		var cdnvarvore = widget.currentRow.CDARVPRODNIVANTE;
		var fieldTpProduto = widget.getField('IDESTOCOMP');
		var filter = [
                {
                    "name": "P_CDPRODUTO",
                    "operator": "=",
                    "value": cdnvarvore
                },
                {
                	"name": "CDOPERADOR",
                    "operator": "=",
                    "value": "OPERADOR_LOGADO"
                }
		    ];
		General.callRestEngine('cadEst#/crud/gropernvprod_operador', 'FilterData', filter).then(response => {
			if(response.dataset.gropernvprod_operador[0]){
				let resultRow = response.dataset.gropernvprod_operador[0];
				if(resultRow.IDPERMINCPRODEST == 'S')
					fieldTpProduto.readOnly = false;
				else{
					fieldTpProduto.readOnly = true;
					widget.currentRow.IDESTOCOMP = 'C';
				}
			}else{
				fieldTpProduto.readOnly = false;
			}
				
		});
		
	};
	
	this.setStockProductFilter = function(args){
		let field = args.owner.field;
		let valueToFilter = General.getFieldValue('CDARVPRODNIVANTE', 'Detalhamento');
		let validaHierarquia = General.getFieldValue('HIERARQUIAPRODESTO', 'Estoque');
		
		if(validaHierarquia == 'S')
			valueToFilter = valueToFilter? valueToFilter+'%' : '%%';
		else
			valueToFilter = '%%';
			
		field.dataSourceFilter[0].value = valueToFilter;
	};
	
	this.openPopupCodBar = function (){
		let widPopupCod = templateManager.container.getWidget('popUpCodigoBarra');
		let widgetProduto = templateManager.container.getWidget('produtos');
		let nomeProd = widgetProduto.currentRow.NMPRODUTO;
		widPopupCod.label = 'Codigos de Barra - '+nomeProd;
		ScreenService.openPopup(widPopupCod);
	};
	
	this.saveCodeBars = function(args){
		let rows = args.dataSource;
		let parent = args.owner.getParent().getWidget('produtos');
		let codprod = parent.currentRow.CDPRODUTO;
		rows.forEach(row => {
			if(row.__is_new){
				row.CDPRODUTO = codprod;
				delete row.__leftActions;
				delete row.__rightActions;
			}
		});
		let edited = rows.some( row => {return row.changed && !row.__is_new;});
		if(edited)
			ScreenService.showMessage('Para alterar o registro deve ser excluído e adicionado novamente.');
		let rowsToSave = rows.filter(row =>{
			return row.__is_new;
		});
		return saveDataset('cadEst#/crud/cdbarraprod/save', rowsToSave).then((success)=>{
			args.owner.reload();
			args.owner.cancelActionClick();
		}, (error)=>{
			ScreenService.showMessage(error.data.error);
			args.owner.cancelActionClick();
		});
	};
	
	var saveDataset = function(dataSourceName, dataToSave, otherParams = {}) {
        return restEngine.post(dataSourceName, Util.extend({
            "requestType": "DataSet",
            "dataset": dataToSave
        }, otherParams || {}), undefined, undefined, undefined, undefined, 8001000);
    };
    
    this.newCodeBar = function(widget){
    	General.widgetNewRow(widget);
    };
    
    this.deleteCodeBar = function(args){
    	let widget = args.owner.widget;
    	General.deleteCrud(widget).then(()=>widget.reload());
    };
    
    this.productFieldFilterHandler = function(field) {
		let widget = field.widget;
		let fieldProdIni = widget.getField('NMPRODINI');
		let fieldProdFim = widget.getField('NMPRODFIN');
		
		switch(field.name){
			case "NMPRODINI":
				if(fieldProdIni.value()!== ""){
					fieldProdIni.validations = { "required": true };
					fieldProdFim.validations = { "required": true };
					fieldProdFim.readOnly = false;
				}else{
					widget.currentRow.NMPRODINI = '';
					widget.currentRow.NMPRODFIN = '';
					widget.currentRow.CDPRODINI = '';
					widget.currentRow.CDPRODFIN = '';
					delete fieldProdIni.validations;
					delete fieldProdFim.validations;
					fieldProdFim.readOnly = true;
				}
				break;
				
			case "NMPRODFIN":
				if(widget.currentRow.NMPRODINI === '' && widget.currentRow.NMPRODFIN === ''){
					delete fieldProdIni.validations;
					delete fieldProdFim.validations;
					fieldProdFim.readOnly = true;
				}
				break;
			default:
				return;
		}
		this.checkProductRange(field);
	};
	
	this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;
      
        if(currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN)? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : '';

            if(!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = '';
                currentRow[field.outData[field.valueField]] = '';

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        } 
    };
    
    var isset = (accessor) => {
	  try {
	    // Note we're seeing if the returned value of our function is not
	    // undefined
	    return typeof accessor() !== 'undefined' && accessor() !== null;
	  } catch (e) {
	    // And we're able to catch the Error it would normally throw for
	    // referencing a property of undefined
	    return false;
	  }
	};
    
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('ger01900_cadastroproduto_controller', ger01900_cadastroproduto_controller);
});

// FILE: ./assets/modules/produto/assets/js/controllers/ger01900_relatoriocustomizavelproduto_controller.js
function ger01900_relatoriocustomizavelproduto_controller(ZhFilterPreferences, ScreenService, General, templateManager, localStorageEngine) {

	this.enableDisableData = function(widget) {
		var PARAM = widget.currentRow.PARAM;

		if(PARAM == 'D'){
			widget.getField('NRDIAS').readOnly = false;
			widget.getField('DATAINI').readOnly = true;
			widget.getField('DATAFIN').readOnly = true;
			
			widget.getField('NRDIAS').validations = {"min": {"limit": 1}, "max": {"value": 999}};
			widget.getField('DATAINI').validations = {};
			widget.getField('DATAFIN').validations = {};
		}else if(PARAM == 'P'){
			widget.getField('NRDIAS').readOnly = true;
			widget.getField('DATAINI').readOnly = false;
			widget.getField('DATAFIN').readOnly = false;
			
			widget.getField('NRDIAS').validations = {};
			widget.getField('DATAINI').validations = {"required": true};
			widget.getField('DATAFIN').validations = {"required": true};
		}
		
		widget.getField('NRDIAS').reload();
		widget.getField('DATAINI').reload();
		widget.getField('DATAFIN').reload();
	
	};

    this.openFilter = function(widget){
        var rowsToSet = {};
        
        var levels = General.getLevelByMask("GER_CDPICTPROD", 1);
		var nivelField = widget.getField("NIVEL");
		var data = [];
		levels.forEach(function(level) {
		    data.push({value: level.nivel, description: level.nivel});
		});
		
		nivelField.dataSource.data = data;
            rowsToSet.TIPODEORDENACAO = 'CODIGO';
            rowsToSet.NIVEL = String(data.length);
            rowsToSet.TPLISTPROD = '2';
            
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(function(){
                // templateManager.container.getActiveWidget().getAction('close').isVisible = false;
                // this.changeFilial(widget);
            }.bind(this));
    };
    
    this.habilitaNvOrdenacao = function(args){
        var widget = args.owner.field.widget;
        if(widget.getField('TIPODEORDENACAO').value() == 'NOME'){
            templateManager.container.getWidget('produtos').order = ["NMPRODUTO"];
            widget.getField('NIVEL').readOnly = false;
            widget.getField('TPLISTPROD').readOnly = true;
            widget.getField('TPLISTPROD').value(String(2));
            
            templateManager.container.getWidget('produtos').dataSource.name = "cadEst#/crud/customizavel_produto_completa_alfabetica";
            templateManager.container.getWidget('produtos').floatingControl.xlsAction.dataSourceName = "customizavel_produto_completa_alfabetica";
        }else{
            templateManager.container.getWidget('produtos').order = ["CDARVPROD"];
            var tam = widget.getField('NIVEL').dataSource.data.length;
            widget.getField('NIVEL').readOnly = true;
            widget.getField('NIVEL').value(String(tam));
            widget.getField('TPLISTPROD').readOnly = false;
            
            templateManager.container.getWidget('produtos').dataSource.name = "cadEst#/crud/customizavel_produto_completa";
            templateManager.container.getWidget('produtos').floatingControl.xlsAction.dataSourceName = "customizavel_produto_completa";
        }
    };
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
	//Parâmetros Filtro da Widget
    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        var filtro = [];
        
        var mascara = 20 - filterRow.CDARVPRODFIN.length;
		var produtoFinal = filterRow.CDARVPRODFIN + 'z'.repeat(mascara).substring(0, mascara);
		
        filtro = [
            {
                "name":"P_PRODINI",
                "operator":"=",
                "value": filterRow.CDARVPRODINI ? filterRow.CDARVPRODINI : null
            },
            {
                "name":"P_PRODFIM",
                "operator":"=",
                "value": produtoFinal
            },
            {
                "name": "P_NIVEL",
                "operator": "=",
                "value": filterRow.NIVEL
            },
            {
                "name": "P_TPLISTPROD",
                "operator": "=",
                "value": filterRow.TPLISTPROD
            }
        ];
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    this.changeFilterLancEstoq = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        if(dtlancestq != ""){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
        }
    };
    
    this.changeFilterLancEstoqByFilial = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        args.owner.field.widget.currentRow.NRLANCESTQ = "";
        if(dtlancestq != ""){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
        }
    };

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ger01900_relatoriocustomizavelproduto_controller', ger01900_relatoriocustomizavelproduto_controller);
});


// FILE: ./assets/modules/produto/assets/js/controllers/ger01901_cadastrotipoarvoreproduto_controller.js
var ger01901_cadastrotipoarvoreproduto_controller = function(ScreenService, General) {

	var G_NRARVPRODPAI;
	var G_NMARVPRODPAI;
	var G_NRTPARVPRODPAI;


	this.desabilitaEHabilitaEstrutura = function(widget) {
		widget.container.getWidget("estrutura").getParent().activate();
		widget.container.getWidget("estrutura").isVisible = true;
	};

	this.filtroParaNivelPai = function(widget) {
		var field = widget.getField('NMARVOREPRODPAI');
		if (typeof widget.currentRow.NRARVOREPROD !== "undefined") {
			field.dataSourceFilter.push({
				"name": "NRARVOREPROD",
				"operator": "<>",
				"value": widget.currentRow.NRARVOREPROD
			});
		}
		field.reload();
	};

	this.getArvProdPai = function(widget) {
		G_NRARVPRODPAI = widget.currentRow.NRARVOREPROD;
		G_NMARVPRODPAI = widget.currentRow.NMARVOREPROD;
	};

	this.novoItemEstrutura = function(widget) {
		var novoNrarvoreprodpai;
		var novoNmarvoreprodpai;
		if (widget.mode === "view") {
			novoNrarvoreprodpai = typeof(widget.currentRow.NRARVOREPROD) !== "undefined" ? widget.currentRow.NRARVOREPROD : null;
			novoNmarvoreprodpai = typeof(widget.currentRow.NMARVOREPROD) !== "undefined" ? widget.currentRow.NMARVOREPROD : null;
		}
		else {
			novoNrarvoreprodpai = null;
			novoNmarvoreprodpai = null;
		}

		widget.edit();
		widget.newRow();
		widget.getField('NMARVOREPRODPAI').setValue(novoNmarvoreprodpai);
		widget.currentRow.NRARVOREPRODPAI = novoNrarvoreprodpai;
		widget.currentRow.NRTPARVPRODPAI = widget.currentRow.NRTPARVPROD;

	};

	this.novoTipoArvore = function(widget) {
		widget.newRow();
	};

	this.onEnterEstrutura = function(widget) {
		if (widget.mode == "list") {
			widget.reload();
		}
	};

	this.onEnterTabEstrutura = function(widget) {
		var nrtparvprod = widget.container.getWidget('tipoarvoreproduto').currentRow.NRTPARVPROD;

		var filter = [
			{
				"name": "NRTPARVPROD",
				"operator": "=",
				"value": nrtparvprod
			}
		];

		widget.dataSourceFilter = filter;
	};

	this.onEnterTabTipoArvore = function(widget) {
		if (widget.mode == "edit") {
			widget.container.getWidget("estrutura").enabled = false;
		}
		else {
			widget.container.getWidget("estrutura").enabled = true;
		}
	};

	this.salvarNovoItemEstrutura = function(widget) {
		widget.reload().then(_ => {
			let currentRow = widget.currentRow;
			const widgetTipoArvProd = widget.container.getWidget('tipoarvoreproduto');
			const RowGroupName = widget.currentRow.NMARVOREPROD;

			currentRow.NRTPARVPROD = widgetTipoArvProd.currentRow.NRTPARVPROD;
			currentRow.NRTPARVPRODPAI = widgetTipoArvProd.currentRow.NRTPARVPROD;
			currentRow.NRARVOREPRODPAI = !(typeof currentRow.NRARVOREPRODPAI === "undefined" || currentRow.NRARVOREPRODPAI === null) ? currentRow.NRARVOREPRODPAI : null;

			if (!currentRow.__is_new && verificaParentesco(widget)) {
				if (this.checkGroupingAndTypeTree(widget, RowGroupName) == false) {
					General.saveCrud(widget).then(function() {
						widget.list();
						widget.reload().then(function() {
							widget.view();
							widget.redraw();
						});
					});
				}
			}
			else if (currentRow.__is_new) {
				if (this.checkGroupingAndTypeTree(widget, RowGroupName) == false) {
					General.saveCrud(widget).then(function() {
						currentRow = {};
						widget.reload();
					});
				}
			}
			else {
				ScreenService.showMessage("O grupo nao pode ser movido para um de seus grupos filhos");
			}
		});

	};

	this.salvarTipoArvore = function(widget) {
		const nomeTipoArvore = widget.currentRow.DSTPARVPROD;
		const widgetAgrupArvProd = widget.container.getWidget("estrutura");

		if (!widget.isValid()) return;

		widget.reload().then(_ => {

			if (this.checkGroupingAndTypeTree(widget, nomeTipoArvore) == false) {
				General.saveCrud(widget).then(() => {
					if (!widget.currentRow.__is_new) {
						widget.list();
						widgetAgrupArvProd.enabled = true;
						widget.reload().then(_ => {
							widget.view();
							widget.redraw();
						});
					}
					else {
						widget.currentRow = {};
						widget.reload();
					}
					widgetAgrupArvProd.enabled = true;
				});
			}
		});
	};

	this.checkGroupingAndTypeTree = (widget, currentRowName) => {
		const data = widget.dataSource.data;
		const dataMin = data.length - 1;
		const currentRow = widget.currentRow;

		if (data.length == 0) return false;
		for (let index in data) {
			if (widget.name === 'tipoarvoreproduto') {
				if (data[index].DSTPARVPROD == currentRowName) {
					ScreenService.showMessage("Impossível adicionar. Tipo de Árvore já existente.");
					widget.reload();
					break;
				}
			}
			else {
				if (currentRow.NRARVOREPRODPAI == null || currentRow.NRARVOREPRODPAI == "") {
					if (data[index].NMARVOREPROD == currentRowName) {
						if ((currentRow.__is_new == true && currentRow._level == 1) || (currentRow.__is_new == false && currentRow._level != 1)) {
							ScreenService.showMessage("Impossível adicionar. Agrupamento de Árvore já existente.");
							widget.reload();
							break;
						}
					}
				}
			}
			if (index == dataMin)
				return false;
		}
	};

	var verificaParentesco = function(widget) {

		var valido = true;

		if (!widget.currentRow.__is_new) {
			var rows = widget.dataSource.data;
			var nrarvoreprodpai = widget.currentRow.NRARVOREPRODPAI !== "undefined" && widget.currentRow.NRARVOREPRODPAI !== null ? widget.currentRow.NRARVOREPRODPAI.toString() : "";
			var nrarvoreprod = widget.currentRow.NRARVOREPROD.toString();
			rows.forEach(function(row) {
				if (row.NRARVOREPROD === nrarvoreprodpai) {
					if (row.NRARVOREPRODPAI === nrarvoreprod)
						valido = false;
				}
			});
		}

		return valido;

	};
};
Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('ger01901_cadastrotipoarvoreproduto_controller', ger01901_cadastrotipoarvoreproduto_controller);
});


// FILE: ./assets/modules/produto/assets/js/controllers/ger01902_arvoreproduto_controller.js
var ger01902_arvoreproduto_controller = function(ScreenService, templateManager, General, metaDataFactory) {
    
    var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    var nivAnteOnCreation;
    
    this.calcMask = function(nvproduto) {
        var mask = getLocalVar('GER_CDPICTPROD');
        mask = mask.split('.');
        mask = mask.slice(0, parseInt(nvproduto));
        return mask.join('.');
    };

    this.cancelEditOrAdd = function(args) {
        var widget = args.owner.widget;
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        } else {
            General.backOrCancel(args);
        }
        this.backToDefaultMask();
    };
    
    this.backToDefaultMask = function() {
        var widget = templateManager.container.getWidget('arvoreProdutoGrid');
        var arvprodField = widget.getField("CDARVPROD");
        
        arvprodField.mask.params.mask = getLocalVar('GER_CDPICTPROD');
        //var newField = metaDataFactory.fieldFactory(arvprodField, widget);
        //widget.fields.splice(0, 1, newField);
    };

    this.deleteProduct = function(widget) {
        let parentOnDataset = false;
        var row = Util.clone(widget.currentRow, ignoredKeys);
        
        if(widget.currentRow._parent)
            parentOnDataset = widget.dataSource.data.filter(row => row.CDARVPROD == widget.currentRow._parent.CDARVPROD)[0];
        
        
        General.callRestEngine('cadEst_produto#/produto_para_arvore/remove', "Row", row).then(function() {
            ScreenService.closeSwipe().then(function() {
                widget.list();
                if(parentOnDataset){
                    parentOnDataset._childCount -= 1;
                    if(parentOnDataset._childCount <= 0){
                        parentOnDataset._hasChild = false;
                        parentOnDataset._opened = false;
                    }
                }
                widget.dataSource.data = widget.dataSource.data.filter(data => {
                    return data != widget.currentRow;
                });
            }); 
        }.bind(this), this.errorRequest); 
    };

    this.errorRequest = function(error) {
        if (error.data.error) {
            ScreenService.showMessage(error.data.error);
        } else {
            ScreenService.showMessage(error);
        }
        ScreenService.hideLoader();
    };

    this.newTree = function(widget) {
        ScreenService.showLoader();
        widget.prodnivante = Util.clone(widget.currentRow, ignoredKeys);
        nivAnteOnCreation = widget.dataSource.data.filter(row => row.CDARVPROD == widget.prodnivante.CDARVPROD)[0];
        var row = {
            'PARENTCDPRODUTO': widget.prodnivante.CDARVPROD || null,
            'PARENTNVPRODUTO': widget.prodnivante.CDNVPRODUTO || null
        };
        General.callRestEngine('cadEst_produto#/sugestedArvprod', 'Row', row).then(function(request) {
            var arvprodField = templateManager.container.getWidget('arvoreProdutoGrid').getField("CDARVPROD");
            var sugested = request.dataset.sugestedArvprod[0];
            
            var parentMask = (parseInt(widget.prodnivante.CDNVPRODUTO || 0) + 1);
            var mask = this.calcMask(parentMask);
            arvprodField.mask.params.mask = mask;
            widget.newRow();
            
            widget.currentRow.CDNVPRODUTO = (parseInt(widget.prodnivante.CDNVPRODUTO || 0) + 1);
            widget.currentRow.CDPRODNIVANTE = widget.prodnivante.CDPRODUTO || null;
            widget.currentRow.CDARVPRODNIVANTE = widget.prodnivante.CDARVPROD || null;
            widget.currentRow.CDARVPROD = sugested.CDDARVPROD;
            //var newField = metaDataFactory.fieldFactory(arvprodField, widget);
            //widget.fields.splice(0, 1, newField);
            
            ScreenService.hideLoader();
        }.bind(this), this.errorRequest);
        
    };

    this.newTreeLevelOne = function(widget) {
        widget.currentRow = {};
        this.newTree(widget);
    };

    this.openSwipe = function(widget) {
        ScreenService.openSwipe(widget).then(function() {
            this.rulesOpenSwipe(widget);
        }.bind(this));
    };

    this.penultimateLevel = function(level) {
        var maxLevel = General.getLevelByMask('GER_CDPICTPROD', 1).length;
        if ((maxLevel -1) > level) {
            return true;
        } else {
            return false;
        }
    };
    
    this.rulesOpenSwipe = function(widget, afterSave = false) {
        //var arvprodField = templateManager.container.getWidget('arvoreProdutoGrid').getField("CDARVPROD");
        
        if (widget.mode != 'edit' && !widget.currentRow.__is_new) {
            // widget.view();
            if (widget.currentRow._hasChild || this.penultimateLevel(widget.currentRow.CDNVPRODUTO)) {
                widget.getAction('adicionar').isVisible = true;
            } else {
                widget.getAction('adicionar').isVisible = false;
            }

            if (widget.currentRow._hasChild) {
                widget.getAction('excluir').isVisible = false;
            } else {
                widget.getAction('excluir').isVisible = true;
            }
        }
        if(afterSave){
            if (widget.currentRow._hasChild || this.penultimateLevel(widget.currentRow.CDNVPRODUTO)) {
                widget.getAction('adicionar').isVisible = true;
            } else {
                widget.getAction('adicionar').isVisible = false;
            }

            if (widget.currentRow._hasChild) {
                widget.getAction('excluir').isVisible = false;
            } else {
                widget.getAction('excluir').isVisible = true;
            }
        }
        //var mask = this.calcMask(widget.currentRow.CDNVPRODUTO || 1);
        // arvprodField.mask.params.mask = mask;
        //var newField = metaDataFactory.fieldFactory(arvprodField, widget);
        //widget.fields.splice(0, 1, newField);
    };

    this.save = function(widget) {
        var row = Util.clone(widget.currentRow, ignoredKeys);
        row.CDARVPROD = valueDesmasked(row.CDARVPROD.toUpperCase());
        
        if (widget.isValid()) {
            General.callRestEngine('cadEst_produto#/produto_para_arvore/save', "Row", row).then(function(success) {
				widget.activate();
				this.rulesOpenSwipe(widget, true);
				if (widget.currentRow.__is_new) {
                    let retCdproduto = success.dataset.columnsToReturn[0].CDPRODUTO;
                    widget.currentRow.CDPRODUTO = retCdproduto;
				    widget.currentRow.__is_new = false;
				    widget.getField('CDARVPROD').mask.params.mask= "@getLocalVar('GER_CDPICTPROD')@";
					widget.view();  
					if(nivAnteOnCreation || row.CDNVPRODUTO > 1){
    					if(!nivAnteOnCreation? true : !nivAnteOnCreation._opened){
        					widget.dataSource.data.pop(); 
    					} 
    					if(nivAnteOnCreation){
                            nivAnteOnCreation._hasChild = true;
                            nivAnteOnCreation._childCount += 1;
    					} 
					}
					widget.getAction('excluir').isVisible = true;
				} else {
					widget.view();
				}
			}.bind(this), this.errorRequest);
        }
    };
    
    this.newGroupOperRules = function(widgetGroupOper){
        let widgetProductTree = widgetGroupOper.getParent();
        widgetGroupOper.currentRow.CDPRODUTO = widgetProductTree.currentRow.CDPRODUTO;
    };
    
    this.saveMaskAction = function(widget){
        let gridArvore = widget.getParent();
        let fieldMask = widget.getField('CDPICTPROD'); 
        fieldMask.value(fieldMask.value().toLowerCase());
        widget.dataSource.name = 'cadEst#/paramgeralMask';
        General.saveCrud(widget).then(success => {
            widget.dataSource.name = 'cadEst#/crud/paramgeral';
            ScreenService.closePopup();
            ScreenService.closeSwipe();
            gridArvore.activate();
        });
    };
    
    this.preventUnwantedKeys = function(args){
        let arrAllowedKeys = ['9','a','A','.'];
        if(!arrAllowedKeys.includes(args.e.key)){
            args.e.preventDefault();
        }
    };
    
    this.setCurrentRowWithData = function (widget){
        widget.currentRow = widget.dataSource.data[0];
    };
    
    this.changeMaskVisibility = function(widget){
        let maskWidget = widget.container.getWidget('maskPopup');
        let fieldMask  = maskWidget.getField('CDPICTPROD');
        
        fieldMask.readOnlyOnEdit = widget.dataSource.data.length == 0?  false : true;
        fieldMask.readOnly       = widget.dataSource.data.length == 0?  false : true;
    };
    
    this.codArvToUpperCase = function(widget){
        let fieldCodArv = widget.getField('CDARVPROD');
        fieldCodArv.value(valueDesmasked(widget.currentRow.CDARVPROD.toUpperCase()));
    };
    
    var valueDesmasked = function(value){
        return value.split('.').join('');
    };
    
    this.changeDataSourceBeforeReport = function(widget){
      console.log(widget);  
    };
    
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('ger01902_arvoreproduto_controller', ger01902_arvoreproduto_controller);
});

// FILE: ./assets/modules/produto/assets/js/controllers/ger01903_impostoproduto_controller.js
var ger01903_impostoproduto_controller = function(ScreenService, templateManager, General, ZhFilterPreferences) {
    
    var self = this;
    
    this.openFilter = function(args){
        var widget = args.owner;
        ZhFilterPreferences.startFiltering(widget);
    };
    
    this.setUnitImposto = function(widget){
        widget.currentRow.CDFILIAL = General.getFieldValue('CDFILIAL', 'impostos');
    };
    
    this.configAfterApply = function(widget){
        var widgetImposto = templateManager.container.getWidget('impostos');
        widgetImposto.currentRow.CDFILIAL = widget.currentRow.CDFILIAL;
         widgetImposto.dataSourceFilter[4].value = widget.currentRow.CDFILIAL;
        widgetImposto.reload().then(function(response){
            widgetImposto.currentRow = response.dataset.paraVendasImpostoProduto[0];
        });
    };
    
    this.loadDataImposto = function(args){ 
        var widget = args.owner;
        var widgetImpo = templateManager.container.getWidget('impostos');
        var fieldPIS    = widgetImpo.getField(VRALIQPIS_FIELD),
            fieldCOFINS = widgetImpo.getField(VRALIQCOFINS_FIELD);
        widgetImpo.dataSourceFilter[3].value =  widget.__filterPopup.currentRow.CDFILIAL;
        widgetImpo.reload().then(function(data){
            widgetImpo.currentRow = data.dataset.paraVendasImpostoProduto[0];
             templateManager.container.getWidget('prodConfig').reload();
            fieldPIS.readOnly    = 
            fieldCOFINS.readOnly = !(!!(widgetImpo.currentRow.CDCSTPISCOF) && !!(widgetImpo.currentRow.CDCFOPPFIS));
            
        });
        
    };
    
    this.configTelaOnChange = function(args){
        var field = args.owner.field;
        var fieldDSBUTTON = field.widget.getField('DSBUTTON');     
            fieldDSBUTTON.readOnly = (field.value() == null || field.value() == '');
            fieldDSBUTTON.dataSourceFilter[2].value = field.widget.currentRow.NRCONFTELA;
            fieldDSBUTTON.value(null);
    };
    
    const IMPOSTPROD_WIDGET         = 'paraVendaImpostoProduto',
          IMPOSTPRODPOPUP_WIDGET    = 'paraVendaImpostoProdutoPopup',
          NMPRODINICIAL_FIELD       = 'NMPRODINICIAL',
          NMPRODFINAL_FIELD         = 'NMPRODFINAL',
          CDCSTICMS_FIELD           = 'CDCSTICMS',
          CDCSTPISCOF_FIELD         = 'CDCSTPISCOF',
          VRALIQPIS_FIELD           = 'VRALIQPIS',
          VRALIQCOFINS_FIELD        = 'VRALIQCOFINS',
          CDCEST_FIELD              = 'CDCEST',
          NMPRODUTOFIN_FIELD        = 'NMPRODUTOFIN';
          
    
    this.editImposto = function (widget) {
		General.widgetEdit(widget);
		
		widget.currentRow.CDFILIAL = Array.isArray(widget.currentRow.CDFILIAL) ? widget.currentRow.CDFILIAL : [widget.currentRow.CDFILIAL];
		widget.currentRow.NMFILIAL = Array.isArray(widget.currentRow.NMFILIAL) ? widget.currentRow.NMFILIAL : [widget.currentRow.NMFILIAL];
		widget.getField('CDFILIAL').reload();
    };
    
    this.cancelNewTax = function(args){
        var widget = args.owner.widget;
        General.widgetCancelForm(widget);
        delete widget.getField('CDFILIAL').validations.required;
    };
    
    this.saveImposto = function (widget) {
        if (widget.isValid()) {
            widget.saveRow().then(function(response) {
			    widget.currentRow.changed = false;
				widget.reload();
				widget.view();
			}).catch(function(response) {
				ScreenService.errorNotification(response.data.error);
			});
        }
    };
    
    this.impostosAfterMoveRow = function(args){
        var widget = args.owner;
        widget.activate();

        setAliquotaVisibility(widget, false);
        
        widget.fields.forEach(function (field){ 
            if(field.dataSourceFilter !== undefined){
                field.dataSourceFilter.forEach(function (filter) {
                    if (filter.name == 'CDFILIAL'){
                        filter.value = widget.currentRow.CDFILIAL;
                    }
                });
            }
        });
    };
    
    this.setCDSITUTRIBorCDSITUTRIBNFE = function (args) {
        var widget = args.owner.field.widget;   
        
        widget.currentRow.CDSITUTRIB = widget.currentRow.CDCSTPISCOF;
        widget.currentRow.CDSITUTRIBNFE = widget.currentRow.CDCSTPISCOF;
    };
    
    this.setClearImposto = function (args) {
        var widget = args.owner.field.widget;  
        widget.currentRow.CDCSTICMS = '';
        widget.getField('CDCSTICMS').value('');
    };
    
    var setAliquotaVisibility = function (widget, setNull) {
        var fieldPIS    = widget.getField(VRALIQPIS_FIELD),
            fieldCOFINS = widget.getField(VRALIQCOFINS_FIELD);
        
        fieldPIS.readOnly    = 
        fieldCOFINS.readOnly = !(!!(widget.currentRow.CDCSTPISCOF) && !!(widget.currentRow.CDCFOPPFIS));
        
        if (setNull)
            widget.currentRow.VRALIQCOFINS  = 
            widget.currentRow.VRALIQPIS     = null;
        
        self.updateDataSourceFilter(widget, fieldPIS);
        self.updateDataSourceFilter(widget, fieldCOFINS);
        
        widget.activate();
    };
    
    this.updateDataSourceFilter = function (widget, widgetToUpdate) {
    	widgetToUpdate.dataSourceFilter.forEach(function(filter) {
			
			filter.value = Util.clone(widget.currentRow[filter.name]);
    		
    		if(filter.name === 'NRORG') {
    			filter.value = getLocalVar('NRORG');
    		}
    	});
    	return widgetToUpdate.reload();
    };
    
    this.isAliquotaVisible = function (widget) {
        
        setAliquotaVisibility(widget, true);
    };
    
    this.saveImpostoProduto = function (widget) {
        var row = widget.currentRow;
		row.CDNVPRODUTO = General.getLevelByMask('GER_CDPICTPROD', 1).length;
        if (widget.isValid()) {
            
            row.VRPEALIMPFIS    = General.valueToFloat(row.VRPEALIMPFIS);
            row.VRPERCREDUCAO   = General.valueToFloat(row.VRPERCREDUCAO);
            row.VRALIQIBPT      = General.valueToFloat(row.VRALIQIBPT);
            row.VRALIQIBPTES    = General.valueToFloat(row.VRALIQIBPTES);
            
            if(parseFloat(row.VRPEALIMPFIS < 0)){
                row.VRPEALIMPFIS = row.VRPEALIMPFIS * (-1);
            }
            
            if(parseFloat(row.VRPERCREDUCAO < 0)){
                row.VRPERCREDUCAO = row.VRPERCREDUCAO * (-1);
            }
            
            if(parseFloat(row.VRALIQIBPT < 0)){
                row.VRALIQIBPT = row.VRALIQIBPT * (-1);
            }
            
            if(parseFloat(row.VRALIQIBPTES < 0)){
                row.VRALIQIBPTES = row.VRALIQIBPTES * (-1);
            }
                 
            if(!row.CDCSTPISCOF){
                row.VRALIQPIS = 0;
                row.VRALIQCOFINS = 0;
            }
            General.widgetSave(widget).then(function(response){
                if(response.messages[0].type == 'W'){
                    ScreenService.showMessage(response.messages[0].message, 'alert');
                }else {
                    ScreenService.showMessage('Certifique-se de que a aba Produto->Fiscal foi parametrizada. Unidades já cadastradas, foram alteradas!', 'alert');
                }
                delete widget.getField('CDFILIAL').validations.required;
    		    widget.reload();
            }).catch(function(response) {
                ScreenService.errorNotification(response.data.error);
            });
        }
    };
    
	this.newProdTax = function(args) { 
		var widget = args.owner.widget;
		widget.getField('CDFILIAL').validations.required = true;
		General.widgetNewRow(widget);
		widget.currentRow.NRORG = getLocalVar('NRORG');
		widget.currentRow.CDPRODUTO = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		widget.currentRow.CDARVPROD = General.getFieldValue('CDARVPROD', 'Detalhamento');
		widget.currentRow.NMPRODUTO = General.getFieldValue('NMPRODUTO', 'Detalhamento');
		widget.currentRow.CDPRPAITABPR = General.getFieldValue('CDPRODUTO', 'Detalhamento');
		widget.currentRow.CDBARPRODUTO = General.getFieldValue('CDBARPRODUTO', 'Detalhamento');
		if(widget.appliedFilter.FILTER_ROW.CDFILIAL !== ''){
			widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
		}
		widget.reload();
	};
	
	this.exportImposto = function(widget) {
	    var exportPopup = templateManager.container.getWidget('exportImpos');
	    ScreenService.openPopup(exportPopup);
    	exportPopup.currentRow = widget.currentRow;
    	exportPopup.currentRow.CDFILIALORIG = exportPopup.currentRow.CDFILIAL;
    	delete exportPopup.currentRow.CDFILIAL;
	    templateManager.container.getWidget('impostos').activate();
	};
    
    this.saveExport = function (widget) {
        var row = widget.currentRow;
		row.CDNVPRODUTO = General.getLevelByMask('GER_CDPICTPROD', 1).length;
        if (widget.isValid()) {
            
            row.VRPEALIMPFIS    = General.valueToFloat(row.VRPEALIMPFIS);
            row.VRPERCREDUCAO   = General.valueToFloat(row.VRPERCREDUCAO);
            row.VRALIQIBPT      = General.valueToFloat(row.VRALIQIBPT);
            row.VRALIQIBPTES    = General.valueToFloat(row.VRALIQIBPTES);
                 
            row.VRPEALIMPFIS = parseFloat(row.VRPEALIMPFIS) < 0 ? row.VRPEALIMPFIS * (-1) : row.VRPEALIMPFIS;
            row.VRPERCREDUCAO = parseFloat(row.VRPERCREDUCAO) < 0 ? row.VRPERCREDUCAO * (-1) : row.VRPERCREDUCAO;
            row.VRALIQIBPT = parseFloat(row.VRALIQIBPT) < 0 ? row.VRALIQIBPT * (-1) : row.VRALIQIBPT;
            row.VRALIQIBPTES = parseFloat(row.VRALIQIBPTES) < 0 ? row.VRALIQIBPTES * (-1) : row.VRALIQIBPTES;
            
            if(!row.CDCSTPISCOF){
                row.VRALIQPIS = 0;
                row.VRALIQCOFINS = 0;
            }
            row.__is_new = true;
            General.callRestEngine('cadEst#/ger01900_cadastroproduto_venda_imposto_export', 'Row', row);
            delete widget.getField('CDFILIAL').validations.required;
            ScreenService.closePopup(widget);
		    ScreenService.closeSwipe();
        }
    };
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('ger01903_impostoproduto_controller', ger01903_impostoproduto_controller);
});
